@@ -34,12 +34,12 @@
 #include "IIIMGdkEventKey.h"
 
 enum IIIMF_MODIFIERS
-  {
-    IIIMF_SHIFT_MODIFIER = 1,
-    IIIMF_CONTROL_MODIFIER = 2,
-    IIIMF_META_MODIFIER = 4,
-    IIIMF_ALT_MODIFIER = 8
-  };
+{
+  IIIMF_SHIFT_MODIFIER = 1,
+  IIIMF_CONTROL_MODIFIER = 2,
+  IIIMF_META_MODIFIER = 4,
+  IIIMF_ALT_MODIFIER = 8
+};
 
 static int
 g2icode (guint kv)
@@ -69,7 +69,7 @@ g2icode (guint kv)
     case GDK_Delete:
       return IIIMF_KEYCODE_DELETE;
     case GDK_Multi_key:
-      return 0;
+      return IIIMF_KEYCODE_COMPOSE;
     case GDK_Codeinput:
       return -IIIMF_KEYCODE_CODE_INPUT;
     case GDK_SingleCandidate:
@@ -244,7 +244,7 @@ g2icode (guint kv)
     case GDK_F10:
       return -IIIMF_KEYCODE_F10;
     case GDK_F11:
-      /*case GDK_L1:*/
+      /*case GDK_L1: */
       return -IIIMF_KEYCODE_F11;
     case GDK_F12:
       /* case GDK_L2: */
@@ -490,7 +490,7 @@ g2icode (guint kv)
       return IIIMF_KEYCODE_DOLLAR;
     case GDK_percent:
       /* note that it is a dummy code which assumes US keyboard.
-	 Use keychar instead. */
+         Use keychar instead. */
       return IIIMF_KEYCODE_5;
     case GDK_ampersand:
       return IIIMF_KEYCODE_AMPERSAND;
@@ -545,7 +545,7 @@ g2icode (guint kv)
       return IIIMF_KEYCODE_GREATER;
     case GDK_question:
       /* note that it is a dummy code which assumes US keyboard.
-	 Use keychar instead. */
+         Use keychar instead. */
       return IIIMF_KEYCODE_SLASH;
     case GDK_at:
       return IIIMF_KEYCODE_AT;
@@ -1002,7 +1002,7 @@ g2icode (guint kv)
     case GDK_Arabic_meem:
     case GDK_Arabic_noon:
     case GDK_Arabic_ha:
-      /*case GDK_Arabic_heh:*/
+      /*case GDK_Arabic_heh: */
     case GDK_Arabic_waw:
     case GDK_Arabic_alefmaksura:
     case GDK_Arabic_yeh:
@@ -1636,7 +1636,7 @@ modifier (guint st)
     {
       mod |= IIIMF_CONTROL_MODIFIER;
     }
-  if (st & GDK_MOD2_MASK)
+  if (st & GDK_MOD3_MASK)
     {
       mod |= IIIMF_META_MODIFIER;
     }
@@ -1649,13 +1649,14 @@ modifier (guint st)
 }
 
 IIIMF_status
-convert_GdkEventKey_to_IIIMCF_keyevent (GdkEventKey *e,
-					IIIMCF_keyevent *pkev)
+convert_GdkEventKey_to_IIIMCF_keyevent (GdkEventKey * e,
+					IIIMCF_keyevent * pkev)
 {
   int c;
 
   c = g2icode (e->keyval);
-  if (c == 0) return IIIMF_STATUS_FAIL;
+  if (c == 0)
+    return IIIMF_STATUS_FAIL;
 
   if (c < 0)
     {
@@ -1674,44 +1675,460 @@ convert_GdkEventKey_to_IIIMCF_keyevent (GdkEventKey *e,
   return IIIMF_STATUS_SUCCESS;
 }
 
-#if 0 /* TODO? */
-IIIMF_status
-convert_IIIMCF_keyevent_to_GdkEventKey (IIIMCF_keyevent *pkev,
-					int pressp,
-					GdkEventKey **pe)
+static guint
+i2gmodifier (int st)
 {
-  int c, s;
-
-  c = i2gcode(pkev->keycode);
-
-  if (!c) return IIIMF_STATUS_FAIL;
-
-  s = 0;
-  if (pkev->modifier & IIIMF_SHIFT_MODIFIER)
+  int mod = 0;
+  if (st & IIIMF_SHIFT_MODIFIER)
     {
-      s |= Qt::ShiftButton;
+      mod |= GDK_SHIFT_MASK;
     }
-  if (pkev->modifier & IIIMF_CONTROL_MODIFIER)
+  if (st & IIIMF_CONTROL_MODIFIER)
     {
-      s |= Qt::ControlButton;
+      mod |= GDK_CONTROL_MASK;
     }
-  if (pkev->modifier & IIIMF_META_MODIFIER)
+  if (st & IIIMF_META_MODIFIER)
     {
-      s |= Qt::MetaButton;
+      mod |= GDK_MOD3_MASK;
     }
-  if (pkev->modifier & IIIMF_ALT_MODIFIER)
+  if (st & IIIMF_ALT_MODIFIER)
     {
-      s |= Qt::AltButton;
+      mod |= GDK_MOD1_MASK;
     }
 
-  if (pressp)
-    *pe = new QKeyEvent(QEvent::KeyPress, c, pkev->keychar, s);
-  else
-    *pe = new QKeyEvent(QEvent::KeyRelease, c, pkev->keychar, s);
+  return mod;
+}
+
+static guint
+i2gcode (int kv, int kc)
+{
+  /* treat special case */
+  switch (kc)
+    {
+    case '?':
+      return GDK_question;
+    case '%':
+      return GDK_percent;
+    default:
+      break;
+    }
+
+  switch (kv)
+    {
+
+      /* case IIIMF_KEYCODE_UNDEFINED */
+    case IIIMF_KEYCODE_CANCEL:
+      return GDK_Cancel;
+    case IIIMF_KEYCODE_BACK_SPACE:
+      return GDK_BackSpace;
+    case IIIMF_KEYCODE_TAB:
+      return GDK_Tab;
+    case IIIMF_KEYCODE_ENTER:
+      return GDK_Return;
+    case IIIMF_KEYCODE_CLEAR:
+      return GDK_Clear;
+    case IIIMF_KEYCODE_SHIFT:
+      return GDK_Shift_L;	/* ... */
+    case IIIMF_KEYCODE_CONTROL:
+      return GDK_Control_L;	/* ... */
+    case IIIMF_KEYCODE_ALT:
+      return GDK_Alt_L;
+    case IIIMF_KEYCODE_PAUSE:
+      return GDK_Pause;
+    case IIIMF_KEYCODE_CAPS_LOCK:
+      return GDK_Caps_Lock;
+      /*
+         case IIIMF_KEYCODE_KANA
+         case IIIMF_KEYCODE_FINAL
+       */
+    case IIIMF_KEYCODE_KANJI:
+      return GDK_Kanji;
+
+    case IIIMF_KEYCODE_ESCAPE:
+      return GDK_Escape;
+    case IIIMF_KEYCODE_CONVERT:
+      return GDK_Henkan_Mode;
+    case IIIMF_KEYCODE_NONCONVERT:
+      return GDK_Muhenkan;
+      /* case IIIMF_KEYCODE_ACCEPT */
+    case IIIMF_KEYCODE_MODECHANGE:
+      return GDK_Mode_switch;
+    case IIIMF_KEYCODE_SPACE:
+      return GDK_space;
+    case IIIMF_KEYCODE_PAGE_UP:
+      return GDK_Page_Up;
+    case IIIMF_KEYCODE_PAGE_DOWN:
+      return GDK_Page_Down;
+    case IIIMF_KEYCODE_END:
+      return GDK_End;
+    case IIIMF_KEYCODE_HOME:
+      return GDK_Home;
+    case IIIMF_KEYCODE_LEFT:
+      return GDK_Left;
+    case IIIMF_KEYCODE_UP:
+      return GDK_Up;
+    case IIIMF_KEYCODE_RIGHT:
+      return GDK_Right;
+    case IIIMF_KEYCODE_DOWN:
+      return GDK_Down;
+    case IIIMF_KEYCODE_COMMA:
+      return GDK_comma;
+    case IIIMF_KEYCODE_MINUS:
+      return GDK_minus;
+    case IIIMF_KEYCODE_PERIOD:
+      return GDK_period;
+    case IIIMF_KEYCODE_SLASH:
+      return GDK_slash;
+    case IIIMF_KEYCODE_0:
+      return GDK_0;
+    case IIIMF_KEYCODE_1:
+      return GDK_1;
+    case IIIMF_KEYCODE_2:
+      return GDK_2;
+    case IIIMF_KEYCODE_3:
+      return GDK_3;
+    case IIIMF_KEYCODE_4:
+      return GDK_4;
+    case IIIMF_KEYCODE_5:
+      return GDK_5;
+    case IIIMF_KEYCODE_6:
+      return GDK_6;
+    case IIIMF_KEYCODE_7:
+      return GDK_7;
+    case IIIMF_KEYCODE_8:
+      return GDK_8;
+    case IIIMF_KEYCODE_9:
+      return GDK_9;
+    case IIIMF_KEYCODE_SEMICOLON:
+      return GDK_semicolon;
+    case IIIMF_KEYCODE_EQUALS:
+      return GDK_equal;
+    case IIIMF_KEYCODE_A:
+      return GDK_a;
+    case IIIMF_KEYCODE_B:
+      return GDK_b;
+    case IIIMF_KEYCODE_C:
+      return GDK_c;
+    case IIIMF_KEYCODE_D:
+      return GDK_d;
+    case IIIMF_KEYCODE_E:
+      return GDK_e;
+    case IIIMF_KEYCODE_F:
+      return GDK_f;
+    case IIIMF_KEYCODE_G:
+      return GDK_g;
+    case IIIMF_KEYCODE_H:
+      return GDK_h;
+    case IIIMF_KEYCODE_I:
+      return GDK_i;
+    case IIIMF_KEYCODE_J:
+      return GDK_j;
+    case IIIMF_KEYCODE_K:
+      return GDK_k;
+    case IIIMF_KEYCODE_L:
+      return GDK_l;
+    case IIIMF_KEYCODE_M:
+      return GDK_m;
+    case IIIMF_KEYCODE_N:
+      return GDK_n;
+    case IIIMF_KEYCODE_O:
+      return GDK_o;
+    case IIIMF_KEYCODE_P:
+      return GDK_p;
+    case IIIMF_KEYCODE_Q:
+      return GDK_q;
+    case IIIMF_KEYCODE_R:
+      return GDK_r;
+    case IIIMF_KEYCODE_S:
+      return GDK_s;
+    case IIIMF_KEYCODE_T:
+      return GDK_t;
+    case IIIMF_KEYCODE_U:
+      return GDK_u;
+    case IIIMF_KEYCODE_V:
+      return GDK_v;
+    case IIIMF_KEYCODE_W:
+      return GDK_w;
+    case IIIMF_KEYCODE_X:
+      return GDK_x;
+    case IIIMF_KEYCODE_Y:
+      return GDK_y;
+    case IIIMF_KEYCODE_Z:
+      return GDK_z;
+    case IIIMF_KEYCODE_OPEN_BRACKET:
+      return GDK_bracketleft;
+    case IIIMF_KEYCODE_BACK_SLASH:
+      return GDK_backslash;
+    case IIIMF_KEYCODE_CLOSE_BRACKET:
+      return GDK_bracketright;
+    case IIIMF_KEYCODE_NUMPAD0:
+      return GDK_KP_0;
+    case IIIMF_KEYCODE_NUMPAD1:
+      return GDK_KP_1;
+    case IIIMF_KEYCODE_NUMPAD2:
+      return GDK_KP_2;
+    case IIIMF_KEYCODE_NUMPAD3:
+      return GDK_KP_3;
+    case IIIMF_KEYCODE_NUMPAD4:
+      return GDK_KP_4;
+    case IIIMF_KEYCODE_NUMPAD5:
+      return GDK_KP_5;
+    case IIIMF_KEYCODE_NUMPAD6:
+      return GDK_KP_6;
+    case IIIMF_KEYCODE_NUMPAD7:
+      return GDK_KP_7;
+    case IIIMF_KEYCODE_NUMPAD8:
+      return GDK_KP_8;
+    case IIIMF_KEYCODE_NUMPAD9:
+      return GDK_KP_9;
+    case IIIMF_KEYCODE_MULTIPLY:
+      return GDK_KP_Multiply;
+    case IIIMF_KEYCODE_ADD:
+      return GDK_KP_Add;
+    case IIIMF_KEYCODE_SEPARATOR:
+      return GDK_KP_Separator;
+    case IIIMF_KEYCODE_SUBTRACT:
+      return GDK_KP_Subtract;
+    case IIIMF_KEYCODE_DECIMAL:
+      return GDK_KP_Decimal;
+    case IIIMF_KEYCODE_DIVIDE:
+      return GDK_KP_Divide;
+    case IIIMF_KEYCODE_F1:
+      return GDK_F1;
+    case IIIMF_KEYCODE_F2:
+      return GDK_F2;
+    case IIIMF_KEYCODE_F3:
+      return GDK_F3;
+    case IIIMF_KEYCODE_F4:
+      return GDK_F4;
+    case IIIMF_KEYCODE_F5:
+      return GDK_F5;
+    case IIIMF_KEYCODE_F6:
+      return GDK_F6;
+    case IIIMF_KEYCODE_F7:
+      return GDK_F7;
+    case IIIMF_KEYCODE_F8:
+      return GDK_F8;
+    case IIIMF_KEYCODE_F9:
+      return GDK_F9;
+    case IIIMF_KEYCODE_F10:
+      return GDK_F10;
+    case IIIMF_KEYCODE_F11:
+      return GDK_F11;
+    case IIIMF_KEYCODE_F12:
+      return GDK_F12;
+    case IIIMF_KEYCODE_DELETE:
+      return GDK_Delete;
+    case IIIMF_KEYCODE_DEAD_GRAVE:
+      return GDK_dead_grave;
+    case IIIMF_KEYCODE_DEAD_ACUTE:
+      return GDK_dead_acute;
+    case IIIMF_KEYCODE_DEAD_CIRCUMFLEX:
+      return GDK_dead_circumflex;
+    case IIIMF_KEYCODE_DEAD_TILDE:
+      return GDK_dead_tilde;
+    case IIIMF_KEYCODE_DEAD_MACRON:
+      return GDK_dead_macron;
+    case IIIMF_KEYCODE_DEAD_BREVE:
+      return GDK_dead_breve;
+    case IIIMF_KEYCODE_DEAD_ABOVEDOT:
+      return GDK_dead_abovedot;
+    case IIIMF_KEYCODE_DEAD_DIAERESIS:
+      return GDK_dead_diaeresis;
+    case IIIMF_KEYCODE_DEAD_ABOVERING:
+      return GDK_dead_abovering;
+    case IIIMF_KEYCODE_DEAD_DOUBLEACUTE:
+      return GDK_dead_doubleacute;
+    case IIIMF_KEYCODE_DEAD_CARON:
+      return GDK_dead_caron;
+    case IIIMF_KEYCODE_DEAD_CEDILLA:
+      return GDK_dead_cedilla;
+    case IIIMF_KEYCODE_DEAD_OGONEK:
+      return GDK_dead_ogonek;
+    case IIIMF_KEYCODE_DEAD_IOTA:
+      return GDK_dead_iota;
+    case IIIMF_KEYCODE_DEAD_VOICED_SOUND:
+      return GDK_dead_voiced_sound;
+    case IIIMF_KEYCODE_DEAD_SEMIVOICED_SOUND:
+      return GDK_dead_semivoiced_sound;
+    case IIIMF_KEYCODE_NUM_LOCK:
+      return GDK_Num_Lock;
+    case IIIMF_KEYCODE_SCROLL_LOCK:
+      return GDK_Scroll_Lock;
+    case IIIMF_KEYCODE_AMPERSAND:
+      return GDK_ampersand;
+    case IIIMF_KEYCODE_ASTERISK:
+      return GDK_asterisk;
+    case IIIMF_KEYCODE_QUOTEDBL:
+      return GDK_quotedbl;
+    case IIIMF_KEYCODE_LESS:
+      return GDK_less;
+    case IIIMF_KEYCODE_PRINTSCREEN:
+      return GDK_Print;
+    case IIIMF_KEYCODE_INSERT:
+      return GDK_Insert;
+    case IIIMF_KEYCODE_HELP:
+      return GDK_Help;
+    case IIIMF_KEYCODE_META:
+      return GDK_Meta_L;
+    case IIIMF_KEYCODE_GREATER:
+      return GDK_greater;
+    case IIIMF_KEYCODE_BRACELEFT:
+      return GDK_braceleft;
+    case IIIMF_KEYCODE_BRACERIGHT:
+      return GDK_braceright;
+    case IIIMF_KEYCODE_BACK_QUOTE:
+      return GDK_grave;
+    case IIIMF_KEYCODE_QUOTE:
+      return GDK_apostrophe;
+    case IIIMF_KEYCODE_KP_UP:
+      return GDK_KP_Up;
+    case IIIMF_KEYCODE_KP_DOWN:
+      return GDK_KP_Down;
+    case IIIMF_KEYCODE_KP_LEFT:
+      return GDK_KP_Left;
+    case IIIMF_KEYCODE_KP_RIGHT:
+      return GDK_KP_Right;
+      /* case IIIMF_KEYCODE_ALPHANUMERIC */
+    case IIIMF_KEYCODE_KATAKANA:
+      return GDK_Katakana;
+    case IIIMF_KEYCODE_HIRAGANA:
+      return GDK_Hiragana;
+    case IIIMF_KEYCODE_FULL_WIDTH:
+      return GDK_Zenkaku;
+    case IIIMF_KEYCODE_HALF_WIDTH:
+      return GDK_Hankaku;
+    case IIIMF_KEYCODE_ROMAN_CHARACTERS:
+      return GDK_Romaji;
+      /* case IIIMF_KEYCODE_ALL_CANDIDATES */
+    case IIIMF_KEYCODE_PREVIOUS_CANDIDATE:
+      return GDK_PreviousCandidate;
+    case IIIMF_KEYCODE_CODE_INPUT:
+      return GDK_Codeinput;
+    case IIIMF_KEYCODE_JAPANESE_KATAKANA:
+      return GDK_Katakana;
+    case IIIMF_KEYCODE_JAPANESE_HIRAGANA:
+      return GDK_Hiragana;
+/*	case IIIMF_KEYCODE_JAPANESE_ROMAN */
+    case IIIMF_KEYCODE_KANA_LOCK:
+      return GDK_Kana_Lock;
+      /* case IIIMF_KEYCODE_INPUT_METHOD_ON_OFF */
+    case IIIMF_KEYCODE_AT:
+      return GDK_at;
+    case IIIMF_KEYCODE_COLON:
+      return GDK_colon;
+    case IIIMF_KEYCODE_CIRCUMFLEX:
+      return GDK_asciicircum;
+    case IIIMF_KEYCODE_DOLLAR:
+      return GDK_dollar;
+    case IIIMF_KEYCODE_EURO_SIGN:
+      return GDK_EuroSign;
+    case IIIMF_KEYCODE_EXCLAMATION_MARK:
+      return GDK_exclam;
+    case IIIMF_KEYCODE_INVERTED_EXCLAMATION_MARK:
+      return GDK_exclamdown;
+    case IIIMF_KEYCODE_LEFT_PARENTHESIS:
+      return GDK_parenleft;
+    case IIIMF_KEYCODE_NUMBER_SIGN:
+      return GDK_numbersign;
+    case IIIMF_KEYCODE_PLUS:
+      return GDK_plus;
+    case IIIMF_KEYCODE_RIGHT_PARENTHESIS:
+      return GDK_parenright;
+    case IIIMF_KEYCODE_UNDERSCORE:
+      return GDK_underscore;
+    case IIIMF_KEYCODE_F13:
+      return GDK_F13;
+    case IIIMF_KEYCODE_F14:
+      return GDK_F14;
+    case IIIMF_KEYCODE_F15:
+      return GDK_F15;
+    case IIIMF_KEYCODE_F16:
+      return GDK_F16;
+    case IIIMF_KEYCODE_F17:
+      return GDK_F17;
+    case IIIMF_KEYCODE_F18:
+      return GDK_F18;
+    case IIIMF_KEYCODE_F19:
+      return GDK_F19;
+    case IIIMF_KEYCODE_F20:
+      return GDK_F20;
+    case IIIMF_KEYCODE_F21:
+      return GDK_F21;
+    case IIIMF_KEYCODE_F22:
+      return GDK_F22;
+    case IIIMF_KEYCODE_F23:
+      return GDK_F23;
+    case IIIMF_KEYCODE_F24:
+      return GDK_F24;
+      /*
+         case IIIMF_KEYCODE_COMPOSE:
+         case IIIMF_KEYCODE_ALT_GRAPH
+         case IIIMF_KEYCODE_STOP
+         case IIIMF_KEYCODE_AGAIN
+         case IIIMF_KEYCODE_PROPS
+       */
+    case IIIMF_KEYCODE_UNDO:
+      return GDK_Undo;
+      /*
+         case IIIMF_KEYCODE_COPY
+         case IIIMF_KEYCODE_PASTE
+       */
+    case IIIMF_KEYCODE_FIND:
+      return GDK_Find;
+      /* case IIIMF_KEYCODE_CUT */
+
+    default:
+      return (guint) (kv & 0xff);
+    }
+  return 0;
+}
+
+IIIMF_status
+convert_IIIMCF_keyevent_to_GdkEventKey (IIIMCF_keyevent * pkev,
+					GdkEventKey * e)
+{
+  guint c;
+  GdkKeymap *keymap;
+  GdkKeymapKey *keys;
+  gint n_keys;
+  guint unicode;
+
+  c = i2gcode (pkev->keycode, pkev->keychar);
+  if (c == 0)
+    return IIIMF_STATUS_FAIL;
+
+  unicode = gdk_unicode_to_keyval (pkev->keychar);
+  if (g_unichar_isupper (unicode))
+    pkev->modifier |= IIIMF_SHIFT_MODIFIER;
+
+  e->keyval = c;
+
+#if GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION >= 2
+  keymap = gdk_keymap_get_for_display (gdk_drawable_get_display (e->window));
+#else
+  keymap = gdk_keymap_get_default ();
+#endif
+
+
+  e->state = i2gmodifier (pkev->modifier);
+
+  if (e->state & GDK_SHIFT_MASK)
+    {
+      e->keyval = gdk_keyval_to_upper (e->keyval);
+    }
+  gdk_keymap_get_entries_for_keyval (keymap, e->keyval, &keys, &n_keys);
+
+  if (n_keys)
+    {
+      e->hardware_keycode = keys[0].keycode;
+      g_free (keys);
+    }
+
+  e->time = pkev->time_stamp;
 
   return IIIMF_STATUS_SUCCESS;
 }
-#endif
 
 /* Local Variables: */
 /* c-file-style: "gnu" */
