@@ -87,6 +87,7 @@ class ProtocolDriver extends IIIMComponent implements IIIMProtocol,
     private HashSet aliveIC;
     private int ccdefID = NOID;
     private int manageRuleID = NOID;
+    private Locale prevLocale = null;
 
     static ProtocolDriver getInstance() {
         if (pd == null) {
@@ -380,7 +381,7 @@ class ProtocolDriver extends IIIMComponent implements IIIMProtocol,
                 
 	    dispatchPreeditEvent(new IIIMPreeditEvent(IIIMPreeditEvent.START));
 	    reply = new Protocol(opCode, replyData);
-	    send(reply);                 
+            send(new Protocol(IM_PREEDIT_START_REPLY, replyData));
 	    break;
 	  case IM_PREEDIT_DRAW:
 	    {
@@ -717,7 +718,12 @@ class ProtocolDriver extends IIIMComponent implements IIIMProtocol,
                 String name = data.readString();
 
 		if (odLoader == null) {
-		    break;
+                  replyData.write2(imServerID);
+                  replyData.write2(inputContextID);
+                  replyData.write4(index);
+                  replyData.writeString(name);
+                  send(new Protocol(IM_AUX_START_REPLY, replyData));
+		  break;
 		}
 
 		try {
@@ -757,8 +763,13 @@ class ProtocolDriver extends IIIMComponent implements IIIMProtocol,
 
 		IIIMAuxListener aux = (IIIMAuxListener)auxMap.get(name);
 		if (aux == null) {
-		    debug(" Unknown aux : " + name);
-		    break;
+		  debug(" Unknown aux : " + name);
+                  replyData.write2(imServerID);
+                  replyData.write2(inputContextID);
+                  replyData.write4(index);
+                  replyData.writeString(name);
+                  send(new Protocol(IM_AUX_DRAW_REPLY, replyData));                
+		  break;
 		}
 		aux.auxDraw(new IIIMAuxEvent(IIIMAuxEvent.DRAW, index,
 					     name, idata, sdata));
@@ -1115,7 +1126,7 @@ class ProtocolDriver extends IIIMComponent implements IIIMProtocol,
 
 	String urlAddr = Manager.getProperty("iiimp.server");
 	if (urlAddr == null) {
-	    throw new IOException("No IIIMP server specified");
+		urlAddr = "iiimp://localhost:9010";
 	}
 	URL tmp = new URL(urlAddr);
         url = tmp;
@@ -1529,18 +1540,28 @@ class ProtocolDriver extends IIIMComponent implements IIIMProtocol,
             if(tmp == null) return;
             
             int icID = imServer.getInputContextID(tmp);
-            if( icID == -1) {
+            Manager manager = Manager.getInstance();
+
+            if(icID == -1) {
                 inputContext = new InputContext(tmp, this);
                 imServer.addInputContextHandler(inputContext);
-		Manager manager = Manager.getInstance();
+                prevLocale = manager.getCurrentLocale();
                 inputContext.create(imServer, manager.getCurrentLocale());
 		inputContext.setPreeditListener(manager.getIIIMPreeditListener());
 		inputContext.setCommittedListener(manager.getIIIMCommittedListener());
+            } else {
+               if (prevLocale.equals(manager.getCurrentLocale())){
+                 debug("prevLocale and CurrentLocale are the same");
+               } else {
+                 prevLocale = manager.getCurrentLocale();
+                 inputContext.create(imServer, manager.getCurrentLocale());
+	         inputContext.setPreeditListener(manager.getIIIMPreeditListener());
+	         inputContext.setCommittedListener(manager.getIIIMCommittedListener());
+               }
+            }
+            if (inputContext != null) {
+              inputContext.activate();
             }
-            else inputContext = imServer.getInputContext(icID);
-            
-            if(inputContext != null)
-                inputContext.activate();
         } catch(IOException e) {
 	    if (Manager.DEBUG) {
 		e.printStackTrace();
@@ -1717,21 +1738,15 @@ class ProtocolDriver extends IIIMComponent implements IIIMProtocol,
 	    return Locale.TRADITIONAL_CHINESE;
         } else if (str.equals("zh_CN")) {
 	    return Locale.SIMPLIFIED_CHINESE;
-        }
+        } 
 
 	// if str is not in the spec...
 	int index = str.indexOf('_');
 	if (index > 0) {
 	    String language = str.substring(0, index);
-	    String tmp = str.substring(index + 1);
-	    index = tmp.indexOf('.');
-	    String country = "";
-	    if (index > 0) {
-		country = tmp.substring(0, index);
-	    }
+	    String country = str.substring(index + 1);
 	    return new Locale(language, country);
 	}
-
 	return new Locale(str, "");
     }
 
@@ -1741,20 +1756,27 @@ class ProtocolDriver extends IIIMComponent implements IIIMProtocol,
 	}
 
         String loc = locale.getLanguage();
+        String region = locale.getCountry();
+        String locale_string = "";
+
 	if (loc.equals("ja")) {
 	    return "ja";
         } else if (loc.equals("ko")) {
 	    return "ko";
         } else if (loc.equals("zh")) {
-            String region = locale.getCountry();
             if (region.equals("TW")) {
 		return "zh_TW";
-            } else {
+            } else if (region.equals("CN")){
 		return "zh_CN";
-            }
-        } 
-
-	return loc;
+            } else
+                return "zh_HK";
+        } else {
+           if (region == null || region.length() == 0)
+             locale_string = loc;
+           else
+             locale_string = loc + "_" + region;
+        }
+	return locale_string;
     }
 }
 
@@ -2017,6 +2039,8 @@ class InputContext {
 
 	try {
 	    client.createIC(im.getID(), locale);
+            if (isConversionMode() == true)
+              setConversionMode(true); 
 	} catch (Exception e) {
 	    debug("InputContext createIC: " + e);
 	}
