@@ -253,7 +253,7 @@ iml_make_preedit_draw_with_caret_inst(
                 break;
             }
         }
-        ri = max(sri, fbri);
+        ri = _MAX(sri, fbri);
     }
 #ifdef	DEBUG
     printf("## PWS ##\n");
@@ -269,7 +269,7 @@ iml_make_preedit_draw_with_caret_inst(
         }
     }
     
-    i = min(si, fbi);
+    i = _MIN(si, fbi);
     
     if (caret == -1 && pwl == cwl && i == cwl) {
         /*
