@@ -7,7 +7,7 @@
 
 /*******************************************************************************
                 Initial ICState
- *******************************************************************************/
+*******************************************************************************/
 
 IIIMP_ICState_REQUESTED*
 IIIMP_ICState::start_request()
@@ -30,8 +30,34 @@ IIIMP_ICState::message_proc(
       case IM_TRIGGER_NOTIFY:
       {
 	  bool flag = (pmes->v.trigger_notify.flag == 0) ? true : false;
+          if (ihk->get_lang_selection())
+              ihk->set_lang_selection(false);
 	  return start_request()->toggle_server_convmode(flag);
       }
+      case IM_HOTKEY_NOTIFY:
+      {
+          int hotkey_id = pmes->v.hotkey_notify.hotkey_id;
+          int ikev = pmes->v.hotkey_notify.index;
+          ihk->set_hotkey_mode(true);
+          return ihk->process_hotkey(this, hotkey_id, ikev);
+      }
+      case IM_LOOKUP_CHOICE_START_REPLY:
+      {
+          int keycode = 0;
+          return ihk->draw_lang_selection_window(this, keycode);
+      }
+      case IM_LOOKUP_CHOICE_DRAW_REPLY:
+      {
+          if (ihk->get_hotkey_mode())
+              return ihk->send_hotkey_reply(this);
+          return true;
+      }
+      case IM_LOOKUP_CHOICE_DONE_REPLY:
+      {
+          if (ihk->get_hotkey_mode())
+              return ihk->send_hotkey_reply(this);
+          return true;
+      }
       case IM_SETICVALUES:
        return start_request()->set_values(pmes);
       case IM_SETICFOCUS:
@@ -42,8 +68,15 @@ IIIMP_ICState::message_proc(
        return start_request()->reset();
       case IM_AUX_SETVALUES:
        return start_request()->aux_setvalues(pmes);
+      case IM_AUX_GETVALUES:
+       return start_request()->aux_getvalues(pmes);
       case IM_FORWARD_EVENT:
-       return start_request()->forward_event(pmes);
+      {
+	  if (ihk->get_lang_selection())
+	      return ihk->process_lang_selection_window(this, pmes);
+	  else
+	      return start_request()->forward_event(pmes);
+      }
       case IM_FORWARD_EVENT_WITH_OPERATIONS:
        return start_request()->forward_event_with_operations(pmes);
       case IM_DESTROYIC:
@@ -67,6 +100,8 @@ IIIMP_ICState::IIIMP_ICState(
     pshared->preedit = false;
     pshared->status = false;
     pshared->lookup_choice = false;
+
+    ihk = new IIIMP_hotkey(this);
 }
 
 IIIMP_ICState*
@@ -103,17 +138,85 @@ IIIMP_ICState::destroy()
     ICState::destroy();
 }
 
+IMSvr *
+IIIMP_ICState::get_imsvr ()
+{
+    IMSvr *imsvr;
+    IMInputContext *pimic;
+
+    pimic = get_iminputcontext ();
+    imsvr = pimic->get_imconnection()->get_imsvr();
+
+    if (imsvr) return imsvr;
+    else return NULL;
+}
+
+LEMgr *
+IIIMP_ICState::get_lemgr ()
+{
+    IMSvr *pimsvr = get_imsvr ();
+    return pimsvr->get_lemgr();
+
+}
+
+IMLangList *
+IIIMP_ICState::get_langlist ()
+{
+    LEMgr *plemgr = get_lemgr ();
+
+    IMLangList *planglist =
+        const_cast<IMLangList *>(plemgr->get_all_langlist());
+    IMLangList::iterator it;
+    for (it = planglist->begin (); it != planglist->end (); it++){
+	// LOG_DEBUG("LangID [%s]", it->get_id ());
+    }
+    return planglist;
+}
+
+bool 
+IIIMP_ICState::create_langimlist ()
+{
+    LEMgr *plemgr = get_lemgr ();
+    IMDescriptorList::iterator it;
+    IMLangList::iterator itt;
+    int index = 1;
+
+    IMDescriptorList *pimdlist = const_cast<IMDescriptorList *>(plemgr->get_all_imdesclist());
+
+    for (it = pimdlist->begin (); it != pimdlist->end (); it++){
+        u16string imname;
+        u16string hrn;
+        imname = it->get_imname();
+        hrn = it->get_hrn();
+        // LOG_DEBUG("IMName [%s]", imname.get_charstr());
+        IMLangList *llist = const_cast<IMLangList *>(it->get_languages());
+        for (itt = llist->begin (); itt != llist->end (); itt++) { 
+	    // LOG_DEBUG("Supported Languages [%s]", itt->get_id ());
+	    langimlist.push_back(LangIM(index, itt->get_id(), u16string(imname), u16string(hrn)));
+	    index++;
+        }
+    }
+    return true;
+}
+
+LangIMList *
+IIIMP_ICState::get_langimlist ()
+{
+    return &langimlist;
+}
+
 IIIMP_ICState::~IIIMP_ICState()
 {
     if (!substatep()) {
 	delete pshared;
 	delete get_imlexec();
+        if (ihk) delete ihk;
     }
 }
 
 /*******************************************************************************
                 ICState_REQUESTED
- *******************************************************************************/
+*******************************************************************************/
 
 bool
 IIIMP_ICState_REQUESTED::message_proc(
@@ -182,7 +285,7 @@ IIIMP_ICState_REQUESTED::wait_aux(
 }
 
 IIIMP_ICState_REQUESTED::IIIMP_ICState_REQUESTED(
-	IIIMP_ICState *pbase
+    IIIMP_ICState *pbase
 ) : IIIMP_ICState(*pbase)
 {
     pprev_state = pbase;
@@ -302,7 +405,7 @@ IIIMP_ICState_REQUESTED::aux_setvalues(
 	    aux_draw.string_values = &imtextvec[0];
 
 	    IMAuxEvent aux_event;
-	    aux_event.type = IM_EventAux;
+	    aux_event.type = IM_EventAuxSet;
 	    aux_event.aux = &aux_draw;
 	    result = get_ichandler()->send_event(get_imlexec(),
 						 (IMInputEvent*) &aux_event);
@@ -325,6 +428,135 @@ IIIMP_ICState_REQUESTED::aux_setvalues(
     return result;
 }
 
+bool
+IIIMP_ICState_REQUESTED::aux_getvalues(
+    IIIMP_message *pmes
+)
+{
+    bool result = false;
+    IIIMP_aux_getvalues_v *pauxval = &pmes->v.aux_getvalues;
+    int i;
+
+    IMAuxGetEvent aux_event;
+    IMAuxDrawCallbackStruct *aux_draw, aux_data_struct;
+    aux_draw = &aux_data_struct;
+
+    vector<char> auxnamev;
+    vector<int> intvalv;
+    vector<IMText> imtextvec;
+
+    u16string auxnamestr = CONV_IIIMP_STR(pauxval->input_method_name);
+    if (auxnamestr.get_charstr()) {
+	auxnamev.reserve(auxnamestr.size() + 1);
+	memcpy(&auxnamev[0], auxnamestr.get_charstr(), auxnamestr.size() + 1);
+	aux_draw->aux_name = &auxnamev[0];
+	aux_draw->aux_index = pauxval->class_index;
+
+	{
+	    // integer value array.
+	    int i;
+	    int num = pauxval->integer_value->count;
+	    IIIMP_card32 *pc32 = pauxval->integer_value->ptr;
+	    intvalv.reserve(num);
+	    aux_draw->count_integer_values = num;
+	    for (i = 0; i < num; i++, pc32++) {
+		intvalv.push_back(*pc32);
+	    }
+	    aux_draw->integer_values = &intvalv[0];
+
+	    // string value array.
+	    int count = 0;
+	    IIIMP_string *pstr;
+
+	    for (pstr = pauxval->string_value; pstr; pstr = pstr->next) {
+		IMText text;
+		memset(&text, 0, sizeof(text));
+		text.encoding = UTF16_CODESET;
+		text.char_length = pstr->len;
+		text.text.utf_chars = pstr->ptr;
+		imtextvec.push_back(text);
+		count++;
+	    }
+	    aux_draw->count_string_values = count;
+	    aux_draw->string_values = &imtextvec[0];
+
+	    aux_event.type = IM_EventAuxGet;
+	    aux_event.fromaux = aux_draw;
+	    result = get_ichandler()->send_event(get_imlexec(),
+						 (IMInputEvent*) &aux_event);
+	}
+    }
+
+    IIIMP_string* pimname = iiimp_string_new(get_iiimptrans()->get_data_s(),
+					     pauxval->input_method_name->len,
+					     pauxval->input_method_name->ptr);
+    if (!pimname) {
+	// TODO: we should throw memory exception.
+	return false;
+    }
+
+    // send aux_getvalues_reply
+    aux_draw = aux_event.toaux;
+
+    IIIMP_string *pstrh = NULL;
+    IIIMP_card32_list *pilist = NULL;
+
+    if (aux_draw->count_integer_values > 0) {
+	// `Int' and `IIIMP_card32' is not always compatible especially
+	// in array form.  Thus, it's inevitable to convert `int' array
+	// manually.
+	IIIMP_card32* pc32h, *pc32;
+	pc32h = pc32 = new IIIMP_card32[aux_draw->count_integer_values];
+	if (!pc32) return -1;
+	int *pi = aux_draw->integer_values;
+	for (i = 0; i < aux_draw->count_integer_values; i++) {
+	    *pc32++ = (unsigned) *pi++;
+	}
+	pilist = iiimp_card32_list_new(get_iiimptrans()->get_data_s(),
+				       aux_draw->count_integer_values,
+				       pc32h);
+	if (!pilist) goto error;
+	delete[] pc32h;
+    }
+
+    IIIMP_string *pstr, *pcur;
+    IMText *pimt;
+    for (i = 0, pimt = aux_draw->string_values;
+	 i < aux_draw->count_string_values;
+	 i++, pimt++) {
+	pcur = convert_IMText_to_iiimp_string(get_iiimptrans()->get_data_s(),
+					      pimt);
+	if (!pcur) goto error;
+	if (!pstrh) {
+	    pstrh = pcur;
+	} else {
+	    pstr->next = pcur;
+	}
+	pstr = pcur;
+    }
+
+    preply = iiimp_aux_getvalues_reply_new(get_iiimptrans()->get_data_s(),
+					   get_im_id(),
+					   get_ic_id(),
+					   pauxval->class_index,
+					   pimname,
+					   pilist,
+					   pstrh);
+
+error:
+    result = dealing() && result;
+
+#if 0
+    // FIXME. The pilist and pstrh must be freed, but freeing them will
+    // cause htt_server to freeze.
+    if (pilist)
+	iiimp_card32_list_delete(get_iiimptrans()->get_data_s(), pilist);
+    if (pstrh)
+	iiimp_string_delete(get_iiimptrans()->get_data_s(), pstrh);
+#endif
+    return result;
+}
+
 bool
 IIIMP_ICState_REQUESTED::forward_event(
     IIIMP_message *pmes
@@ -360,7 +592,7 @@ IIIMP_ICState_REQUESTED::forward_event(
 	  result = get_ichandler()->send_event(get_imlexec(), &ev);
       }
 	   
-       break;
+      break;
       case IIIMP_CONTENTS_STRING:
       case IIIMP_CONTENTS_TEXT:
        // ignore
@@ -890,7 +1122,7 @@ IIIMP_ICState_REQUESTED::aux_done(
 
 /*******************************************************************************
                 ICState_WAITING
- *******************************************************************************/
+*******************************************************************************/
 
 bool
 IIIMP_ICState_WAITING::match_waiting_message(
