@@ -604,6 +604,9 @@ IMAuth::set_fd_from(
     socklen_t size;
 
     size = sizeof(ss);
+
+    auth_type = IMAuth::UNDEFINED;
+
     if (getpeername(fd, pname, &size) < 0) return 0;
 
     if (pname->sa_family == AF_INET) {
@@ -613,6 +616,7 @@ IMAuth::set_fd_from(
 	struct sockaddr_in *pad;
 	struct hostent *phe_rev, *phe;
 
+	auth_type = IMAuth::TCP;
 	pad = ((struct sockaddr_in*) pname);
 	addr = inet_ntoa(pad->sin_addr);
 	if (!addr) return 0;
@@ -650,6 +654,8 @@ IMAuth::set_fd_from(
 	int from_address2[addr6_elements_maxnum];
 	int from_address2_bitlen;
 
+	auth_type = IMAuth::TCP;
+
 	if (getnameinfo(pname, size,
 			nihost, sizeof(nihost),
 			niserv, sizeof(niserv),
@@ -693,6 +699,14 @@ IMAuth::set_fd_from(
 	}
 	freeaddrinfo(pst2);
     }
+#endif
+#if defined(HAVE_UNIX_SOCKET)
+    else if (pname->sa_family == AF_UNIX) {
+        auth_type = IMAuth::UNIX;
+	if (from_hostname) free(from_hostname);
+	from_hostname = strdup("localhost");
+	return 1;
+    }
 #endif
     else {
 	return 0;
@@ -751,23 +765,33 @@ IMAuth::auth_fd(
 )
 {
     int flag;
+
     access_type at = get_access_type(fd);
 
     switch (at) {
       case PERMIT:
-       flag = 1;
+       if (auth_type == IMAuth::UNIX) {
+	   flag = check_password(fd, user, password);
+	   /* TODO */
+	   if (flag == -1) { 
+	       /* this system doesn't support unix domain credential. */
+	       flag = 1;
+	   }
+       } else {
+           flag = 1;
+       }
        break;
       case DENY:
        flag = 0;
        break;
       case CHECKUSER:
-       flag = check_password(user, password);
+       flag = check_password(fd, user, password);
        break;
       case PASSWORD:
        if (!password || !user) {
 	   flag = 0;
        } else {
-	   flag = check_password(user, password);
+	   flag = check_password(fd, user, password);
        }
        break;
       default:
@@ -785,7 +809,7 @@ IMAuth::auth_fd(
 	if (at != IMAuth::UNKNOWN) {
 	    LOG_WARNING("Denied the access from %s@%s.", user, from_hostname);
 	} else {
-	    LOG_WARNING("Denied the access %s(unknown source) was granted", user);
+	    LOG_WARNING("Denied the access from %s(unknown source)", user);
 	}
     }
 
@@ -795,6 +819,7 @@ IMAuth::auth_fd(
 IMAuth::IMAuth()
 {
     ae_num = 0;
+    auth_type = IMAuth::UNDEFINED;
     from_hostname = NULL;
     from_address = NULL;
     command_name = domainname = NULL;
