@@ -7,6 +7,10 @@
 #include "IMUtil.hh"
 #include "IMScheduler_MTPC.hh"
 
+#ifdef HAVE_TLS
+#include "IMTLS.hh"
+#endif
+
 int main(
     int argc,
     char **argv
@@ -31,6 +35,11 @@ int main(
     //object manager
     IMObjectMgr *pimobjmgr = IMObjectMgr::construct();
     if (!pimobjmgr) return IMSvr::IMSVR_SYSTEM_ERROR;
+#ifdef HAVE_TLS
+    // tls manager
+    IMTLS *ptls = IMTLS::construct();
+    if (!ptls) return IMSvr::IMSVR_SYSTEM_ERROR;
+#endif
 
     exitcode = svr.start();
 
@@ -38,6 +47,9 @@ int main(
     IMSignal::cleanup();
     IMLog::cleanup();
     IMDaemon::cleanup();
+#ifdef HAVE_TLS
+    IMTLS::cleanup();
+#endif
 
     return exitcode;
 }
