@@ -187,7 +187,11 @@ main(argc, argv)
    * Setup the signal handlers to monitor htt_server, htt_props
    * abnormal termination
    */
+#ifdef SETPGRP_VOID
   grpid = setpgrp();
+#else
+  grpid = setpgrp(0, 0);
+#endif
 
 #ifdef SunOS
   sigset(SIGTERM, clean_up);
@@ -703,7 +707,13 @@ start_htt_server(htt_server_pid, argv)
      * most of the cases
      */
     sleep(1);
+
+#ifdef SETPGRP_VOID
     setpgrp();
+#else
+    setpgrp(0, 0);
+#endif
+
 #ifdef SunOS
     if (flag == OPENWIN_PATH) {
       putenv(OPENWIN_MOTIF_PRELOAD_ENV);
