@@ -44,6 +44,8 @@ Sun Microsystems, Inc. or its licensors is granted.
 #endif
 
 #include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -735,6 +737,10 @@ set_mod_mask(int modnum, int *mask) {
     }
 }
 
+#ifdef linux
+static void GetKanaFlag(Display *disp);
+#endif
+
 static void
 setup_modifier_map(Display *disp)
 {
@@ -850,6 +856,7 @@ setup_modifier_map(Display *disp)
 }
 
 #ifdef linux
+static void
 GetKanaFlag(Display *disp)
 {
     Atom            ret_type;
@@ -934,6 +941,8 @@ modify_Event(
 		    if (isalpha(keysym + 'a' - 1)) {
 		      keysym += ('a' - 1);
 		    }
+		    if (isalpha(keysym))
+		      keysym = tolower(keysym);	
 		  }
 		}
 		break;
@@ -1004,7 +1013,7 @@ modify_Event(
     }
 }
 
-static first_call = True;
+static int first_call = True;
 
 void
 IIIMCF_keyevent_To_XKeyEvent(
@@ -1014,8 +1023,8 @@ IIIMCF_keyevent_To_XKeyEvent(
     XKeyEvent *kev
 )
 {
-    if (first_call && kev->display != (Display*)0) {
-	setup_modifier_map(kev->display);
+    if (first_call && dpy != (Display*)0) {
+	setup_modifier_map(dpy);
 	first_call = False;
     }
 
@@ -1057,8 +1066,8 @@ XKeyEvent_To_IIIMCF_keyevent(
 
     if (first_call && ev->display != (Display*)0) {
 	setup_modifier_map(ev->display);
+	first_call = False;
     }
-    first_call = False;
 
     state = ev->state;
 
@@ -1092,9 +1101,6 @@ XKeyEvent_To_IIIMCF_keyevent(
         keysym = XK_Henkan_Mode;
     } else
 #endif /* linux */
-	if (IsModifierKey(keysym)) {
-	    return False;
-	}
 
 #ifdef	linux
     if (KanaMode != -1){
@@ -1271,8 +1277,6 @@ XKeysym_To_IIIMCF_keyevent(
     IIIMCF_keyevent *pkev
 )
 {
-    int i;
-
     pkev->keychar = 0;
     pkev->keycode = 0;
     pkev->time_stamp = 0;
@@ -1305,9 +1309,6 @@ XKeysym_To_IIIMCF_keyevent(
         keysym = XK_Henkan_Mode;
     } else
 #endif /* linux */
-	if (IsModifierKey(keysym)) {
-	    return False;
-	}
 
     if (keysym < 256) {
 	if ((state & ControlMask)) {
