@@ -210,7 +210,7 @@ static StateKeyWordRec keyword_tbl[] = {
     {"CODE_INPUT", T_CODE_INPUT_STATE},
     {"REMOTE_IM", T_REMOTE_IM_STATE},
     {"LANGUAGE", T_LANGUAGE},
-    0,
+    {NULL, 0}
 };
 
 typedef enum {
@@ -235,12 +235,12 @@ static AttribKeyWordRec attr_tbl[] = {
     {"FONTSET", A_FONTSET, StringAttr},
     {"POSITION", A_POSITION, LiteralAttr},
     {"PLACEMENT", A_PLACEMENT, LiteralAttr},
-    0,
+    {NULL, 0, 0}
 };
 
 #define MAXSTRLEN 100
 
-static int lastch = NULL;
+static int lastch = 0;
 
 static int
 nextch(fp)
@@ -248,9 +248,9 @@ nextch(fp)
 {
     int c;
 
-    if (lastch != NULL) {
+    if (lastch != 0) {
 	c = lastch;
-	lastch = NULL;
+	lastch = 0;
     } else {
 	c = getc(fp);
 	if (c == '\\') {
@@ -583,7 +583,6 @@ LocalIMState	**state;
     Bool exclam, tilde;
     KeySym rhs_keysym =  (KeySym)NoSymbol;
     char *rhs_string_mb = NULL;
-    int l;
     DefTree	**top;
     char *rhs_target_name = NULL, *ret;
     char *range_name = NULL;
@@ -1027,37 +1026,35 @@ Ximp_XIM xim;
 DefTree *p;
 {
     XimCommon im = (XimCommon)xim;
-    int	return_value;
     LocalIMState	*state;
 
     /* next */
-    if(p->next){
-	if(return_value = set_change_state_info((Ximp_XIM)im, p->next))
-	    return (return_value);
+    if (p->next) {
+      if (set_change_state_info((Ximp_XIM)im, p->next) == -1)
+	return -1;
     }
 
     /* succession */
-    if(p->succession){
-	return set_change_state_info((Ximp_XIM)im, p->succession);
-    }
+    if (p->succession)
+      return set_change_state_info((Ximp_XIM)im, p->succession);
 
     /* self */
-    if(p->target_name == NULL)
-	return 0;
+    if (p->target_name == NULL)
+      return 0;
 
     state = im->local_impart->top_state;
 
-    while(state){
-	if (!strcmp(state->name, p->target_name)){
-	    p->target = state;
-	    break;
-	}
-	state = state->next;
+    while(state) {
+      if (!strcmp(state->name, p->target_name)){
+	p->target = state;
+	break;
+      }
+      state = state->next;
     }
-    if(state != NULL)
-	return 0;
+    if (state != NULL)
+      return 0;
     else
-	return -1;
+      return -1;
 }
 
 int
@@ -1075,10 +1072,9 @@ XimCommon im;
 
     state = im->local_impart->top_state;
     while(state) {
-	if (set_change_state_info((Ximp_XIM)im, state->parser)){
-	    return (-1);
-	}
-	state = state->next;
+      if (set_change_state_info((Ximp_XIM)im, state->parser) == -1)
+	return (-1);
+      state = state->next;
     }
 
     return (max_ev_seq);
