@@ -138,7 +138,6 @@ _Ximp_Get_resource_name(im, res_name, res_class,
 {
   size_t len;
 
- set_res_name:
   if (im->core.res_name == NULL) {
     if ((len = _genutil_strlcpy(res_name, "*ximp.", res_name_len)) >= res_name_len)
       goto set_res_class;
@@ -200,7 +199,7 @@ _Ximp_parse( im, event )
     if( (ss = strstr( event, "<Key>" )) == NULL )
 	return;
     detail = ss + 5;
-    *ss = NULL;
+    *ss = 0;
 
     modifier = event;
     key.modifier = 0;
@@ -210,7 +209,7 @@ _Ximp_parse( im, event )
     do {
 	while( *modifier == ' '  ||  *modifier == '\t' )
 	    modifier++;
-	if( *modifier == NULL )
+	if( *modifier == 0)
 	    break;
 	if( strncmp( modifier, "None", 4 ) == 0 ) {
 	    if( key.modifier_mask != 0  ||  exclamation )
@@ -243,12 +242,12 @@ _Ximp_parse( im, event )
 	}
 	if( mod[ii].name == NULL )
 	    return;
-    } while( *modifier != NULL );
+    } while( *modifier != 0);
 
     while( *detail == ' '  ||  *detail == '\t' )
 	detail++;
     for( ss = detail; isalnum(*ss)  ||  *ss == '_'; ss++ );
-    *ss = NULL;
+    *ss = 0;
     if( (key.keysym = XStringToKeysym( detail )) != NoSymbol ) {
 	if( !(keylist = im->ximp_impart->process_start_keys) ) {
 	    if( (keylist = (Ximp_KeyList *)Xcalloc(1, sizeof(Ximp_KeyList))) == NULL )
@@ -358,15 +357,15 @@ _Ximp_InputServerMode(im)
 	    string = value.addr;
 	    do {
 		ss = line;
-		while( (c = *string) != NULL ) {
+		while( (c = *string) != 0 ) {
 		    string++;
 		    if( c == '\n' )
 			break;
 		    *ss++ = c;
 		}
-		*ss = NULL;
+		*ss = 0;
 		_Ximp_parse( im, line );
-	    } while( *string != NULL );
+	    } while( *string != 0 );
 	    Xfree( line );
 	}
     }
