@@ -441,6 +441,9 @@ XIMArg *values;
   int i;
   XIMStyles *p_style = im->ximp_impart->im_styles;
 
+  if (im->local_impart == NULL)
+    return NULL;
+
   for (p = values; p->name != NULL; p++) {
     if (strcmp(p->name, XNQueryInputStyle) == 0) {
       if ((styles = (XIMStyles *)Xmalloc(sizeof(XIMStyles) +
@@ -463,7 +466,6 @@ XIMArg *values;
 
 static SwitchMode
 SwitchFilter(XicCommon ic, XEvent *ev) {
-  XimCommon im = (XimCommon)ic->core.im;
   DefTree *p = NULL;
   unsigned int mod_state;
   KeySym keysym;
@@ -974,7 +976,10 @@ _Ximp_StatusGetAttributes(ic, vl, return_name)
     XRectangle	*p_rect;
     XIMCallback 	*p_callback;
 
-    for(p = vl; p->name != NULL; p++) {
+  if (((XimCommon)ic->core.im)->local_impart == NULL)
+    return True;
+
+  for(p = vl; p->name != NULL; p++) {
 	if(strcmp(p->name, XNArea)==0) {
 	    if(XIMP_CHK_STSAREAMASK(ic)) {
 		if((p_rect = (XRectangle *)Xmalloc(sizeof(XRectangle))) == NULL) {
@@ -1145,6 +1150,9 @@ char		**return_name;
     XPoint		*p_point;
     XIMCallback 	*p_callback;
 
+    if (((XimCommon)ic->core.im)->local_impart == NULL)
+      return True;
+
     for(p = vl; p->name != NULL; p++) {
 	if(strcmp(p->name, XNArea)==0) {
 	    if(XIMP_CHK_PREAREAMASK(ic)) {
@@ -1353,6 +1361,9 @@ _Ximp_Local_GetICValues(XIC xic, XIMArg *arg) {
     char		*return_name = NULL;
     int		 len;
 
+    if (((XimCommon)ic->core.im)->local_impart == NULL)
+      return NULL;
+
     for(p = arg; p->name != NULL; p++) {
 	if(strcmp(p->name, XNInputStyle) == 0) {
 	    if(ic->ximp_icpart->value_mask & XIMP_INPUT_STYLE) {
@@ -1453,8 +1464,12 @@ XIC xic;
 static void
 _Ximp_Local_SetFocus(XIC xic) {
   XicCommon ic = (XicCommon)xic;
+  XIC current_ic;
 
-  XIC current_ic = ((XimCommon)ic->core.im)->local_impart->current_ic;
+  if (((XimCommon)ic->core.im)->local_impart == NULL)
+    return;
+
+  current_ic = ((XimCommon)ic->core.im)->local_impart->current_ic;
 
   if (current_ic != (XIC)NULL) {
     _Ximp_Local_UnSetFocus((XIC)current_ic);
@@ -1475,6 +1490,9 @@ static void
 _Ximp_Local_UnSetFocus(XIC xic) {
   XicCommon ic = (XicCommon)xic;
 
+  if (((XimCommon)ic->core.im)->local_impart == NULL)
+    return;
+
   ((XimCommon)ic->core.im)->local_impart->current_ic = (XIC)NULL;
   _XUnregisterFilter(ic->core.im->core.display, ic->core.focus_window,
 		     Ximp_Local_KeyFilter, (XPointer)ic);
@@ -1558,7 +1576,6 @@ char			*return_name;
 			XIMP_SET_PREBGPIXMAPMASK(ic, change_mask);
 
 		} else if(strcmp(p->name, XNFontSet)==0) {
-		  XicCommon xic = (XicCommon)ic;
 		  ic->core.preedit_attr.fontset = (XFontSet)p->value;
 		  if(p->value != NULL) {
 		    if(ic->ximp_icpart->preedit_font)
@@ -1572,12 +1589,12 @@ char			*return_name;
 		      return_name = p->name;
 		      return( False );
 		    }
-		    tmp[0] = NULL;
+		    tmp[0] = 0;
 		    for(i = 0; i < list_ret; i++) {
 		      strcat(tmp, name_list[i]);
 		      strcat(tmp, ",");
 		    }
-		    tmp[len + i - 1] = NULL;
+		    tmp[len + i - 1] = 0;
 		    ic->ximp_icpart->preedit_font = tmp;
 		    XIMP_SET_PREFONTMASK(ic, change_mask);
 		  } else {
@@ -1705,7 +1722,6 @@ char			*return_name;
 			XIMP_SET_STSBGPIXMAPMASK(ic, change_mask);
 
 		} else if(strcmp(p->name, XNFontSet)==0) {
-		  XicCommon xic = (XicCommon)ic;
 		  ic->core.status_attr.fontset = (XFontSet)p->value;
 		  if (p->value != NULL) {
 		    if(ic->ximp_icpart->status_font)
@@ -1719,12 +1735,12 @@ char			*return_name;
 		      return_name = p->name;
 		      return( False );
 		    }
-		    tmp[0] = NULL;
+		    tmp[0] = 0;
 		    for(i = 0; i < list_ret; i++) {
 		      strcat(tmp, name_list[i]);
 		      strcat(tmp, ",");
 		    }
-		    tmp[len + i - 1] = NULL;
+		    tmp[len + i - 1] = 0;
 		    ic->ximp_icpart->status_font = tmp;
 		    XIMP_SET_STSFONTMASK(ic, change_mask);
 		  } else {
@@ -1860,6 +1876,9 @@ _Ximp_Local_SetICValues(XIC xic, XIMArg *arg) {
   char		*ret;
   XimpChangeMaskRec	 change_mask;
 
+  if (((XimCommon)ic->core.im)->local_impart == NULL)
+    return NULL;
+
   XIMP_SET_NULLMASK(change_mask);
   if((ret = _Ximp_Local_SetICValueData((Ximp_XIC)ic, arg,
 				       XIMP_SET_IC, &change_mask)))
@@ -1878,6 +1897,9 @@ static char *_Ximp_Local_MbReset(XIC xic) {
   XicCommon ic = (XicCommon)xic;
   XIMComposeIM lim = ((XimCommon)ic->core.im)->local_impart;
 
+  if (ic->local_icpart == NULL)
+    return NULL;
+
   ic->local_icpart->composed = (DefTree *)NULL;
   if (lim->use_binary_table == True)
     ic->local_icpart->context = (DefTree *)BCF_STATE_PARSER(lim->state_addr, (CARD32)(ic->local_icpart->imstate));
