@@ -79,9 +79,9 @@ DrawLookupString2(XicCommon ic,
 		  Display *display, Window win, XFontSet fontset,
 		  GC gc, GC rgc,
 		  int x, int y, char *str, int str_len,
-		  XIMChoiceObject2 *choice, int index) {
-  XIMText *label = &choice->label[index];
-  XIMText *value = &choice->value[index];
+		  XIMChoiceObject2 *choice, int idx) {
+  XIMText *label = &choice->label[idx];
+  XIMText *value = &choice->value[idx];
   int i;
   int j;
   wchar_t *wcstr = 0, *wcstrp;
@@ -107,7 +107,7 @@ DrawLookupString2(XicCommon ic,
 
   wcstrp = wcstr;
 
-  pfeedback = choice->label_feedback[index];
+  pfeedback = choice->label_feedback[idx];
   feedback_length = label->length;
 
   for (i = 0; i < feedback_length; i++, pfeedback++) {
@@ -139,7 +139,7 @@ DrawLookupString2(XicCommon ic,
 
   wcstrp = wcstr;
 
-  pfeedback = choice->value_feedback[index];
+  pfeedback = choice->value_feedback[idx];
   feedback_length = value->length;
   for (i = 0; i < feedback_length; pfeedback++) {
     /* set colors */
@@ -193,7 +193,7 @@ getChoiceIndex(XicCommon ic, XEvent *event) {
   int currentCol = 0;
   int currentRow = 0;
   int w, h;
-  int index;
+  int idx;
 
   if (!lookup) return -1;
 
@@ -222,13 +222,13 @@ getChoiceIndex(XicCommon ic, XEvent *event) {
   }
   currentRow = boundValue(currentRow, 0, curRows - 1);
 
-  index = currentRow * curColumns + currentCol;
+  idx = currentRow * curColumns + currentCol;
 
-  return index;
+  return idx;
 }
 
 static void
-selectCandidate(XicCommon ic, int index) {
+selectCandidate(XicCommon ic, int idx) {
   LookupWin lookup = (LookupWin)(ic->gui_icpart->lookup);
   KeySym keysym;
   XEvent ev;
@@ -236,12 +236,12 @@ selectCandidate(XicCommon ic, int index) {
 
   if (!XIM_IS_IIIMP(ic->core.im)) return;
 
-  if (index >= 0 && index < lookup->length)
-    charValue = lookup->labelArray[index][0];
+  if (idx >= 0 && idx < lookup->length)
+    charValue = lookup->labelArray[idx][0];
   else {
     fprintf(stderr,
 	    "ButtonPress: something is wrong with %d index\n",
-	    index);
+	    idx);
     return;
   }
 
@@ -257,11 +257,9 @@ selectCandidate(XicCommon ic, int index) {
   } else if (charValue >= '0' && charValue <= '9') {
     keysym = XK_0 + charValue - '0';
   } else {
-    /* Index is neither alpabet nor number?, how we map keysym value? */
-    fprintf(stderr,
-	    "ButtonPress: something is wrong with %d charValue\n",
-	    charValue);
-    return;
+      /* Index is neither alpabet nor number?, well let's assgin
+	 index + 1 to keysym value */
+      keysym = XK_0 + idx + 1;
   }
 	  
   ev.xkey.x = 0;
@@ -273,6 +271,7 @@ selectCandidate(XicCommon ic, int index) {
   ev.xkey.window = lookup->window;
   ev.xkey.keycode = XKeysymToKeycode(ev.xany.display,
 				     keysym);
+
   IMForwardEvent(ic, &ev);
   PutBackXKeyEvent(ic);
   return;
@@ -281,12 +280,11 @@ selectCandidate(XicCommon ic, int index) {
 static Bool
 FilterButtonPress(Display *d, Window window,
 		  XEvent *ev, XPointer client_data) {
-  int index;
+  int idx;
   XicCommon ic = (XicCommon)client_data;
-  XimCommon im = (XimCommon)ic->core.im;
 
-  index = getChoiceIndex(ic, ev);
-  if (index == -1) return False;
+  idx = getChoiceIndex(ic, ev);
+  if (idx == -1) return False;
 
   /* wait for button released */
   return True;
@@ -295,20 +293,20 @@ FilterButtonPress(Display *d, Window window,
 static Bool
 FilterButtonRelease(Display *d, Window window,
 		    XEvent *ev, XPointer client_data) {
-  int index;
+  int idx;
   XicCommon ic = (XicCommon)client_data;
   XimCommon im = (XimCommon)ic->core.im;
 
-  index = getChoiceIndex(ic, ev);
-  if (index == -1) return False;
+  idx = getChoiceIndex(ic, ev);
+  if (idx == -1) return False;
 
   if (XIM_IS_COMPOSE(im) &&
       (XIC_COMPOSE(ic, imstate)->type == LookupState ||
        XIC_COMPOSE(ic, imstate)->type == CodeInputState)) {
-    Ximp_Local_Lookup_Button_Filter(ic, index, ev); /* in lookup.c */
+    Ximp_Local_Lookup_Button_Filter(ic, idx, ev); /* in lookup.c */
     return True;
   } else {
-    selectCandidate(ic, index);
+    selectCandidate(ic, idx);
   }
   return True;
 }
@@ -316,9 +314,9 @@ FilterButtonRelease(Display *d, Window window,
 static Bool
 FilterMotionNotify(Display *d, Window w, XEvent *ev, XPointer client_data) {
   XicCommon ic = (XicCommon)client_data;
-  int index = getChoiceIndex(ic, ev);
-  if (index == -1) return False;
-  highlightCandidate(ic, index, True);
+  int idx = getChoiceIndex(ic, ev);
+  if (idx == -1) return False;
+  highlightCandidate(ic, idx, True);
   return True;
 }
 
@@ -604,7 +602,8 @@ SetupLookupWindow(XicCommon ic) {
     lookup->fontset = ic->core.preedit_attr.fontset;
   }
   if (!lookup->fontset) {
-    lookup->fontset = XFactoryCreateDefaultFontSet(display);
+    lookup->fontset = XFactoryCreateDefaultFontSet(display,
+						   XIM_IIIMP(ic->core.im, default_font_name));
     lookup->need_free_fontset = True;
   }
 
@@ -623,8 +622,7 @@ moveLookupWindow(XicCommon ic) {
     y = ic->core.preedit_attr.area.y;
     XFactoryGetLocationOnScreen(ic->core.im->core.display,
 				ic->core.client_window, x, y, &point);
-  }
-  if (XIMP_CHK_PRESPOTLMASK(ic)) {
+  } else if (XIMP_CHK_PRESPOTLMASK(ic)) {
     point.x = -1;
     point.y = -1;
     PreeditCaretPlacement(ic, &point);
@@ -642,6 +640,20 @@ moveLookupWindow(XicCommon ic) {
     } else {
       point.y += 4; /* offset */
     }
+  } else {
+      Window parent;
+      Window root;
+      int x_return,y_return;
+      unsigned int width_return,height_return,border_width_return,depth_return;
+      parent =  XFactoryGetFrameWindow (ic->core.im->core.display,
+		      ic->core.focus_window);
+      XGetGeometry (ic->core.im->core.display,
+		      parent, &root,&x_return,&y_return,
+		      &width_return,&height_return,&border_width_return, &depth_return);
+      XFactoryGetLocationOnScreen(ic->core.im->core.display,
+		      parent, 0, height_return, &point);
+      point.x -= border_width_return;
+      point.y += border_width_return;
   }
 
   if (lookup->fontset) {
@@ -1138,14 +1150,16 @@ drawProc(XicCommon ic, XPointer p) {
 
   resizeLookupWindow(ic);
 
-  XMapRaised(ic->core.im->core.display, lookup->window);
+  if (IsIMLookupEnabled ()) {
+    XMapRaised(ic->core.im->core.display, lookup->window);
 
-  if (True == lookup->mapped) {
-    UpdateLookup(ic);
+    if (True == lookup->mapped) {
+      UpdateLookup(ic);
+    }
+
+    lookup->mapped = True;
   }
 
-  lookup->mapped = True;
-
   return;
 }
 
@@ -1162,9 +1176,11 @@ doneProc(XicCommon ic, XPointer p) {
 
   if (!lookup) return;
 
-  lookup->mapped = False;
-  XUnmapWindow(ic->core.im->core.display,
-	       lookup->window);
+  if (IsIMLookupEnabled () || lookup->mapped) {
+    lookup->mapped = False;
+    XUnmapWindow(ic->core.im->core.display,
+		 lookup->window);
+  }
 
   FreeLookupData(lookup);
   lookup->n_choices = 0;
