@@ -159,7 +159,6 @@ static Bool
 RepaintPreedit(Display *d, Window w, XEvent *ev, XPointer client_data) {
   XicCommon ic = (XicCommon)client_data;
   PreeditWin preedit = NULL;
-  int i;
 
   if (!ic->gui_icpart) return True;
   preedit = (PreeditWin)ic->gui_icpart->preedit;
@@ -757,7 +756,8 @@ SetPreeditFont(XicCommon ic, XPointer call_data) {
     }
     preedit->fontset = ic->core.preedit_attr.fontset;
   } else if (preedit->fontset == NULL) {
-    preedit->fontset = XFactoryCreateDefaultFontSet(ic->core.im->core.display);
+    preedit->fontset = XFactoryCreateDefaultFontSet(ic->core.im->core.display,
+						    XIM_IIIMP(ic->core.im, default_font_name));
     preedit->need_free_fontset = True;
   }
   preedit->fse = XExtentsOfFontSet(preedit->fontset);
@@ -779,7 +779,6 @@ PreeditReplaceString(XicCommon ic, PreeditWin preedit,
   int	i;
   int	j;
   int	m;
-  int	wchar_len;
   int	text_length;
   PreeditChars preedit_chars;
   PreeditChars preedit_chars_backup;
@@ -796,7 +795,6 @@ PreeditReplaceString(XicCommon ic, PreeditWin preedit,
   XIMFeedback *chg_xfb;
   IMFeedback  *cur_fb;
   IMFeedback  *chg_fb;
-  XimCommon im;
 
   preedit_chars = &(preedit->preedit_chars);
 
@@ -1334,7 +1332,6 @@ void
 PreeditDrawText(XicCommon ic, XPointer p) {
   PreeditWin preedit = (PreeditWin)(ic->gui_icpart->preedit);
   PreeditChars preedit_chars;
-  int change_len = 0;
   int chg_first;
   int chg_length;
   XIMDrawTextStruct *preedit_draw = (XIMDrawTextStruct*)p;
