@@ -470,17 +470,14 @@ set_window(XicCommon ic, XPointer call_data) {
   XFontSetExtents *fse;
   int line_spacing;
   int font_logical_height;
-  int x, y, width = 0, height = 0;
+  int x, y, height = 0;
   int area_width;
   int area_height;
-  int wrap_width;
   int first_line_x_offset;
   int len;
   wchar_t *wcstr;
   unsigned short *wcstr_width;
-  int n;
   int line_num;
-  int i;
 
   if (!XIMP_CHK_FOCUSWINMASK(ic)) return;
 
@@ -519,8 +516,6 @@ set_window(XicCommon ic, XPointer call_data) {
   }
   y = (spot_location->y - font_logical_height);
 
-  TRACE_MESSAGE('p', ("set_window: width=%d height=%d\n", width, height));
-
   /* calculate area width */
   if (XIMP_CHK_PREAREAMASK(ic)) {
     area_width = preedit_attr_area->width;
@@ -533,6 +528,9 @@ set_window(XicCommon ic, XPointer call_data) {
     first_line_x_offset = spot_location->x;
   }
 
+  TRACE_MESSAGE('p', ("set_window: width=%d height=%d\n",
+		area_width, area_height));
+
   if (XIMP_CHK_PRELINESPMASK(ic)) {
     line_spacing = ic->core.preedit_attr.line_spacing;
     if (line_spacing > area_height)
