@@ -67,6 +67,13 @@ MapStatusWindow(XicCommon ic, Window win) {
     XUnmapWindow(display, win);
     return False;
   } else {
+
+    if (!IsIMStatusAttached ())
+      {
+	XUnmapWindow(display, win);
+	return True;
+      }
+
     if (cwin_att.map_state == IsUnmapped) {
       if (ic->core.input_style & XIMStatusNothing) {
 	return False;
@@ -270,13 +277,10 @@ SetupStatusWindow(XicCommon ic, Window parent) {
   unsigned int width, height;
   unsigned long fg, bg;
   XWindowAttributes attr;
-  unsigned long val_mask;
-  XGCValues gcval;
   Display *display = ic->core.im->core.display;
   XIMFilterRec filters[4];
   int n;
   unsigned int event_mask;
-  XimCommon im = (XimCommon)ic->core.im;
   extern Bool popup_button_press(Display *d, Window w, XEvent *ev,
 				 XPointer client_data);
 
@@ -375,10 +379,14 @@ SetupStatusWindow(XicCommon ic, Window parent) {
     status->window = win;
   }
 #if 0
-  val_mask = GCForeground | GCBackground;
-  gcval.foreground = fg;
-  gcval.background = bg;
-  status->gc = XCreateGC(display, win, val_mask, &gcval);
+ {
+   unsigned long val_mask;
+   XGCValues gcval;
+   val_mask = GCForeground | GCBackground;
+   gcval.foreground = fg;
+   gcval.background = bg;
+   status->gc = XCreateGC(display, win, val_mask, &gcval);
+ }
 #ifdef USE_STATUS_FEEDBACK
   gcval.foreground = bg;
   gcval.background = fg;
@@ -486,7 +494,8 @@ SetStatusFont(XicCommon ic, XPointer call_data) {
     status->len = 0;
     status->status_string = NULL;
   } else if (status->fontset == NULL) {
-    status->fontset = XFactoryCreateDefaultFontSet(ic->core.im->core.display);
+    status->fontset = XFactoryCreateDefaultFontSet(ic->core.im->core.display,
+						   XIM_IIIMP (ic->core.im, default_font_name));
     status->need_free_fontset = True;
   }
   return;
@@ -504,7 +513,9 @@ StatusDrawText(XicCommon ic, XPointer call_data) {
   XIMText *text = (XIMText*)status_draw->call_data;
   IMFeedbackList *feedback_list = status_draw->feedback_list;
 
-  if (!text) return;		/* do noting */
+  if (text == (XIMText*)NULL || text->length == 0 ||
+      text->string.multi_byte == NULL)
+      return;			/* do nothing */
 
   if (!status) {
     XIC_GUI(ic, change_status)((XIC)ic, STATUS_CREATE, NULL);
