@@ -57,6 +57,10 @@ Sun Microsystems, Inc. or its licensors is granted.
 #include <string.h>
 #include <sys/utsname.h>
 #include <iiimcf.h>
+#include <langinfo.h>
+
+static IIIMCF_language get_input_language (XicCommon ic, char *input_language,
+					   Bool exact_match);
 
 static IIIMF_status
 process_keyevent(
@@ -162,12 +166,12 @@ draw_status(
     IIIMCF_text text;
     XIMText cbtext;
     IMFeedbackList *feedback_list = NULL;
-    XICCallback *cb = &ic->core.preedit_attr.draw_callback;
+    XICCallback *cb = &ic->core.status_attr.draw_callback;
     XIMStatusDrawCallbackStruct call_data;
     XimCommon im = (XimCommon)ic->core.im;
     Bool useUnicode;
 
-    memset(&call_data, 0, sizeof(XIMPreeditDrawCallbackStruct));
+    memset(&call_data, 0, sizeof(XIMStatusDrawCallbackStruct));
     memset(&cbtext, 0, sizeof(XIMText));
 
     useUnicode = (XIM_USE_UNICODE(im) && cb->callback &&
@@ -375,7 +379,6 @@ commit(
 {
     IIIMF_status st;
     IIIMCF_text text;
-    char *ptr;
     XEvent ev;
     Bool useUnicode;
     XimCommon im = (XimCommon)ic->core.im;
@@ -448,6 +451,7 @@ IMProcessIncomingEvent(
 	      case IIIMCF_EVENT_TYPE_TRIGGER_NOTIFY:
 	       TRACE_MESSAGE('i', ("IIIMCF_EVENT_TYPE_TRIGGER_NOTIFY\n"));
 	       process_trigger_notify(ic, ev);
+	       ic_switcher_set_conversion_mode (ic);
 	       break;
 
 	      case IIIMCF_EVENT_TYPE_UI_PREEDIT_START:
@@ -499,6 +503,10 @@ IMProcessIncomingEvent(
 	       IIimpAuxDone(ic, ev);
 	       break;
 
+	      case IIIMCF_EVENT_TYPE_AUX_GETVALUES:
+	       TRACE_MESSAGE('i', ("IIIMCF_EVENT_TYPE_AUX_GETVALUES\n"));
+	       IIimpAuxGetValuesReply(ic, ev);
+	       break;
 	      case IIIMCF_EVENT_TYPE_UI_COMMIT:
 	       TRACE_MESSAGE('i', ("IIIMCF_EVENT_TYPE_UI_COMMIT\n"));
 	       commit(ic);
@@ -590,10 +598,13 @@ IMChangeFocus(
     IIIMF_status st;
     IIIMCF_event event;
 
-    if (flag)
+    if (flag) {
 	st = iiimcf_create_seticfocus_event(&event);
-    else
+	IIimpAuxSetICFocus(ic);
+    } else {
 	st = iiimcf_create_unseticfocus_event(&event);
+	IIimpAuxUnsetICFocus(ic);
+    }
     if (st != IIIMF_STATUS_SUCCESS) return st;
 
     st = iiimcf_forward_event(XIC_IIIMP(ic, context), event);
@@ -872,6 +883,273 @@ IMAuxSetValues(
     return True;
 }
 
+Bool
+IMAuxGetValues(
+    XicCommon ic,
+    IIIMCF_event ev
+)
+{
+    if (!ic) return False;
+
+    if (iiimcf_forward_event(XIC_IIIMP(ic, context), ev)
+	!= IIIMF_STATUS_SUCCESS) return False;
+    IMProcessIncomingEvent(ic);
+
+    return True;
+}
+
+void
+ic_change_conversion_mode (XicCommon ic, char *conv_mode)
+{
+  IIIMCF_event ev;
+  if (conv_mode && !strcmp ((char*)conv_mode, "on"))
+    {
+      IIIMF_status st;
+      st = iiimcf_create_trigger_notify_event (CONV_ON, &ev);
+      if (st != IIIMF_STATUS_SUCCESS)
+	return;
+    }
+  else if (conv_mode && !strcmp ((char*)conv_mode, "off"))
+    {
+      IIIMF_status st;
+      st = iiimcf_create_trigger_notify_event (CONV_OFF, &ev);
+      if (st != IIIMF_STATUS_SUCCESS)
+	return;
+    }
+  if (iiimcf_forward_event(XIC_IIIMP(ic, context), ev) != IIIMF_STATUS_SUCCESS)
+    return;
+  IMProcessIncomingEvent(ic);
+}
+
+void
+ic_initialize_with_input_language (XicCommon ic, char *new_lang)
+{
+  IIIMCF_language iiim_lang = NULL;
+  IIIMF_status st;
+  IIIMCF_attr attr;
+  IIIMCF_event ev;
+  int conversion_mode = CONV_OFF;
+  IIIMCF_context c;
+  XimCommon im = (XimCommon)ic->core.im;
+  char *encoding = nl_langinfo(CODESET);
+
+  if (XIC_IIIMP(ic, context) == NULL)
+    return;
+
+  if (strcmp(encoding, "UTF-8"))
+    /* cannot support input language switch in non-UTF-8 locale*/
+    return;
+
+  if (new_lang)
+    {
+      iiim_lang = get_input_language (ic, new_lang, True);
+      if (iiim_lang != NULL)
+	{
+	  Xfree (ic->current_language);
+	  ic->current_language = strdup (new_lang);
+	}
+      else
+	/* invalid new lang */
+	return;
+    }
+  else
+    iiim_lang = get_input_language (ic, ic->current_language, False);
+
+  /* save conversion mode to restore this after switching */
+  st = iiimcf_get_current_conversion_mode (XIC_IIIMP(ic, context),
+					   &conversion_mode);
+#if 0
+  /* to cancel any remaining preedit text */
+  if (!context_iiim->finalizing)
+    g_signal_emit_by_name (context_iiim, "preedit_changed");
+
+  /* clear candidate if any */
+  if (context_iiim->candidate_start == TRUE)
+    {
+      iiim_destroy_candidate_window (context_iiim);
+      context_iiim->candidate_start = FALSE;
+    }
+#endif
+
+  st = iiimcf_destroy_context(XIC_IIIMP(ic, context));
+  XIC_IIIMP(ic, context) = NULL;
+
+  st = iiimcf_create_attr (&attr);
+  if (st != IIIMF_STATUS_SUCCESS)
+    return;
+  iiimcf_attr_put_ptr_value (attr, IIIMCF_ATTR_INPUT_LANGUAGE,
+			     iiim_lang);
+  st = iiimcf_create_context(XIM_IIIMP(im, handle), attr, &c);
+  XIC_IIIMP(ic, context) = c;
+  iiimcf_destroy_attr (attr);
+  if (st != IIIMF_STATUS_SUCCESS)
+    return;
+
+  st = iiimcf_create_seticfocus_event (&ev);
+
+  if (st != IIIMF_STATUS_SUCCESS)
+    return;
+
+  iiimcf_forward_event(XIC_IIIMP(ic, context), ev);
+  if (conversion_mode)
+    {
+      st = iiimcf_create_trigger_notify_event (CONV_ON, &ev);
+      if (st != IIIMF_STATUS_SUCCESS)
+	return;
+      iiimcf_forward_event(XIC_IIIMP(ic, context), ev);
+    }
+  IMProcessIncomingEvent(ic);
+}
+
+/* get Lang List */
+IIIMCF_language*
+iiim_get_languages (XimCommon im, int *n_lang)
+{
+  IIIMF_status st;
+  IIIMCF_handle h = XIM_IIIMP(im, handle);
+  IIIMCF_language *lang = NULL;
+
+  if (!h)
+    return NULL;
+
+  st = iiimcf_get_supported_languages (h, n_lang, &lang);
+  if (st != IIIMF_STATUS_SUCCESS)
+    return NULL;
+
+  return lang;
+}
+
+static IIIMCF_language
+get_input_language (XicCommon ic, char *input_language, Bool exact_match)
+{
+  int i;
+  IIIMF_status st;
+  char *langid;
+  int n_lang;
+  IIIMCF_language *lang_list;
+  XimCommon im = (XimCommon)ic->core.im;
+
+  lang_list = iiim_get_languages (im, &n_lang);
+  if (lang_list)
+    {
+      int match_len;
+      char *p;
+      for (i = 0; i < n_lang; i++)
+	{
+	  st = iiimcf_get_language_id (lang_list[i],
+				       (const char **)&langid);
+	  if (st != IIIMF_STATUS_SUCCESS)
+	    continue;
+	  if (!strncmp (langid, input_language, strlen (langid)))
+	    {
+	      Xfree (ic->current_language);
+	      ic->current_language = strdup (langid);
+	      return lang_list[i];
+	    }
+	}
+
+      if (exact_match)
+	return NULL;
+      /*
+	cannot find exact match, so try a fuzzy matching
+	for <lang_region> part of the input_language
+      */
+      if (strstr (input_language, ".") == NULL &&
+	  strstr (input_language, "_") == NULL)
+	{
+	  for (i = 0; i < n_lang; i++)
+	    {
+	      st = iiimcf_get_language_id (lang_list[i],
+					   (const char **)&langid);
+	      if (st != IIIMF_STATUS_SUCCESS)
+		continue;
+	      if (!strncmp (langid, input_language, strlen (input_language)))
+		{
+		  Xfree (ic->current_language);
+		  ic->current_language = strdup (langid);
+		  return lang_list[i];
+		}
+	    }
+	}
+      else
+	{
+	  if ((p = strstr (input_language, ".")) != NULL)
+	    {
+	      match_len = p - input_language;
+	      if (match_len > 0)
+		{
+		  for (i = 0; i < n_lang; i++)
+		    {
+		      st = iiimcf_get_language_id (lang_list[i],
+						   (const char **)&langid);
+		      if (st != IIIMF_STATUS_SUCCESS)
+			continue;
+		      if (!strncmp (langid, input_language, match_len))
+			{
+			  Xfree (ic->current_language);
+			  ic->current_language = strdup (langid);
+			  return lang_list[i];
+			}
+		    }
+		}
+	    }
+	  /*
+	    cannot find a match yet, so try another fuzzy matching
+	    for only <lang> part of the input_language
+	  */
+	  if ((p = strstr (input_language, "_")) != NULL)
+	    {
+	      match_len = p - input_language;
+	      if (match_len > 0)
+		{
+		  for (i = 0; i < n_lang; i++)
+		    {
+		      st = iiimcf_get_language_id (lang_list[i],
+						   (const char **)&langid);
+		      if (st != IIIMF_STATUS_SUCCESS)
+			continue;
+		      if (!strncmp (langid, input_language, match_len))
+			{
+			  Xfree (ic->current_language);
+			  ic->current_language = strdup (langid);
+			  return lang_list[i];
+			}
+		    }
+		}
+	    }
+	}
+      /*
+	still cannot find a match. Just take one for "en" as
+	a default.
+      */
+      for (i = 0; i < n_lang; i++)
+	{
+	  st = iiimcf_get_language_id (lang_list[i],
+				       (const char **)&langid);
+	  if (st != IIIMF_STATUS_SUCCESS)
+	    continue;
+	  if (!strncmp (langid, "en", 2))
+	    {
+	      Xfree (ic->current_language);
+	      ic->current_language = strdup (langid);
+	      return lang_list[i];
+	    }
+	}
+      /*
+	I've given up. Use whatever the 1st in the list as the fallback.
+      */
+      st = iiimcf_get_language_id (lang_list[0],
+				   (const char **)&langid);
+      if (st == IIIMF_STATUS_SUCCESS)
+	{
+	  Xfree (ic->current_language);
+	  ic->current_language = strdup (langid);
+	  return lang_list[0];
+	}
+    }
+  return NULL;
+}
+
 /* Local Variables: */
 /* c-file-style: "iiim-project" */
 /* End: */
