@@ -61,7 +61,7 @@
 #include <strings.h>
 #include <ctype.h>
 #include <sys/param.h>
-#ifndef linux
+#ifdef HAVE_SYNCN_H
 #include <synch.h>
 #endif
 
@@ -87,9 +87,44 @@
 #define SPARCV9_DIR_LEN		(8)
 #endif /* __sparcv9 */
 
+typedef struct _aux_entry {
+    int created;
+    aux_dir_t dir;
+    unsigned int if_version;
+} aux_entry_t;
+
+ 
+typedef struct _aux_handle {
+    aux_name_t aux_name;
+    char *file_name;
+    void *handle;
+    aux_entry_t *ae;
+    int ae_num; 
+    struct _aux_handle *next;
+} aux_handle_t;
+
+
+struct _aux_im_data {
+    int im_id;
+    int ic_id;
+    aux_entry_t *ae;
+    void *data;
+    aux_im_data_t *next;
+};
+
+
+typedef struct _aux_ic_info {
+    int im_id;
+    int ic_id;
+    XicCommon ic_common;
+    struct _aux_ic_info *next;
+} aux_ic_info_t;
+
 #define AUX_DIR_SYMBOL		"aux_dir"
 #define AUX_CONF_MAGIC		"# IIIM X auxiliary"
 
+#define AUX_INFO_SYMBOL		"aux_info"
+
 
 #define IS_SPACE(len, ptr)	((0 < (len)) &&				\
 				 (('\t' == *(p)) || (' ' == *(p))))
@@ -120,7 +155,8 @@ aux_so_load(
 static aux_t*
 aux_get(
     XicCommon ic,
-    IIIMCF_event ev
+    IIIMCF_event ev,
+    const IIIMP_card16 *aux_name
 );
 static aux_entry_t*
 aux_entry_get(
@@ -148,6 +184,8 @@ typedef struct {
  */
 static void		service_aux_setvalue(aux_t *,
 					     const unsigned char *, int);
+static void		service_aux_getvalue(aux_t *,
+					     const unsigned char *, int);
 static int		service_im_id(aux_t *);
 static int		service_ic_id(aux_t *);
 static void		service_data_set(aux_t *, int, void *);
@@ -184,6 +222,11 @@ static Window		service_focus_window(aux_t *);
 static int		service_screen_number(aux_t *);
 static int		service_point_screen(aux_t *, XPoint *);
 static int		service_point_caret_screen(aux_t *, XPoint *);
+static Bool		service_get_conversion_mode(aux_t*);
+static void		service_set_conversion_mode(aux_t*, int);
+static aux_t *		service_aux_get_from_id(int im_id, int ic_id,
+						CARD16 *aux_name,
+						int aux_name_length);
 
 static aux_service_t aux_service = {
 	service_aux_setvalue,
@@ -208,7 +251,11 @@ static aux_service_t aux_service = {
 	service_focus_window,
 	service_screen_number,
 	service_point_screen,
-	service_point_caret_screen
+	service_point_caret_screen,
+	service_get_conversion_mode,
+	service_set_conversion_mode,
+	service_aux_getvalue,
+	service_aux_get_from_id
 };
 
 
@@ -226,6 +273,7 @@ static const char *	AuxChangeReasonName[] = {
 
 
 static aux_handle_t *	aux_handle;
+static aux_ic_info_t *	aux_ic_info = NULL;
 
 /*
  * object downloading
@@ -241,8 +289,6 @@ IIimpAuxDownload(
     char file_name[MAXPATHLEN];
     char *dir_name;
     int	dir_name_len;
-    char *p;
-    int	len;
     Bool useUnicode = False; /* default */
     IIIMF_status st;
     const IIIMP_card16 *u16filename;
@@ -381,7 +427,6 @@ create_composed_from_event(
     unsigned char *p;
     AUXComposed *pac;
     IIIMF_status st;
-    IIIMP_card16 *pu16;
     const IIIMP_card16 *aux_name;
     IIIMP_card32 class_idx;
     int num_intvals;
@@ -569,7 +614,7 @@ IIimpAuxStart(
 
     TRACE_MESSAGE('A', ("iiimpAux: start\n"));
 
-    aux = aux_get(ic, ev);
+    aux = aux_get(ic, ev, NULL);
     if (!aux) {
 	TRACE_MESSAGE('A', ("iiimpAux: start: failed to get aux\n"));
 	return;
@@ -602,7 +647,7 @@ IIimpAuxDraw(
 
     TRACE_MESSAGE('A', ("iiimpAux: draw\n"));
 
-    aux = aux_get(ic, ev);
+    aux = aux_get(ic, ev, NULL);
     if (!aux) {
 	TRACE_MESSAGE('A', ("iiimpAux: draw: failed to get aux\n"));
 	return;
@@ -634,7 +679,7 @@ IIimpAuxDone(
 
     TRACE_MESSAGE('A', ("iiimpAux: done\n"));
 
-    aux = aux_get(ic, ev);
+    aux = aux_get(ic, ev, NULL);
     if (!aux) {
 	TRACE_MESSAGE('A', ("iiimpAux: done: failed to get aux\n"));
 	return;
@@ -651,6 +696,38 @@ IIimpAuxDone(
     return;
 }
 
+/*
+ * IM_AUX_GETVALUES
+ */
+void
+IIimpAuxGetValuesReply(
+    XicCommon ic,
+    IIIMCF_event ev
+)
+{
+    aux_t *aux;
+    AUXComposed ac;
+
+    TRACE_MESSAGE('A', ("iiimpAux: getvalues reply\n"));
+
+    aux = aux_get(ic, ev, NULL);
+    if (!aux) {
+	TRACE_MESSAGE('A', ("iiimpAux: getvalues reply: failed to get aux\n"));
+	return;
+    }
+
+    memset(&ac, 0, sizeof(ac));
+    ac.aux = aux;
+    ac.ev = ev;
+
+    if (aux->im->ae->dir.method->getvalues_reply)
+	aux->im->ae->dir.method->getvalues_reply(aux,
+						 (XPointer) &ac,
+						 /* Always set 0! */
+						 0);
+    return;
+}
+
 #if 0
 /*
  * not defined yet
@@ -668,7 +745,7 @@ IIimpAuxSwitched(
 
     TRACE_MESSAGE('A', ("iiimpAux: switched\n"));
 
-    if (NULL == (aux = aux_get(ic, call_data))) {
+    if (NULL == (aux = aux_get(ic, call_data, NULL))) {
 	TRACE_MESSAGE('A', ("iiimpAux: switched: failed to get aux\n"));
 	return;
     }
@@ -682,6 +759,84 @@ IIimpAuxSwitched(
 }
 #endif
 
+void
+IIimpAuxSetICFocus(
+    XicCommon ic
+) 
+{
+    aux_t *aux;
+    aux_im_data_t *aux_im;
+
+    aux = XIC_IIIMP(ic, aux);
+    if (!aux)
+	return;
+
+    for (aux_im = aux->im_list; aux_im; aux_im = aux_im->next) {
+	if (aux_im->ae->if_version >= AUX_IF_VERSION_2 &&
+	    aux_im->ae->dir.method->set_icforcus != NULL) {
+	    aux->im = aux_im;
+	    aux_im->ae->dir.method->set_icforcus(aux);
+	}
+    }
+}
+
+void
+IIimpAuxUnsetICFocus(
+    XicCommon ic
+)
+{
+    aux_t *aux;
+    aux_im_data_t *aux_im;
+
+    aux = XIC_IIIMP(ic, aux);
+    if (!aux)
+	return;
+
+    for (aux_im = aux->im_list; aux_im; aux_im = aux_im->next) {
+	if (aux_im->ae->if_version >= AUX_IF_VERSION_2 &&
+	    aux_im->ae->dir.method->unset_icforcus != NULL) {
+	    aux->im = aux_im;
+	    aux_im->ae->dir.method->unset_icforcus(aux);
+	}
+    }
+}
+
+static void
+delete_aux_ic(
+    XicCommon ic
+)
+{
+    aux_t *aux;
+    aux_im_data_t *aux_im;
+    aux_ic_info_t *aux_ic;
+    aux_ic_info_t *prev_aux_ic;
+
+    aux = XIC_IIIMP(ic, aux);
+    if (!aux)
+	return;
+
+    for (aux_im = aux->im_list; aux_im; aux_im = aux_im->next) {
+	if (aux_im->ae->if_version >= AUX_IF_VERSION_2 &&
+	    aux_im->ae->dir.method->destroy_ic != NULL) {
+	    aux->im = aux_im;
+	    aux_im->ae->dir.method->destroy_ic(aux);
+	}
+    }
+
+    prev_aux_ic = NULL;
+    for (aux_ic = aux_ic_info; aux_ic != NULL; aux_ic = aux_ic->next) {
+	if (aux_ic->ic_common == ic) {
+	    if (prev_aux_ic == NULL)
+		aux_ic_info = aux_ic->next;
+	    else
+		prev_aux_ic->next = aux_ic->next;
+	    Xfree(aux_ic);
+	    break;
+	}
+	prev_aux_ic = aux_ic;
+    }
+}
+
 void
 IIimpDestroryAuxData(
     XicCommon ic
@@ -690,9 +845,12 @@ IIimpDestroryAuxData(
     aux_t *aux;
     aux_im_data_t *pai1, *pai2;
 
+    if (!ic || !ic->iiimp_icpart) return;
     aux = XIC_IIIMP(ic, aux);
     if (!aux) return;
 
+    delete_aux_ic(ic);
+
     for (pai1 = aux->im_list; pai1; ) {
 	pai2 = pai1->next;
 	free(pai1);
@@ -743,7 +901,6 @@ aux_load(
 {
     int		fd;
     char		buf[64];
-    void *		addr;
     int		magic_len;
     int		len;
 
@@ -782,7 +939,6 @@ aux_conf_load(
     struct stat	st_buf;
     void *		addr;
     char *		p;
-    char *		pp;
     char *		aux_name;
     int		aux_name_len;
     char *		aux_so;
@@ -925,6 +1081,27 @@ aux_conf_load(
 }
 
 
+/*
+ *
+ */
+static aux_dir_t *
+get_aux_dir_from_aux_info (void *handle, unsigned int *ifversion)
+{
+    aux_info_t	*aux_info;
+
+    aux_info = (aux_info_t*)dlsym(handle, AUX_INFO_SYMBOL);
+
+    if(aux_info && aux_info->if_version >= AUX_IF_VERSION_2 &&
+	aux_info->register_service != NULL) {
+	aux_info->register_service (AUX_IF_VERSION_2, &aux_service);
+	*ifversion = aux_info->if_version;
+	return aux_info->dir;
+    }
+
+	return NULL;
+}
+
+
 /*
  *
  */
@@ -940,7 +1117,7 @@ aux_so_load(
     aux_handle_t *	ah;
     aux_handle_t *	ah_free;
     int		i;
-    char		path[MAXPATHLEN];
+    unsigned int ifversion;
 
     TRACE_MESSAGE('A', ("iiimpAux: aux_so_load\n"));
 
@@ -965,10 +1142,15 @@ aux_so_load(
 	return NULL;
     }
 
-    aux_dir = (aux_dir_t *)dlsym(handle, AUX_DIR_SYMBOL);
+
+    aux_dir = get_aux_dir_from_aux_info (handle, &ifversion);
     if (NULL == aux_dir) {
-	dlclose(handle);
-	return NULL;
+	aux_dir = (aux_dir_t *)dlsym(handle, AUX_DIR_SYMBOL);
+	if (NULL == aux_dir) {
+	    dlclose(handle);
+	    return NULL;
+	}
+	ifversion = 0;
     }
 
     for (adn = 0, ad = aux_dir; 0 < ad->name.len; ad += 1, adn += 1);
@@ -1005,9 +1187,11 @@ aux_so_load(
 	dlclose(handle);
 	return NULL;
     }
+    memset(ah->ae, 0, adn * (sizeof (aux_entry_t)));
     for (i = 0; i < adn; i++) {
 	(ah->ae + i)->created = 0;
 	memcpy(&((ah->ae + i)->dir), aux_dir + i, sizeof (aux_dir_t));
+	(ah->ae + i)->if_version = ifversion;
     }
 
     ah->next = aux_handle;
@@ -1040,17 +1224,27 @@ create_aux_im_data(
     const IIIMP_card16 *auxname
 )
 {
-    int len;
     aux_entry_t *ae;
     aux_im_data_t *aux_im;
+    XicCommon ic;
+    XimCommon im;
+    IIIMCF_handle handle;
+    IIIMF_status st;
 
     ae = aux_entry_get(auxname);
     if (!ae) return NULL;
     aux_im = (aux_im_data_t *) Xmalloc(sizeof(*aux_im));
     if (!aux_im) return NULL;
     memset(aux_im, 0, sizeof(*aux_im));
+    ic = (XicCommon)aux->ic;
+    im = (XimCommon)ic->core.im;
+    handle = XIM_IIIMP(im, handle);
+    st = iiimcf_get_im_id(handle, &aux_im->im_id);
+    if (st != IIIMF_STATUS_SUCCESS) {
+	XFree(aux_im);
+	return NULL;
+    }
     /* The following members are only for fake. */
-    aux_im->im_id = 0;
     aux_im->ic_id = id;
     /* end */
     aux_im->ae = ae;
@@ -1069,25 +1263,30 @@ create_aux_im_data(
 static aux_t *
 aux_get(
     XicCommon ic,
-    IIIMCF_event ev
+    IIIMCF_event ev,
+    const IIIMP_card16 *aux_name
 )
 {
     IIIMF_status st;
-    const IIIMP_card16 *auxname;
     aux_t *aux;
     aux_im_data_t *aux_im;
+    aux_ic_info_t *aux_ic;
+    XimCommon im;
+    IIIMCF_handle handle;
 
     TRACE_MESSAGE('A', ("iiimpAux: aux_get\n"));
 
-    st = iiimcf_get_aux_event_value(ev, &auxname,
-				    NULL, NULL, NULL, NULL, NULL);
-    if (st != IIIMF_STATUS_SUCCESS) return NULL;
+    if (aux_name == NULL) {
+	st = iiimcf_get_aux_event_value(ev, &aux_name,
+					NULL, NULL, NULL, NULL, NULL);
+	if (st != IIIMF_STATUS_SUCCESS) return NULL;
+    }
 
 #if defined(ENABLE_TRACE)
     if (TRACE_P('s')) {
 	const IIIMP_card16 *pu;
 	TRACE_MESSAGE('s', ("iiimpAux: aux_get: "));
-	for (pu = auxname; *pu; pu++) {
+	for (pu = aux_name; *pu; pu++) {
 	    TRACE_MESSAGE('s', ("%04x ", *pu));
 	}
 	TRACE_MESSAGE('s', ("\n"));
@@ -1106,14 +1305,29 @@ aux_get(
 	aux->ic = ic;
 	aux->service = &aux_service;
 	XIC_IIIMP(ic, aux) = aux;
+
+	aux_ic = Xmalloc(sizeof(*aux_ic));
+	if (!aux_ic) return NULL;
+	memset(aux_ic, 0, sizeof(*aux_ic));
+	im = (XimCommon)ic->core.im;
+	handle = XIM_IIIMP(im, handle);
+	st = iiimcf_get_im_id(handle, &aux_ic->im_id);
+	if (st != IIIMF_STATUS_SUCCESS) {
+	    XFree(aux_ic);
+	    return NULL;
+	}
+	aux_ic->ic_id = XIC_IIIMP(ic, id);
+	aux_ic->ic_common = ic;
+	aux_ic->next = aux_ic_info;
+	aux_ic_info = aux_ic;
     }
 
     if (aux) {
 	/*
-	 * search for aux_im_data corresponding to auxname.
+	 * search for aux_im_data corresponding to aux_name.
 	 */
 	for (aux_im = aux->im_list; aux_im; aux_im = aux_im->next) {
-	    if (!auxname_strncmp(auxname,
+	    if (!auxname_strncmp(aux_name,
 				 aux_im->ae->dir.name.ptr,
 				 aux_im->ae->dir.name.len)) {
 		aux->im = aux_im;
@@ -1131,7 +1345,7 @@ aux_get(
 	XIC_IIIMP(ic, aux) = aux;
     }
 
-    aux_im = create_aux_im_data(aux, XIC_IIIMP(ic, id), auxname);
+    aux_im = create_aux_im_data(aux, XIC_IIIMP(ic, id), aux_name);
 
     if (!aux_im) return NULL;
 
@@ -1284,6 +1498,67 @@ service_aux_setvalue(
 
 }
 
+static void
+service_aux_getvalue(
+    aux_t * aux,
+    const unsigned char * p,
+    int len
+)
+{
+    AUXComposed *pac = (AUXComposed*) p;
+
+    if (pac->ev) {
+	IMAuxGetValues(aux->ic, pac->ev);
+    } else if (pac->pad) {
+	int i;
+	aux_data_t *pad = pac->pad;
+	IIIMF_status st;
+	IIIMCF_event ev;
+	IIIMP_card32 *pintvals;
+	const IIIMP_card16 **pstrs;
+
+#ifdef ENABLE_TRACE
+	static void show_AC(aux_data_t *pad);
+	show_AC(pac->pad);
+#endif
+	if (pad->integer_count > 0) {
+	    pintvals = (IIIMP_card32*) malloc(sizeof(*pintvals) * pad->integer_count);
+	    if (!pintvals) return;
+	    for (i = 0; i < pad->integer_count; i++) {
+		pintvals[i] = pad->integer_list[i];
+	    }
+	} else {
+	    pintvals = NULL;
+	}
+	if (pad->string_count > 0) {
+	    pstrs = (const IIIMP_card16**) malloc(sizeof(*pstrs) * pad->string_count);
+	    if (!pstrs) {
+		if (pintvals) free(pintvals);
+		return;
+	    }
+	    for (i = 0; i < pad->string_count; i++) {
+		pstrs[i] = (const IIIMP_card16*) pad->string_list[i].ptr;
+	    }
+	} else {
+	    pstrs = NULL;
+	}
+
+	st = iiimcf_create_aux_getvalues_event((IIIMP_card16*) pad->aux_name,
+					       pad->aux_index,
+					       pad->integer_count,
+					       pintvals,
+					       pad->string_count,
+					       pstrs,
+					       &ev);
+	if (st == IIIMF_STATUS_SUCCESS) {
+	    IMAuxGetValues(aux->ic, ev);
+	}
+
+	if (pintvals) free(pintvals);
+	if (pstrs) free(pstrs);
+    }
+
+}
 
 static int
 service_im_id(
@@ -1703,6 +1978,54 @@ service_point_caret_screen(
     return XIC_GUI(aux->ic, screen_number);
 }
 
+static Bool
+service_get_conversion_mode(aux_t * aux)
+{
+  XIC xic = (XIC)aux->ic;
+  XIMPreeditState preedit_state;
+
+  XVaNestedList preedit_attr = XVaCreateNestedList(0,
+						   XNPreeditState, &preedit_state,
+						   0);
+  XGetICValues((XIC)xic,
+	       XNPreeditAttributes, preedit_attr,
+	       0);
+  XFree(preedit_attr);
+  return preedit_state;
+}
+
+static void
+service_set_conversion_mode(aux_t * aux, int conversion_mode)
+{
+  XIC xic = (XIC)aux->ic;
+  XIMPreeditState preedit_state;
+
+  preedit_state = (conversion_mode == 1 ? XIMPreeditEnable : XIMPreeditDisable);
+  XVaNestedList preedit_attr = XVaCreateNestedList(0,
+						   XNPreeditState, preedit_state,
+						   0);
+  XSetICValues((XIC)xic,
+	       XNPreeditAttributes, preedit_attr,
+	       0);
+  XFree(preedit_attr);
+}
+
+static aux_t *
+service_aux_get_from_id(int im_id, int ic_id,
+	CARD16 *aux_name, int aux_name_length)
+{
+    aux_ic_info_t *aux_ic;
+
+    for (aux_ic = aux_ic_info; aux_ic != NULL; aux_ic = aux_ic->next) {
+	if (aux_ic->im_id == im_id && aux_ic->ic_id == ic_id)
+	    break;
+    }
+    if (aux_ic == NULL)
+	return NULL;
+
+    return aux_get(aux_ic->ic_common, NULL, aux_name);
+}
+
 /* Local Variables: */
 /* c-file-style: "iiim-project" */
 /* End: */
