@@ -44,7 +44,7 @@ Sun Microsystems, Inc. or its licensors is granted.
 #if defined(sun)
 #define USE_ICONV
 #else
-#define USE_CSCONV
+#define USE_ICONV
 #endif
 #endif
 
@@ -332,6 +332,62 @@ done:
     return ret;
 }
 
+#ifdef USE_ICONV
+static iconv_t i_conv_ToUTF8 = NULL;
+#endif
+
+int
+IMConvertToUTF8(
+    char *from_buf,
+    size_t from_left,
+    char **to_buf,
+    size_t *to_left
+)
+{
+  size_t ret = 0;
+#ifdef USE_CSCONV
+  static csconv_t csconv_cd = NULL;
+#endif
+
+#ifdef USE_ICONV
+  if (skip_native_to_utf8_conv) {
+    strcpy (to_buf, from_buf);
+    *to_left -= from_left;
+    return ret;
+  }
+
+  if (i_conv_ToUTF8 == (iconv_t)-1)
+    /* already failed */
+    return -1;
+
+  if (i_conv_ToUTF8 == NULL) {
+    char *encoding = nl_langinfo(CODESET);
+
+    if (!strcmp(encoding, "UTF-8")) {
+      skip_native_to_utf8_conv = True;
+      strcpy (to_buf, from_buf);
+      *to_left -= from_left;
+      return ret;
+    } else {
+      if ((i_conv_ToUTF8 = iconv_open("UTF-8", encoding))
+	  == (iconv_t) - 1) {
+	/* fallback */
+	encoding = "646";
+	if ((i_conv_ToUTF8 = iconv_open("UTF-8", encoding))
+	    == (iconv_t) - 1)
+	  goto done;
+      }
+    }
+  }
+
+  ret = iconv(i_conv_ToUTF8, (const char**)&from_buf, &from_left,
+	      to_buf, to_left);
+ done:
+#endif /* USE_ICONV */
+
+  return ret;
+}
+
 int
 IIimpConvertToUTF16(
     char *from_buf,
@@ -500,7 +556,7 @@ UTF16ToString(
 	ustr[len] = 0;
 	*pstr = (char*) ustr;
 
-	return len;
+	return IIIMF_STATUS_SUCCESS;
     } else {
 	int dst_len, dst_len_orig, src_len;
 	char *str;
@@ -567,8 +623,8 @@ IIimpConvertToXIMText(
     IIIMF_status st;
     IIIMP_card16 ch;
     const IIIMP_card32 *pids, *pfbs;
-    XIMFeedback *ximfb;
-    IMFeedbackList *pfl, *pfl2;
+    XIMFeedback *ximfb = NULL, *ximfb_top = NULL;
+    IMFeedbackList *pfl = NULL, *pfl2 = NULL;
     IMFeedback *pfb, *pfb2;
     int i, len;
     int j, nfb;
@@ -583,7 +639,7 @@ IIimpConvertToXIMText(
     }
 
     pfl = pfl2 = NULL;
-    ximfb = (XIMFeedback*) Xmalloc(sizeof(*ximfb) * len);
+    ximfb_top = ximfb = (XIMFeedback*) Xmalloc(sizeof(*ximfb) * len);
     memset(ximfb, 0, sizeof(*ximfb) * len);
     if (!ximfb) goto memory_error;
     if (ppfeedbacks) {
@@ -641,7 +697,7 @@ IIimpConvertToXIMText(
 memory_error:
     st = IIIMF_STATUS_MALLOC;
 error:
-    if (ximfb) free(ximfb);
+    if (ximfb_top) free(ximfb_top);
     if (pfl) FreeFeedbackList(pfl, len);
     if (pxt->string.multi_byte) free(pxt->string.multi_byte);
 
