@@ -47,6 +47,8 @@ Sun Microsystems, Inc. or its licensors is granted.
 #include "status.h"
 #include <X11/keysym.h>		/* For IsModifierKey marco */
 #include <X11/Xutil.h>		/* For IsModifierKey marco */
+#include "iiimpUtil.h"
+#include "iiimpSwitcher.h"
 
 #include "status.h"
 
@@ -102,6 +104,11 @@ static int IIIMP_WcLookupString(
     Status *status
 );
 
+extern IIIMCF_language *iiim_get_languages (XimCommon im, int *n_lang);
+
+extern IIIMF_status IIimpConvertToString(IIIMCF_text text,
+					 Bool useUnicode,
+					 char **pstr);
 static XICMethods
 get_iiimp_ic_methods()
 {
@@ -221,8 +228,9 @@ IIIMP_Local_KeyFilter(
 )
 {
     XicCommon ic = (XicCommon)client_data;
-    XimCommon im = (XimCommon)ic->core.im;
-    Bool b;
+
+    if (!IsIMEnabled())
+      return False;
 
     if (ic->ximp_icpart == NULL) {
 	/* most likely associated ic has been already destroyed */
@@ -250,6 +258,8 @@ IIIMP_Local_KeyFilter(
     return IMForwardEvent(ic, ev);
 }
 
+extern char *im_canonicalize_langname(XimCommon im);
+
 /* Public functions */
 XIC
 CreateIC(
@@ -270,6 +280,8 @@ CreateIC(
     
     if (!IIIMP_CreateIC_SWITCH((XIC)ic, arg)) goto Error;
 
+    ic->current_language = im_canonicalize_langname(im);
+
     _XRegisterFilterByType(ic->core.im->core.display, ic->core.focus_window,
 			   KeyPress, KeyRelease,
 			   IIIMP_Local_KeyFilter, (XPointer)ic);
@@ -334,11 +346,15 @@ IIIMP_DestroyIC(
 
     _XUnregisterFilter(ic->core.im->core.display, ic->core.focus_window,
 		       IIIMP_Local_KeyFilter, (XPointer)ic);
+    if (ic->current_language)
+      Xfree (ic->current_language);
     SWITCH_DestroyIC(xic);
     CommonDestroyIC(xic);
     return;
 }
 
+extern Bool GetIMSettings (XicCommon ic);
+
 static void
 IIIMP_SetFocus(
     XIC xic
@@ -356,6 +372,20 @@ IIIMP_SetFocus(
 
     IMChangeFocus(ic, 1);
 
+    ic_switcher_set_focus (ic);
+
+    if (im_switcher_active (im))
+      {
+	IIIMCF_language *lang_list;
+	int n_lang;
+
+	lang_list = iiim_get_languages (im, &n_lang);
+	ic_switcher_set_language_list (ic, lang_list, n_lang);
+	ic_switcher_set_input_language (ic, NULL);
+      }
+
+    GetIMSettings (ic);
+
     if (!XIM_IS_SWITCH(ic->core.im)) {
 	_XRegisterFilterByType(ic->core.im->core.display, ic->core.focus_window,
 			       KeyPress, KeyRelease,
@@ -378,11 +408,6 @@ IIIMP_UnSetFocus(
 
     _XUnregisterFilter(ic->core.im->core.display, ic->core.focus_window,
 		       IIIMP_Local_KeyFilter, (XPointer)ic);
-
-    if (ic->gui_icpart) {
-	XIC_GUI(ic, change_status)((XIC)ic, STATUS_HIDE, NULL);
-    }
-
     return;
 }
 
@@ -726,6 +751,8 @@ SWITCH_DestroyIC(
 
     iiimcf_destroy_context(XIC_IIIMP(ic, context));
 
+    ic_switcher_finalize (ic);
+
     _XUnregisterFilter(ic->core.im->core.display,
 		       ic->core.focus_window,
 		       IIIMP_Local_KeyFilter, (XPointer)ic);
