@@ -612,6 +612,10 @@ GetICValueData(XicCommon ic, XIMArg *values, XimpChangeaMask change_mask) {
       XIMUnicodeCharacterSubset **value;
       XIMUnicodeCharacterSubset *subset;
       XimCommon xim = (XimCommon)ic->core.im;
+
+      if (xim->unicode_char_subsets == NULL)
+	return p->name;
+
       subset = &(xim->unicode_char_subsets->supported_subsets[ic->subset_id]);
       value = (XIMUnicodeCharacterSubset**)p->value;
       *value = subset;
