@@ -4,14 +4,21 @@
 /* $Id: iiimp-data.h,v 1.2 2002/09/07 08:28:52 himi Exp $ */
 
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <sys/types.h>
 
-#if defined(linux)
+#ifdef HAVE_STDINT_H
 #include <stdint.h>
+#endif
+
+#if defined(linux) || defined(__FreeBSD__)
 typedef unsigned char	uchar_t;
 typedef unsigned int	uint_t;
-#endif /* linux */
+#endif /* linux || freebsd */
 
 #include <iiimp-opcode.h>
 
@@ -162,10 +169,11 @@ typedef struct {
 typedef struct iiimp_inputmethod_descriptor {
     size_t		nbyte;
     int			id;		/* Input Method Identifier */
-    IIIMP_string *	rdun;		/* reverse domain unique name */
+    IIIMP_string *	idname;		/* Input Method name */
     IIIMP_string *	hrn;		/* human readable name */
     size_t		language_nbyte;
     IIIMP_language *	language;	/* supported language list */
+    IIIMP_string *	rdun;		/* reverse domain unique name */
     struct iiimp_inputmethod_descriptor *	next;
 } IIIMP_inputmethod_descriptor;
 
@@ -265,6 +273,52 @@ typedef IIIMP_simple	IIIMP_disconnect;
 
 typedef IIIMP_simple	IIIMP_disconnect_reply;
 
+typedef struct {
+    IIIMP_card16                hotkey_id;
+    IIIMP_card8                 state_flag;  /* #1 disabled */
+                                             /* #0 enabled  */
+
+    IIIMP_card8                 action_flag; 
+    /* #2 Event forwarding toggle */
+    /* #1 Event forwarding ON     */
+    /* #0 Event forwarding OFF    */
+} HOTKEYCTRL;
+
+typedef struct {
+    HOTKEYCTRL                  hotkeyctrl;
+    size_t		        nbyte;
+    IIIMP_keyevent_list *       hotkeylist;
+    IIIMP_string *              label;
+} HOTKEY;
+
+typedef struct {
+    size_t		nbyte;
+    int			count;
+    HOTKEY *	        hotkey;
+} HOTKEY_LIST;
+
+typedef struct {
+    IIIMP_card7         opcode;
+    IIIMP_card16        im_id;
+    int                 flag;
+    HOTKEY_LIST *       hotkeys; 
+} IIIMP_register_hotkeys;
+
+typedef struct {
+    IIIMP_card7   opcode;
+    IIIMP_card16  im_id;
+    int           flag;
+    HOTKEYCTRL *  hotkeyctrl;
+} IIIMP_control_hotkeys;
+
+typedef struct {
+    IIIMP_card7    opcode;
+    IIIMP_card16   im_id;
+    IIIMP_card16   ic_id;
+    IIIMP_card16   hotkey_id;
+    IIIMP_card16   index;    /* Index of LISTofKEYEVENT in HOTKEY */
+} IIIMP_hotkey_notify;
+
 typedef struct {
     IIIMP_card7			opcode;
     IIIMP_card16		im_id;
@@ -280,6 +334,7 @@ typedef struct {
 } IIIMP_trigger_notify;
 
 typedef IIIMP_simple	IIIMP_trigger_notify_reply;
+typedef IIIMP_simple	IIIMP_hotkey_notify_reply;
 
 typedef struct {
     IIIMP_card7		opcode;
@@ -475,6 +530,8 @@ typedef IIIMP_aux_simple	IIIMP_aux_done;
 typedef IIIMP_aux_simple	IIIMP_aux_done_reply;
 typedef IIIMP_aux_value		IIIMP_aux_setvalues;
 typedef IIIMP_aux_simple	IIIMP_aux_setvalues_reply;
+typedef IIIMP_aux_value		IIIMP_aux_getvalues;
+typedef IIIMP_aux_value		IIIMP_aux_getvalues_reply;
 
 
 typedef struct {
@@ -501,11 +558,28 @@ typedef struct {
     IIIMP_keyevent_list *	trigger_off;
 } IIIMP_register_trigger_keys_v;
 
+typedef struct {
+    int                 flag;
+    HOTKEY_LIST *       hotkeys; 
+} IIIMP_register_hotkeys_v;
+
 typedef struct {
     int	flag;
 } IIIMP_trigger_notify_v;
 
+typedef struct {
+    int	hotkey_id;
+    int index;
+} IIIMP_hotkey_notify_v;
+
+typedef struct {
+    int	hotkey_id;
+    int	current_state_flag;
+} IIIMP_hotkey_state_notify_v;
+
 typedef IIIMP_simple_v	IIIMP_trigger_notify_reply_v;
+typedef IIIMP_simple_v	IIIMP_hotkey_notify_reply_v;
+typedef IIIMP_simple_v	IIIMP_hotkey_state_notify_reply_v;
 
 typedef struct {
     IIIMP_imattribute *	attr_list;
@@ -648,6 +722,8 @@ typedef IIIMP_aux_simple_v	IIIMP_aux_done_v;
 typedef IIIMP_aux_simple_v	IIIMP_aux_done_reply_v;
 typedef IIIMP_aux_value_v	IIIMP_aux_setvalues_v;
 typedef IIIMP_aux_simple_v	IIIMP_aux_setvalues_reply_v;
+typedef IIIMP_aux_value_v	IIIMP_aux_getvalues_v;
+typedef IIIMP_aux_value_v	IIIMP_aux_getvalues_reply_v;
 
 typedef struct {
     int	number;
@@ -667,6 +743,11 @@ typedef struct {
 	IIIMP_register_trigger_keys_v	register_trigger_keys;
 	IIIMP_trigger_notify_v		trigger_notify;
 	IIIMP_trigger_notify_reply_v	trigger_notify_reply;
+	IIIMP_register_hotkeys_v	register_hotkeys;
+	IIIMP_hotkey_notify_v		hotkey_notify;
+	IIIMP_hotkey_notify_reply_v	hotkey_notify_reply;
+	IIIMP_hotkey_state_notify_v	hotkey_state_notify;
+	IIIMP_hotkey_state_notify_reply_v	hotkey_state_notify_reply;
 	IIIMP_setimvalues_v		setimvalues;
 	IIIMP_setimvalues_reply_v	setimvalues_reply;
 	IIIMP_getimvalues_v		getimvalues;
@@ -678,9 +759,9 @@ typedef struct {
 	IIIMP_commit_string_v		commit_string;
 	
 	IIIMP_forward_event_with_operations_v
-					forward_event_with_operations;
+	forward_event_with_operations;
 	IIIMP_forward_event_with_operations_reply_v
-					forward_event_with_operations_reply;
+	forward_event_with_operations_reply;
 
 	IIIMP_createic_v		createic;
 	IIIMP_createic_reply_v		createic_reply;
@@ -712,14 +793,14 @@ typedef struct {
 
 	IIIMP_lookup_choice_start_v	lookup_choice_start;
 	IIIMP_lookup_choice_start_reply_v
-					lookup_choice_start_reply;
+	lookup_choice_start_reply;
 	IIIMP_lookup_choice_draw_v	lookup_choice_draw;
 	IIIMP_lookup_choice_draw_reply_v	lookup_choice_draw_reply;
 	IIIMP_lookup_choice_done_v	lookup_choice_done;
 	IIIMP_lookup_choice_done_reply_v	lookup_choice_done_reply;
 	IIIMP_lookup_choice_process_v	lookup_choice_process;
 	IIIMP_lookup_choice_process_reply_v
-					lookup_choice_process_reply;
+	lookup_choice_process_reply;
 
 	IIIMP_aux_simple_v		aux_simple;
 	IIIMP_aux_value_v		aux_value;
@@ -732,6 +813,8 @@ typedef struct {
 	IIIMP_aux_done_reply_v		aux_done_reply;
 	IIIMP_aux_setvalues_v		aux_setvalues;
 	IIIMP_aux_setvalues_reply_v	aux_setvalues_reply;
+	IIIMP_aux_getvalues_v		aux_getvalues;
+	IIIMP_aux_getvalues_reply_v	aux_getvalues_reply;
 
 	IIIMP_protocol_version_v	protocol_version;
     }		v;
@@ -754,19 +837,14 @@ typedef enum {
 /* public interface prototype */
 
 extern IIIMP_message *	iiimp_message_unpack(
-				IIIMP_data_s *		data_s,
-				IIIMP_card7		opcode,
-				size_t *		nbyte,
-				const uchar_t **	ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_card7		opcode,
+    size_t *		nbyte,
+    const uchar_t **	ptr);
 extern uchar_t *	iiimp_message_pack(
-				IIIMP_data_s *		data_s,
-				IIIMP_message *		m,
-				size_t *		nbyte);
-
-extern IIIMP_data_s *	iiimp_data_s_new(void);
-extern IIIMP_data_s *	iiimp_data_s_with_protocol_version_new(int protocol_version);
-extern int		iiimp_data_s_get_protocol_version(IIIMP_data_s * data_s);
-extern void		iiimp_data_s_delete(IIIMP_data_s * im);
+    IIIMP_data_s *		data_s,
+    IIIMP_message *		m,
+    size_t *		nbyte);
 
 extern FILE *	iiimp_data_print_fp(IIIMP_data_s * data_s, FILE * fp);
 extern uint_t	iiimp_data_print_flag(IIIMP_data_s * data_s, uint_t log_flag);
@@ -814,6 +892,14 @@ iiimp_register_trigger_keys_pack(
     IIIMP_keyevent_list *	trigger_off,
     size_t *			buf_size);
 
+extern uchar_t *
+iiimp_register_hotkeys_pack(
+    IIIMP_data_s *		data_s,
+    IIIMP_card16		im_id,
+    IIIMP_card16		flag,
+    HOTKEY_LIST *	        hotkeys,
+    size_t *			buf_size);
+
 extern uchar_t *
 iiimp_trigger_notify_pack(
     IIIMP_data_s *	data_s,
@@ -822,6 +908,24 @@ iiimp_trigger_notify_pack(
     IIIMP_card16	flag,
     size_t *		buf_size);
 
+extern uchar_t *
+iiimp_hotkey_notify_pack(
+    IIIMP_data_s *	data_s,
+    IIIMP_card16	im_id,
+    IIIMP_card16	ic_id,
+    IIIMP_card16	hotkey_id,
+    IIIMP_card16	index,
+    size_t *		buf_size);
+
+extern uchar_t *
+iiimp_hotkey_state_notify_pack(
+    IIIMP_data_s *	data_s,
+    IIIMP_card16	im_id,
+    IIIMP_card16	ic_id,
+    IIIMP_card16	hotkey_id,
+    IIIMP_card8		current_state_flag,
+    size_t *		buf_size);
+
 extern uchar_t *
 iiimp_setimvalues_pack(
     IIIMP_data_s *	data_s,
@@ -997,6 +1101,10 @@ iiimp_aux_value_pack(
 		iiimp_simple_pack(data_s, IM_DISCONNECT_REPLY, im_id, 0, buf_size)
 #define iiimp_trigger_notify_reply_pack(data_s, im_id, ic_id, buf_size) \
 		iiimp_simple_pack(data_s, IM_TRIGGER_NOTIFY_REPLY, im_id, ic_id, buf_size)
+#define iiimp_hotkey_notify_reply_pack(data_s, im_id, ic_id, buf_size) \
+		iiimp_simple_pack(data_s, IM_HOTKEY_NOTIFY_REPLY, im_id, ic_id, buf_size)
+#define iiimp_hotkey_state_notify_reply_pack(data_s, im_id, ic_id, buf_size) \
+		iiimp_simple_pack(data_s, IM_HOTKEY_STATE_NOTIFY_REPLY, im_id, ic_id, buf_size)
 #define iiimp_setimvalues_reply_pack(data_s, im_id, buf_size) \
 		iiimp_simple_pack(data_s, IM_SETIMVALUES_REPLY, im_id, 0, buf_size)
 #define iiimp_forward_event_reply_pack(data_s, im_id, ic_id, buf_size) \
@@ -1065,6 +1173,10 @@ iiimp_aux_value_pack(
 		iiimp_aux_value_pack(data_s, IM_AUX_SETVALUES, im_id, ic_id, buf_size)
 #define iiimp_aux_setvalues_reply_pack(data_s, im_id, ic_id, buf_size) \
 		iiimp_aux_simple_pack(data_s, IM_AUX_SETVALUES_REPLY, im_id, ic_id, buf_size)
+#define iiimp_aux_getvalues_pack(data_s, im_id, ic_id, buf_size) \
+		iiimp_aux_value_pack(data_s, IM_AUX_GETVALUES, im_id, ic_id, buf_size)
+#define iiimp_aux_getvalues_reply_pack(data_s, im_id, ic_id, buf_size) \
+		iiimp_aux_value_pack(data_s, IM_AUX_GETVALUES_REPLY, im_id, ic_id, buf_size)
 
 
 extern IIIMP_card16_list *
@@ -1174,9 +1286,10 @@ extern IIIMP_inputmethod_descriptor *
 iiimp_inputmethod_descriptor_new(
     IIIMP_data_s *	data_s,
     IIIMP_card16	id,
-    IIIMP_string *	rdun,
+    IIIMP_string *	idname,
     IIIMP_string *	hrn,
-    IIIMP_language *	language);
+    IIIMP_language *	language,
+    IIIMP_string *	rdun);
 
 extern IIIMP_binaryfile_object *
 iiimp_binaryfile_object_new(
@@ -1279,6 +1392,12 @@ iiimp_keyevent_list_new(
     int			count,
     IIIMP_keyevent *	k);
 
+extern HOTKEY_LIST *
+iiimp_hotkey_list_new(
+    IIIMP_data_s *	data_s,
+    int			count,
+    HOTKEY *	k);
+
 extern IIIMP_contents *
 iiimp_contents_string_new(
     IIIMP_data_s *	data_s,
@@ -1294,7 +1413,6 @@ iiimp_contents_keyevent_list_new(
     IIIMP_data_s *		data_s,
     IIIMP_keyevent_list *	value);
 
-
 /* public interface prototype - im_message_new */
 
 extern IIIMP_message *
@@ -1342,10 +1460,41 @@ iiimp_trigger_notify_new(
     IIIMP_card16	ic_id,
     IIIMP_card16	flag);
 
+extern IIIMP_message *
+iiimp_register_hotkeys_new(
+    IIIMP_data_s *		data_s,
+    IIIMP_card16		im_id,
+    IIIMP_card16		flag,
+    HOTKEY_LIST *	        hotkeys);
+
+extern IIIMP_message *
+iiimp_hotkey_notify_new(
+    IIIMP_data_s *	data_s,
+    IIIMP_card16	im_id,
+    IIIMP_card16	ic_id,
+    IIIMP_card16	hotkey_id,
+    IIIMP_card16	index);
+
+extern IIIMP_message *
+iiimp_hotkey_state_notify_new(
+    IIIMP_data_s *	data_s,
+    IIIMP_card16	im_id,
+    IIIMP_card16	ic_id,
+    IIIMP_card16	hotkey_id,
+    IIIMP_card8		current_state_flag);
+
 #define iiimp_trigger_notify_reply_new(data_s, im_id, ic_id)	\
 		iiimp_simple_new((data_s), IM_TRIGGER_NOTIFY_REPLY, \
 				 (im_id), (ic_id))
 
+#define iiimp_hotkey_notify_reply_new(data_s, im_id, ic_id)	\
+		iiimp_simple_new((data_s), IM_HOTKEY_NOTIFY_REPLY, \
+				 (im_id), (ic_id))
+
+#define iiimp_hotkey_state_notify_reply_new(data_s, im_id, ic_id)	\
+		iiimp_simple_new((data_s), IM_HOTKEY_STATE_NOTIFY_REPLY, \
+				 (im_id), (ic_id))
+
 extern IIIMP_message *
 iiimp_setimvalues_new(
     IIIMP_data_s *	data_s,
@@ -1643,6 +1792,17 @@ iiimp_aux_value_new(
 				     (im_id), (ic_id), \
 				     (class_index), (input_method_name))
 
+#define iiimp_aux_getvalues_new(data_s, im_id, ic_id, class_index, input_method_name, int_val, str_val)	\
+		iiimp_aux_value_new((data_s), IM_AUX_GETVALUES, \
+				     (im_id), (ic_id), \
+				     (class_index), (input_method_name), \
+				     (int_val), (str_val))
+
+#define iiimp_aux_getvalues_reply_new(data_s, im_id, ic_id, class_index, input_method_name, int_val, str_val)	\
+		iiimp_aux_value_new((data_s), IM_AUX_GETVALUES_REPLY, \
+				     (im_id), (ic_id), \
+				     (class_index), (input_method_name), \
+				     (int_val), (str_val))
 extern IIIMP_status
 iiimp_card16_list_create(
     IIIMP_data_s *		data_s,
@@ -1766,9 +1926,10 @@ iiimp_inputmethod_descriptor_create(
     IIIMP_data_s *			data_s,
     IIIMP_inputmethod_descriptor **	ptr_ret,
     IIIMP_card16			id,
-    IIIMP_string *			rdun,
+    IIIMP_string *			idname,
     IIIMP_string *			hrn,
-    IIIMP_language *			language);
+    IIIMP_language *			language,
+    IIIMP_string *			rdun);
 
 extern IIIMP_status
 iiimp_binaryfile_object_create(
