@@ -45,6 +45,9 @@ Sun Microsystems, Inc. or its licensors is granted.
 #endif
 
 #include <stdio.h>
+#include <unistd.h>
+#include <pwd.h>
+#include <sys/types.h>
 #include <canna/jrkanji.h>
 
 #ifndef _WCHAR_T
@@ -53,7 +56,7 @@ Sun Microsystems, Inc. or its licensors is granted.
 
 #include "SunIM.h"
 
-static char canna_le_init_filename[] = "../locale/ja/CannaLE/canna.init";
+static char canna_le_init_filename[] = ".canna";
 
 #define CANNA_COMMIT_STRING_BUFSIZE (8192 * 2)
 #define CANNA_CODESET_LOCALE "ja_JP.eucJP"
@@ -112,6 +115,10 @@ void    if_canna_SetSCFocus();
 void    if_canna_UnsetSCFocus();
 void    if_canna_SendEvent();
 
+void		canna_status_draw_off(iml_session_t *s);
+static void	canna_change_mode(iml_session_t *s, int id);
+static Bool	canna_drop_privilege(const char *username);
+
 /* IF Method */
 if_methods_t canna_methods = {
     if_canna_OpenIF,
@@ -215,11 +222,10 @@ setup_csconv()
   Desktop data:
  */
 
-#if 0
 typedef struct
 {
+    int drop_priv;
 } CannaLEDesktop;
-#endif
 
 /*
   Session data:
@@ -231,9 +237,7 @@ typedef struct
 {
     int context_id;
     jrKanjiStatusWithValue ksv;
-    int status_start;
-    int preedit_start;
-    int luc_start;
+    int conversion_start;
 } CannaLESession;
 
 CannaLESession*
@@ -556,57 +560,71 @@ send_commit(iml_session_t *s, IMText *p, int executep)
 */
 
 static char*
-canna_init_filename(char *if_path)
+canna_init_filename(char *user)
 {
     char *buf;
     int ipsize;
-    FILE *f;
+    struct passwd *pw;
 
-    ipsize = strlen(if_path);
+    if (!user)
+	return NULL;
+    setpwent();
+    if ((pw = getpwnam(user)) == NULL) {
+	endpwent();
+	return NULL;
+    }
+    ipsize = strlen(pw->pw_dir);
     buf = (char*) malloc((ipsize + 2) * sizeof(char)
 			 + sizeof(canna_le_init_filename));
     if (ipsize < 1) return NULL;
-    strcpy(buf, if_path);
+    strcpy(buf, pw->pw_dir);
     buf[ipsize] = '/';
     buf[ipsize + 1] = '\0';
     strcat(buf, canna_le_init_filename);
 
+    endpwent();
+
     /* check whether the file is readable. */
-    f = fopen(buf, "r");
-    if (!f) {
+    if (access(buf, R_OK) != 0) {
 	free(buf);
 	return NULL;
     }
-    fclose(f);
 
     return buf;
 }
 
 Bool
-canna_init(char *if_path)
+canna_init(iml_session_t *s, char *user)
 {
     char **warning = NULL;
-    char *init_filename;        
+    char *init_filename;
 
-    init_filename = canna_init_filename(if_path);
+    init_filename = canna_init_filename(user);
     if (init_filename) {
-	jrKanjiControl(0, KC_SETINITFILENAME, init_filename);
+	jrKanjiControl(canna_session_context(s), KC_SETINITFILENAME, init_filename);
 	free(init_filename);
     }
-    jrKanjiControl(0, KC_INITIALIZE, (char *) &warning);
+    jrKanjiControl(canna_session_context(s), KC_INITIALIZE, (char *) &warning);
 
     if (warning) {
 	char          **p;
 
 	for (p = warning; *p; p++)
-	    fprintf(stderr, "htt: canna.so: %s\n", *p);
+	    fprintf(stderr, "CannaLE: %s\n", *p);
 
 	return False;
     }
 
-    jrKanjiControl(0, KC_SETAPPNAME, (char *) "CannaLE");
+    jrKanjiControl(canna_session_context(s), KC_SETAPPNAME, (char *) "CannaLE");
 
-    canna_context_id_counter = 1;
+    /* set user info */
+    if (user) {
+	jrUserInfoStruct info;
+
+	memset(&info, 0, sizeof (info));
+	info.uname = user;
+	jrKanjiControl(canna_session_context(s), KC_SETUSERINFO, (char *)&info);
+    }
 
     return True;
 }
@@ -638,28 +656,31 @@ canna_status_draw(iml_session_t *s)
     iml_inst *rrv = NULL;
     IMText *p;
     CannaLESession *pcls = canna_session_data(s);
+    unsigned char *str;
+    int len;
 
     jrKanjiStatusWithValue *pksv;
     pksv = canna_session_status(s);
 
-    /* When KanjiStatus is uninited, return immediately. */
-    if (!pksv->ks->mode) return;
+    if (pcls->conversion_start == False) {
+	canna_status_draw_off(s);
+	return;
+    }
+    len = jrKanjiControl(canna_session_context(s), KC_QUERYMAXMODESTR, 0);
+    str = (unsigned char *) malloc(sizeof (unsigned char) * (len + 1));
+    jrKanjiControl(canna_session_context(s), KC_QUERYMODE, str);
 
     {
 	/* Create IMText with feedback. */
-	int len;
-	unsigned char *str;
 	int ft1;
-	str = pksv->ks->mode;
-	len = strlen(str);
 	ft1 = Canna_Feedback_Normal;
 	p = canna_string_to_IMText(s, 1, &len, &str, &ft1);
     }
+    free(str);
 
-    if (pcls->status_start == False) {
+    if (!IS_REGION_ACTIVE(s, STATUS)) {
         lp = s->If->m->iml_make_status_start_inst(s);
         s->If->m->iml_link_inst_tail(&rrv, lp);
-        pcls->status_start = True;
     }
     lp = s->If->m->iml_make_status_draw_inst(s, p);
     s->If->m->iml_link_inst_tail(&rrv, lp);
@@ -679,10 +700,9 @@ canna_preedit_draw(iml_session_t *s)
     /* When KanjiStatus is uninited, return immediately. */
     if (!pks->echoStr) return;
 
-    if (pcls->preedit_start == False) {
+    if (!IS_REGION_ACTIVE(s, PREEDIT)) {
         lp = s->If->m->iml_make_preedit_start_inst(s);
         s->If->m->iml_link_inst_tail(&rrv, lp);
-        pcls->preedit_start = True;
     }
 
     {
@@ -706,15 +726,35 @@ canna_preedit_draw(iml_session_t *s)
     s->If->m->iml_execute(s, &rrv);
 }
 
+void
+canna_status_draw_off(iml_session_t *s)
+{
+    CannaLESession *pcls = canna_session_data(s);
+    IMText *p;
+    iml_inst *lp;
+    iml_inst *rrv = NULL;
+    unsigned char *str = "";
+    int len = strlen(str);
+    int ft1 = Canna_Feedback_Normal;
+
+    p = canna_string_to_IMText(s, 1, &len, &str, &ft1);
+    if (!IS_REGION_ACTIVE(s, STATUS)) {
+	lp = s->If->m->iml_make_status_start_inst(s);
+	s->If->m->iml_link_inst_tail(&rrv, lp);
+    }
+    lp = s->If->m->iml_make_status_draw_inst(s, p);
+    s->If->m->iml_link_inst_tail(&rrv, lp);
+    s->If->m->iml_execute(s, &lp);
+}
+
 void
 canna_preedit_done(iml_session_t *s)
 {
     CannaLESession *pcls = canna_session_data(s);
-    if (pcls->preedit_start == True) {
+    if (IS_REGION_ACTIVE(s, PREEDIT)) {
 	iml_inst *lp;
         lp = s->If->m->iml_make_preedit_done_inst(s);
 	s->If->m->iml_execute(s, &lp);
-        pcls->preedit_start = False;
     }
 }
 
@@ -789,7 +829,7 @@ canna_start_lookup_choice(iml_session_t *s,
 			  iml_inst **prrv, int num)
 {
     CannaLESession *pcls = canna_session_data(s);
-    if (pcls->luc_start == False) {
+    if (!IS_REGION_ACTIVE(s, LOOKUP)) {
 	iml_inst *lp;
 	IMLookupStartCallbackStruct *start;
         start = ((IMLookupStartCallbackStruct *)
@@ -807,7 +847,6 @@ canna_start_lookup_choice(iml_session_t *s,
 
         lp = s->If->m->iml_make_lookup_start_inst(s, start);
 	s->If->m->iml_link_inst_tail(prrv, lp);
-        pcls->luc_start = True;
     }
 }
 
@@ -891,41 +930,42 @@ void
 canna_lookup_choice_done(iml_session_t *s)
 {
     CannaLESession *pcls = canna_session_data(s);
-    if (pcls->luc_start == True) {
+    if (IS_REGION_ACTIVE(s, LOOKUP)) {
 	iml_inst *lp;
         lp = s->If->m->iml_make_lookup_done_inst(s);
 	s->If->m->iml_execute(s, &lp);
-        pcls->luc_start = False;
     }
 }
 
 void
 canna_make_conversion_off(iml_session_t *s)
 {
-    IMText *p;
-    jrKanjiStatusWithValue *pksv;
-    pksv = canna_session_status(s);
+    CannaLESession *pcls = canna_session_data(s);
 
-    pksv->val = CANNA_MODE_AlphaMode;
-    jrKanjiControl(canna_session_context(s),
-		   KC_CHANGEMODE, (char*) pksv);
-    p = canna_kakutei(s);
-    send_commit(s, p, 1);
-    canna_status_draw(s);
-    canna_lookup_choice_done(s);
-    canna_preedit_done(s);
+    if (pcls->conversion_start == True) {
+	pcls->conversion_start = False;
+	canna_change_mode(s, CANNA_MODE_AlphaMode);
+	canna_status_draw(s);
+	canna_status_draw_off(s);
+	canna_lookup_choice_done(s);
+	canna_preedit_done(s);
+    }
 }
 
 void
 canna_make_conversion_on(iml_session_t  *s)
 {
-    jrKanjiStatusWithValue *pksv;
-    pksv = canna_session_status(s);
+    CannaLESession *pcls = canna_session_data(s);
 
-    pksv->val = CANNA_MODE_HenkanMode;
-    jrKanjiControl(canna_session_context(s),
-		   KC_CHANGEMODE, (char*) pksv);
-    canna_status_draw(s);
+    if (pcls->conversion_start == False) {
+	jrKanjiStatusWithValue *pksv;
+
+	pcls->conversion_start = True;
+	if (canna_get_current_mode(s) == CANNA_MODE_AlphaMode) {
+	    canna_change_mode(s, CANNA_MODE_HenkanMode);
+	}
+	canna_status_draw(s);
+    }
 }
 
 int
@@ -1006,7 +1046,17 @@ canna_translate_keyevent(IMKeyListEvent *kev)
     case IM_VK_ENTER:
 	return 0x0D;
 
+    case IM_VK_BACK_SPACE:
+	return 0x08;
+
+    case IM_VK_DELETE:
+	return 0x04;
+
     default:
+	if (k->modifier & IM_CTRL_MASK) {
+	    /* no process any Ctrl characters here */
+	    break;
+	}
 	if ((k->keyChar > 0)
 	    && (k->keyChar < 0xFFFF)) {
 	    /* Should we translate it to EUC? */
@@ -1035,9 +1085,19 @@ canna_process_keyevent(iml_session_t *s, IMKeyListEvent *kev)
 	n = jrKanjiString(canna_session_context(s),
 			  ch, buf, size, pks);
 	buf[n] = '\0';
+
+	/* if there is no preedit characters and no related characters,
+	 * shouldn't process it.
+	 */
+	if (n == 1 && (pks->info & KanjiThroughInfo) && pks->length == 0) {
+		pks->info &= ~KanjiThroughInfo;
+		goto no_process;
+	}
 	if (n > 0) {
 	    IMText *p;
-    	    p = canna_commit_string(s, buf);
+
+	    pks->info &= ~KanjiThroughInfo;
+	    p = canna_commit_string(s, buf);
 	    send_commit(s, p, 1);
 	}
 	if (pks->length >= 0)
@@ -1051,6 +1111,7 @@ canna_process_keyevent(iml_session_t *s, IMKeyListEvent *kev)
 		canna_lookup_choice_done(s);
 	}
     }else{
+no_process:;
 	/* I don't process this keyevent.  Return it. */
 	iml_inst *lp;
 	lp = s->If->m->iml_make_keypress_inst(s, ((IMKeyEventStruct *)
@@ -1103,13 +1164,6 @@ Bool
 if_canna_OpenIF(iml_if_t *If)
 {
     int st;
-    if (canna_init(If->ifpath_name)) {
-	if (!jrKanjiControl(0, KC_QUERYCONNECTION, (char *) 0)) {
-	    fprintf(stderr,
-		"htt: canna.so: Unable to connect with canna server.\n");
-	    return False;
-	}
-    }
 
     st = csconv_status(); 
     if (st == CSC_UNLOADED) {
@@ -1124,8 +1178,6 @@ if_canna_OpenIF(iml_if_t *If)
 Bool
 if_canna_CloseIF(iml_if_t *If)
 {
-    jrKanjiControl(0, KC_FINALIZE, (char *) 0);
-
     return True;
 }
 
@@ -1146,12 +1198,25 @@ if_canna_OpenDesktop(iml_desktop_t * desktop,
 		     IMArgList args,
 		     int num_args)
 {
+    CannaLEDesktop *d;
+
+    d = (CannaLEDesktop *) malloc(sizeof (CannaLEDesktop));
+    memset(d, 0, sizeof(CannaLEDesktop));
+    d->drop_priv = canna_drop_privilege(desktop->user_name);
+    desktop->specific_data = (void *) d;
+
+    canna_context_id_counter = 1;
+
     return True;
 }
 
 Bool
 if_canna_CloseDesktop(iml_desktop_t * desktop)
 {
+    CannaLEDesktop *d = (CannaLEDesktop *) desktop->specific_data;
+
+    free(d);
+
     return True;
 }
 
@@ -1160,21 +1225,31 @@ if_canna_CreateSC(iml_session_t *s, IMArgList args, int num_args)
 {
     CannaLESession *pcls = (CannaLESession*) malloc(sizeof(CannaLESession));
     jrKanjiStatus *pks = (jrKanjiStatus*) malloc(sizeof(jrKanjiStatus));
+    iml_desktop_t *desktop = s->desktop;
+    CannaLEDesktop *d = (CannaLEDesktop *) desktop->specific_data;
     unsigned char *buf;
+
     buf = (unsigned char *) malloc(CANNA_COMMIT_STRING_BUFSIZE);
     if ((!pcls) || (!pks) || (!buf)) return False;
     pcls->ksv.ks = pks;
     pcls->ksv.buffer = buf;
     buf[0] = '\0';
     pcls->ksv.bytes_buffer = CANNA_COMMIT_STRING_BUFSIZE;
-    pcls->status_start = False;
-    pcls->luc_start = False;
-    pcls->preedit_start = False;
+    pcls->conversion_start = False;
     pcls->context_id = canna_context_id_counter++;
     /* Init jrKanjiStatus variable with 0. */
     memset(pks, 0, sizeof(jrKanjiStatus));
 
     s->specific_data = (void*) pcls;
+
+    /* initialize here because of avoid wrong initialization for multiple users */
+    if (canna_init(s, d->drop_priv ? desktop->user_name : NULL)) {
+	if (!jrKanjiControl(canna_session_context(s), KC_QUERYCONNECTION, (char *) 0)) {
+	    fprintf(stderr, "htt: CannaLE: Unable to connect with canna server.\n");
+	    return False;
+	}
+    }
+
     return True;
 }
 
@@ -1182,10 +1257,9 @@ Bool
 if_canna_DestroySC(iml_session_t *s)
 {
     CannaLESession *pcls;
+
     pcls = canna_session_data(s);
-    jrKanjiControl(canna_session_context(s),
-		   KC_CLOSEUICONTEXT,
-		   (char*) canna_session_status(s));
+    jrKanjiControl(canna_session_context(s), KC_FINALIZE, (char *) 0);
     free(pcls->ksv.buffer);
     free(pcls->ksv.ks);
     free(pcls);
@@ -1296,6 +1370,68 @@ if_canna_SendEvent(iml_session_t *s, IMInputEvent *ev)
     return;
 }
 
+static void
+canna_change_mode(iml_session_t *s, int id)
+{
+    CannaLESession *pcls = canna_session_data(s);
+    jrKanjiStatusWithValue *pksv;
+
+    pksv = canna_session_status(s);
+    pksv->val = id;
+    jrKanjiControl(canna_session_context(s),
+		   KC_CHANGEMODE, (char*) pksv);
+}
+
+static int
+canna_get_current_mode(iml_session_t *s)
+{
+    CannaLESession *pcls = canna_session_data(s);
+    char mode[4];
+
+    jrKanjiControl(canna_session_context(s), KC_SETMODEINFOSTYLE, (char *) 1);
+    jrKanjiControl(canna_session_context(s), KC_QUERYMODE, (char *)mode);
+    jrKanjiControl(canna_session_context(s), KC_SETMODEINFOSTYLE, (char *) 0);
+
+    return mode[0] - '@';
+}
+
+static Bool
+canna_drop_privilege(const char *username)
+{
+    struct passwd *pw;
+    uid_t uid;
+    gid_t gid;
+
+    if (!username)
+	goto error;
+    if ((pw = getpwnam(username)) == NULL) {
+	goto error;
+    } else {
+	uid = pw->pw_uid;
+	gid = pw->pw_gid;
+	if (uid < 500)
+	    goto error;
+    }
+    if (setregid(gid, gid) < 0) {
+	return False;
+    }
+    if (setreuid (uid, uid) < 0) {
+	return False;
+    }
+    return True;
+
+error:
+    if ((pw = getpwnam("nobody")) != NULL) {
+	gid = pw->pw_gid;
+	uid = pw->pw_uid;
+	setregid(gid, gid);
+	setreuid(uid, uid);
+    } else {
+	assert(0);
+    }
+    return False;
+}
+
 /* Local Variables: */
 /* c-file-style: "bsd" */
 /* End: */
