@@ -479,7 +479,7 @@ JINT EnumCandi(JINT* pnOrgYj, JINT nLenYj, SysCandi* psc, UdcCandi* puc, JINT* p
 			{
 				nEqualFlag = TRUE;
 				
-				for(m = 0; m < min(nLenYj, nCzLen); m++)
+				for(m = 0; m < _MIN(nLenYj, nCzLen); m++)
 					if ( ( pnOrgYj[m] != GbkHz2244ToYjSM((JINT)wCzHz[m]) ) && 			\
 					     ( pnOrgYj[m] != GbkHz2244ToYj((JINT)wCzHz[m])   ) )
 					{
@@ -832,7 +832,7 @@ JINT EnumCandi(JINT* pnOrgYj, JINT nLenYj, SysCandi* psc, UdcCandi* puc, JINT* p
 				{
 					nEqualFlag = TRUE;
 				
-					for(m = 0; m < min(nLenYj, nUdc28Len); m++)
+					for(m = 0; m < _MIN(nLenYj, nUdc28Len); m++)
 						if ( ( pnOrgYj[m] != GbkHz2244ToYjSM((JINT)wCzHz[m]) ) && 			\
 						     ( pnOrgYj[m] != GbkHz2244ToYj  ((JINT)wCzHz[m]) ) )
 						{
@@ -1004,8 +1004,8 @@ VOID SortCandi(SysCandi* psc, UdcCandi* puc)
 	if ((nNumM <= 1) && (nNumD <= 1) && (nNumUdc28 <= 1))
 		return;
 	
-	nBuf = max(nSizM, nSizD);
-	nBuf = max(nBuf, nSizUdc28);
+	nBuf = _MAX(nSizM, nSizD);
+	nBuf = _MAX(nBuf, nSizUdc28);
 	
 	pwBuf = (JWORD *)malloc((nBuf + MALIGN) * sizeof(JWORD));
 	if (pwBuf == NULL)
