@@ -209,9 +209,6 @@ typedef struct {
 
 /* data per session */
 typedef struct {
-    int status_start;
-    int preedit_start;
-    int luc_start;
     int conv_on;
     
     IMText **luc_candidates;
@@ -503,9 +500,6 @@ if_newpy_CreateSC(
     int i;
 
     MyDataPerSession *p = (MyDataPerSession *) calloc(1, sizeof(MyDataPerSession));
-    p->status_start = False;
-    p->luc_start = False;
-    p->preedit_start = False;
     p->preedit_buf = (UTFCHAR *) calloc(1, sizeof(UTFCHAR) * BUFSIZE);
     p->commit_buf = (UTFCHAR *) calloc(1, sizeof(UTFCHAR) * BUFSIZE);
     
@@ -687,7 +681,7 @@ if_newpy_GetSCValue(
     return True;
 }
 
-UTFCHAR off_string[] = {0x897f, 0x6587, 0};
+UTFCHAR off_string[] = {0};
 UTFCHAR title_string[] = {0x67e5, 0x627e, 0x8868, 0};
 
 void
@@ -703,10 +697,9 @@ my_conversion_on(
     lp = s->If->m->iml_make_start_conversion_inst(s);
     s->If->m->iml_link_inst_tail(&rrv, lp);
     
-    if (session_data->preedit_start == False) {
+    if (!IS_REGION_ACTIVE(s, PREEDIT)) {
         lp = s->If->m->iml_make_preedit_start_inst(s);
         s->If->m->iml_link_inst_tail(&rrv, lp);
-        session_data->preedit_start = True;
     }
     lp = s->If->m->iml_execute(s, &rrv);
     status_draw(s);
@@ -728,17 +721,15 @@ my_conversion_off(
     /* if preedit exists, commit the string */
     commit(s);
     
-    if (session_data->luc_start == True) {
+    if (IS_REGION_ACTIVE(s, LOOKUP)) {
         lp = s->If->m->iml_make_lookup_done_inst(s);
         s->If->m->iml_link_inst_tail(&rrv, lp);
-        session_data->luc_start = False;
     }
-    if (session_data->preedit_start == True) {
+    if (IS_REGION_ACTIVE(s, PREEDIT)) {
         lp = s->If->m->iml_make_preedit_erase_inst(s);
         s->If->m->iml_link_inst_tail(&rrv, lp);
         lp = s->If->m->iml_make_preedit_done_inst(s);
         s->If->m->iml_link_inst_tail(&rrv, lp);
-        session_data->preedit_start = False;
         session_data->caret_pos = -1;
     }
     lp = s->If->m->iml_make_end_conversion_inst(s);
@@ -985,10 +976,9 @@ im_data *imdata;
 	}
 	lookup_draw(s, luc_tmp, imdata->luc_num);
     }
-    else if(session_data->luc_start == True){
+    else if (IS_REGION_ACTIVE(s, LOOKUP)) {
         lp = (iml_inst*)s->If->m->iml_make_lookup_done_inst(s);
         s->If->m->iml_execute(s, &lp);
-	session_data->luc_start = False;
     }
 }
 
@@ -1171,10 +1161,9 @@ status_draw(
     p->char_length = len;
     p->feedback = create_feedback(s, p->char_length);
 
-    if (session_data->status_start == False) {
+    if (!IS_REGION_ACTIVE(s, STATUS)) {
         lp = s->If->m->iml_make_status_start_inst(s);
         s->If->m->iml_link_inst_tail(&rrv, lp);
-        session_data->status_start = True;
     }
     lp = s->If->m->iml_make_status_draw_inst(s, p);
     s->If->m->iml_link_inst_tail(&rrv, lp);
@@ -1257,10 +1246,9 @@ preedit_draw(
     MyDataPerSession *session_data = (MyDataPerSession *) s->specific_data;
     IMText *p = make_preedit_imtext(s);
     
-    if (session_data->preedit_start == False) {
+    if (!IS_REGION_ACTIVE(s, PREEDIT)) {
         lp = s->If->m->iml_make_preedit_start_inst(s);
         s->If->m->iml_link_inst_tail(&rrv, lp);
-        session_data->preedit_start = True;
     }
     if(p->char_length == 0) {
 	lp = s->If->m->iml_make_preedit_erase_inst(s);
@@ -1329,7 +1317,7 @@ lookup_draw(iml_session_t * s, UTFCHAR ** luc_tmp, int luc_num)
     }
     labels = session_data->luc_labels;
     
-    if (session_data->luc_start == False) {
+    if (!IS_REGION_ACTIVE(s, LOOKUP)) {
         session_data->luc_top = 0;
         start = (IMLookupStartCallbackStruct *) s->If->m->iml_new(s, sizeof(IMLookupStartCallbackStruct));
         memset(start, 0, sizeof(IMLookupStartCallbackStruct));
@@ -1345,7 +1333,6 @@ lookup_draw(iml_session_t * s, UTFCHAR ** luc_tmp, int luc_num)
         start->CBPreference = NULL;
         lp = s->If->m->iml_make_lookup_start_inst(s, start);
         s->If->m->iml_execute(s, &lp);
-        session_data->luc_start = True;
     }
     draw = (IMLookupDrawCallbackStruct *) s->If->m->iml_new(s, sizeof(IMLookupDrawCallbackStruct));
     memset(draw, 0, sizeof(IMLookupDrawCallbackStruct));
