@@ -108,14 +108,14 @@ IMFeedbackList *create_feedback(iml_session_t * s, int size);
 
 static IMObjectDescriptorStruct *objects = 0;
 
-typedef struct AtokSessionPrivate_ {
+typedef struct Sampleja3SessionPrivate_ {
     int lookup_current_candidate;
-}   AtokSessionPrivate;
+}   Sampleja3SessionPrivate;
 
-typedef struct AtokDesktopPrivate_ {
+typedef struct Sampleja3DesktopPrivate_ {
     iml_session_t *root_session;
     iml_session_t *luc_session;
-}   AtokDesktopPrivate;
+}   Sampleja3DesktopPrivate;
 
 static char *class_names[] = {
     "com.sun.sampleja3.PaletteAux",
@@ -232,8 +232,8 @@ if_sampleja3_OpenDesktop(
 		    int num_args
 )
 {
-    AtokDesktopPrivate *dp = (AtokDesktopPrivate *) calloc(1, sizeof(AtokDesktopPrivate));
-    desktop->specific_data = (AtokDesktopPrivate *) dp;
+    Sampleja3DesktopPrivate *dp = (Sampleja3DesktopPrivate *) calloc(1, sizeof(Sampleja3DesktopPrivate));
+    desktop->specific_data = (Sampleja3DesktopPrivate *) dp;
     return True;
 }
 
@@ -242,7 +242,7 @@ if_sampleja3_CloseDesktop(
 		     iml_desktop_t * desktop
 )
 {
-    AtokDesktopPrivate *dp = (AtokDesktopPrivate *) desktop->specific_data;
+    Sampleja3DesktopPrivate *dp = (Sampleja3DesktopPrivate *) desktop->specific_data;
     free(dp);
     return True;
 }
@@ -254,7 +254,7 @@ if_sampleja3_CreateSC(
 		 int num_args
 )
 {
-    AtokSessionPrivate *p = (AtokSessionPrivate *) calloc(1, sizeof(AtokSessionPrivate));
+    Sampleja3SessionPrivate *p = (Sampleja3SessionPrivate *) calloc(1, sizeof(Sampleja3SessionPrivate));
     s->specific_data = (void *) p;
     return True;
 }
@@ -264,7 +264,7 @@ if_sampleja3_DestroySC(
 		  iml_session_t * s
 )
 {
-    AtokSessionPrivate *sp = (AtokSessionPrivate *) s->specific_data;
+    Sampleja3SessionPrivate *sp = (Sampleja3SessionPrivate *) s->specific_data;
     free(sp);
     return True;
 }
@@ -288,7 +288,7 @@ if_sampleja3_SetSCValues(
 {
     int i;
     IMArg *p = args;
-    AtokSessionPrivate *sp = (AtokSessionPrivate *) s->specific_data;
+    Sampleja3SessionPrivate *sp = (Sampleja3SessionPrivate *) s->specific_data;
 
     for (i = 0; i < num_args; i++, p++) {
 	switch (p->id) {
@@ -300,7 +300,7 @@ if_sampleja3_SetSCValues(
 	    break;
 	case SC_REALIZE:
 	    if (s->desktop->session_count == 1) {
-		AtokDesktopPrivate *dp = (AtokDesktopPrivate *) s->desktop->specific_data;
+		Sampleja3DesktopPrivate *dp = (Sampleja3DesktopPrivate *) s->desktop->specific_data;
 		dp->root_session = s;
 
 		aux_start(s, AUX_PALETTE_CLASS_NAME);
@@ -308,6 +308,24 @@ if_sampleja3_SetSCValues(
 	    }
 	    sampleja3_conversion_off(s);
 	    break;
+
+	case SC_CLIENT_LOCALE:
+	  if (p->value) printf("SC_CLIENT_LOCALE: %s\n", p->value);
+	  break;
+	case SC_CLIENT_INPUT_METHOD_NAME:
+	  if (p->value) printf("SC_CLIENT_INPUT_METHOD_NAME: %s\n", p->value);
+	  break;
+	case SC_CLIENT_CHARACTER_SUBSETS:
+	  if (p->value) {
+	      int *pi = (int*) p->value;
+	      printf("SC_CLIENT_CHARACTER_SUBSETS:");
+	      for (; *pi; pi++) {
+		  printf("%d ", *pi);
+	      }
+	      printf("\n");
+	  }
+	  break;
+			
 	default:
 	    break;
 	}
@@ -343,7 +361,7 @@ if_sampleja3_SendEvent(
 		  IMInputEvent * ev_
 )
 {
-    AtokDesktopPrivate *dp = (AtokDesktopPrivate *) s->desktop->specific_data;
+    Sampleja3DesktopPrivate *dp = (Sampleja3DesktopPrivate *) s->desktop->specific_data;
 
     IMInputEvent *ev = modify_event(s, ev_);
     if (ev->type == IM_EventKeyList) {
@@ -386,8 +404,8 @@ sampleja3_receive_keyevent(
 		      IMKeyEventStruct * key
 )
 {
-    AtokDesktopPrivate *dp = (AtokDesktopPrivate *) s->desktop->specific_data;
-    AtokSessionPrivate *sp = (AtokSessionPrivate *) s->specific_data;
+    Sampleja3DesktopPrivate *dp = (Sampleja3DesktopPrivate *) s->desktop->specific_data;
+    Sampleja3SessionPrivate *sp = (Sampleja3SessionPrivate *) s->specific_data;
     IMText *p;
     iml_inst *lp;
     int lookup_aux_integers[3];
@@ -398,7 +416,7 @@ sampleja3_receive_keyevent(
 	   key->keyCode, key->keyChar, key->modifier);
 
     if (ls) {
-	sp = (AtokSessionPrivate *) ls->specific_data;
+	sp = (Sampleja3SessionPrivate *) ls->specific_data;
 	if (key->keyCode == IM_VK_SPACE) {
 	    sp->lookup_current_candidate++;
 	    if (sp->lookup_current_candidate >= NUM_CANDIDATES) {
@@ -497,8 +515,8 @@ luc_draw(iml_session_t * s, int index)
     int luc_top = 0;
     UTFCHAR buf[BUFSIZE];
 
-    AtokSessionPrivate *sp = (AtokSessionPrivate *) s->specific_data;
-    AtokDesktopPrivate *dp = (AtokDesktopPrivate *) s->desktop->specific_data;
+    Sampleja3SessionPrivate *sp = (Sampleja3SessionPrivate *) s->specific_data;
+    Sampleja3DesktopPrivate *dp = (Sampleja3DesktopPrivate *) s->desktop->specific_data;
 
     IMText **candidates;
     IMText **labels;
