@@ -18,10 +18,13 @@ static int EIMIL_ID_counter = 0;
 static EIMIL_value EIMIL_t_val;
 
 static const unsigned char* EIMIL_xmlns_uri = "http://www.OpenI18N.org/EIMIL/NS/1.0";
-static EIMIL_dictionary *pdic_internal;
-EIMIL_symbol *pEIMIL_nil_sym, *pEIMIL_t_sym;
-EIMIL_symbol *pEIMIL_feedback_sym;
-EIMIL_symbol *pEIMIL_candidates_sym;
+EIMIL_dictionary *pdic_internal = NULL;
+EIMIL_symbol *pEIMIL_nil_sym = NULL;
+EIMIL_symbol *pEIMIL_t_sym = NULL;
+EIMIL_symbol *pEIMIL_feedback_sym = NULL;
+EIMIL_symbol *pEIMIL_candidates_sym = NULL;
+
+static int EIMIL_inited = 0;
 
 /********************************************************************************
 			Char/Token manipulation functions.
@@ -165,7 +168,6 @@ EIMIL_set_error(
 {
     va_list va;
     char errbuf[EIMIL_MAXERRMSG];
-    EIMIL_parser_state* pps = &ped->pcommon->ps;
 
     va_start(va, fmt);
     vsnprintf(errbuf, sizeof(errbuf), fmt, va);
@@ -436,7 +438,9 @@ EIMIL_get_name(
     UTF8 **pprefix
 )
 {
+#if FAMAO
     EIMIL_parser_state *pps = &ped->pcommon->ps;
+#endif
     Ebyte *c1;
 
     *pprefix = *pname = NULL;
@@ -492,7 +496,9 @@ EIMIL_check_attrs(
 {
     int i, num;
     int eflag = 0;
+#if FAMAO
     EIMIL_parser_state *pps = &ped->pcommon->ps;
+#endif
     EIMIL_attrs *pa;
 
     if (*ppatr)
@@ -549,8 +555,8 @@ EIMIL_parse_attrs(
     Ebyte *c1;
     int atrnum = 0;
     EIMIL_attrs *patr_s = NULL;
-    UTF8 *name;
     UTF8 *p, *prefix;
+    UTF8 *name = NULL;
     Ebyte *val;
 
     while (e > c) {
@@ -589,7 +595,7 @@ EIMIL_parse_attrs(
 	}
 
 	/* XML namespace check. */
-	if (p = strchr(name, ':')) {
+	if ((p = strchr(name, ':')) != NULL) {
 	    prefix = name;
 	    *p = '\0';
 	    p++;
@@ -648,7 +654,6 @@ EIMIL_get_attr_cdata(
     for (e = val; *e; e++);
 
     if (result) {
-	Ebyte *c;
 	*result = EIMIL_resolve_reference(val, e);
 	if (!(*result)) return NULL;
     }
@@ -708,7 +713,9 @@ EIMIL_get_type_from_attrs(
     EIMIL_attrs *patr
 )
 {
+#if FAMAO
     EIMIL_parser_state *pps = &ped->pcommon->ps;
+#endif
     UTF8 *name;
     enum EIMIL_TYPE type;
 
@@ -899,7 +906,7 @@ EIMIL_parse_tag(
 	EIMIL_set_point(pps, c + 2);
 	return EIMIL_PI_TAG;
     } else if (EIMIL_isEXmark(*c)) {
-	if (c1 = EIMIL_match(c, e, "!--")) {
+	if ((c1 = EIMIL_match(c, e, "!--")) != NULL) {
 	    c = EIMIL_skip_to(c1, e, "--");
 	    c += 2;
 	    if (!EIMIL_istag_e(*c)) {
@@ -911,13 +918,13 @@ EIMIL_parse_tag(
 	    EIMIL_set_point(pps, c);
 	    return EIMIL_COMMENT_TAG;
 	}
-	if (c1 = EIMIL_match(c, e, "!DOCTYPE")) {
+	if ((c1 = EIMIL_match(c, e, "!DOCTYPE")) != NULL) {
 	    c = EIMIL_skip_to(c1, e, ">");
 	    EIMIL_set_point(pps, c + 1);
 	    return EIMIL_DOCTYPE_TAG;
 	}
 	EIMIL_check_EOF(c1, e, ped);
-	if (c1 = EIMIL_match(c, e, "![CDATA[")) {
+	if ((c1 = EIMIL_match(c, e, "![CDATA[")) != NULL) {
 	    c = EIMIL_skip_to(c1, e, "]]>");
 	    EIMIL_set_point(pps, c + 3);
 	    return EIMIL_CDATA_TAG;
@@ -1004,11 +1011,12 @@ EIMIL_parse_element(
     void *saved_private;
     Ebyte *s, *c, *e;
     Ebyte *tag_end;
-    EIMIL_attrs *pat;
+    EIMIL_attrs *pat = NULL;
     EIMIL_element_template *pet2;
 
     s = pps->start;
     e = pps->end;
+
     EIMIL_set_point(pps, s);
 
     if (pet_current)
@@ -1027,13 +1035,15 @@ EIMIL_parse_element(
     } else {
 	n = 0;
     }
+
     pelemnums = (int*) alloca(sizeof(int) * n);
     memset(pelemnums, 0, sizeof(int) * n);
     pet2 = pet;
 
     name = NULL;
     i = 0;
-    while (c = EIMIL_next_token(ped)) {
+    while ((c = EIMIL_next_token(ped)) != NULL) {
+
 	EIMIL_set_point(pps, c);
 	if (name) free(name);
 	tt = EIMIL_parse_tag(ped, &uri, &name, &pat);
@@ -1469,6 +1479,7 @@ EIMIL_destruct_value(
     EIMIL_value *pv
 )
 {
+    if (!pv) return;
     switch(pv->type) {
       case EIMIL_TYPE_PROP:
       {
@@ -1494,6 +1505,8 @@ EIMIL_destruct_value(
       case EIMIL_TYPE_MTEXT:
        EIMIL_destruct_mtext(&pv->v.mtext);
        break;
+      default:
+       break;
     }
 
     free(pv);
@@ -1763,6 +1776,8 @@ EIMIL_free_dictionary(
     EIMIL_dictionary *pdic
 )
 {
+    if (!pdic) return;
+
     /* TODO: free EIMIL_objects!!! */
     free(pdic->namedic);
     if (pdic->iddic) free(pdic->iddic);
@@ -1778,12 +1793,19 @@ EIMIL_free_dictionary_and_symbol(
     int i;
     EIMIL_symbol **pps;
 
+    if (!pdic) return;
+
     for (i = 0; i < pdic->size; i++) {
 	for (pps = pdic->namedic[i]; (pps && *pps); pps++) {
 	    EIMIL_destruct_symbol(*pps);
 	}
+        if (pdic->namedic[i]) free(pdic->namedic[i]);
+    }
+    if (pdic->iddic) {
+        for (i = 0; i < pdic->size; i++) {
+            if (pdic->iddic[i]) free(pdic->iddic[i]);
+        }
     }
-
     EIMIL_free_dictionary(pdic);
     return;
 }
@@ -1811,7 +1833,6 @@ list_symbols(
 {
     int i;
     EIMIL_symbol **pps;
-    EIMIL_symbol *psym;
 
     for (i = 0; i < pdic->size; i++) {
 	for (pps = pdic->namedic[i]; (pps && *pps); pps++) {
@@ -1994,24 +2015,25 @@ register_symbol_id(
 
     p = pdic->iddic[idx];
     if (!p) {
-	p = (EIMIL_symbol**) malloc(EIMIL_DICTIONARY_SUBSLOT_UNIT
-				    * sizeof(EIMIL_symbol*));
+        /* create iddic[idx] */
+        p = (EIMIL_symbol**) calloc(sizeof(EIMIL_symbol *), EIMIL_DICTIONARY_SUBSLOT_UNIT);
 	if (!p) return 0;
 	pdic->iddic[idx] = p;
-	p[0] = psym;
-	p[1] = NULL;
+        p[0] = psym;
 	return 1;
     }
 
     for (i = 1;; i++, p++) {
 	if (!*p) {
 	    if ((i % EIMIL_DICTIONARY_SUBSLOT_UNIT) == 0) {
+                /* ID dictionary is fulled with EIMIL_symbol. grow this dictionary. */
 		p = (EIMIL_symbol**) realloc(pdic->iddic[idx],
 					     (i + EIMIL_DICTIONARY_SUBSLOT_UNIT)
 					     * sizeof(EIMIL_symbol*));
 		if (!p) return 0;
 		pdic->iddic[idx] = p;
 		p = p + i - 1;
+                memset(p, 0, EIMIL_DICTIONARY_SUBSLOT_UNIT * sizeof(EIMIL_symbol *));
 	    }
 	    p[0] = psym;
 	    p[1] = NULL;
@@ -2042,16 +2064,16 @@ register_symbol(
 
     p = pdic->namedic[hash];
     if (!p) {
-	p = (EIMIL_symbol**) malloc(EIMIL_DICTIONARY_SUBSLOT_UNIT
-				    * sizeof(EIMIL_symbol*));
+        p = (EIMIL_symbol**) calloc(EIMIL_DICTIONARY_SUBSLOT_UNIT, sizeof(EIMIL_symbol*));
 	if (!p) return NULL;
 	pdic->namedic[hash] = p;
 	psym = EIMIL_make_symbol(name, len, cat);
 	psym->symbolid = id;
 	if (!psym) return NULL;
+        /* register psym to id dictionary */
+        /* FIXME: may cause memory leaks if failed */
 	if (!register_symbol_id(pdic, psym)) return NULL;
-	p[0] = psym;
-	p[1] = NULL;
+        p[0] = psym;
 	return psym;
     }
     for (i = 1; ; i++, p++) {
@@ -2059,25 +2081,31 @@ register_symbol(
 	    psym = EIMIL_make_symbol(name, len, cat);
 	    psym->symbolid = id;
 	    if (!psym) return NULL;
+            /* register psym to id dictionary */
+            /* FIXME: may cause memory leaks if failed */
 	    if (!register_symbol_id(pdic, psym)) return NULL;
 	    if ((i % EIMIL_DICTIONARY_SUBSLOT_UNIT) == 0) {
+                /* Named dictionary is fulled with psym, grow it */
 		p = (EIMIL_symbol**) realloc(pdic->namedic[hash],
 					     (i + EIMIL_DICTIONARY_SUBSLOT_UNIT)
 					     * sizeof(EIMIL_symbol*));
 		pdic->namedic[hash] = p;
 		p = p + i - 1;
+                memset(p, 0, sizeof(EIMIL_DICTIONARY_SUBSLOT_UNIT) * sizeof(EIMIL_symbol*));
 	    }
 	    p[0] = psym;
 	    p[1] = NULL;
 	    return psym;
 	}
 	psym = *p;
+        /* check whether name is already registerd */
 	if ((psym->namelen == len)
 	    && (memcmp(name, psym->name, len) == 0))
 	    return NULL;
     }
 
     return NULL;
+
 }
 
 static int
@@ -2132,7 +2160,7 @@ EIMIL_intern_soft(
 )
 {
     EIMIL_symbol *psym;
-    if (psym = lookup_predefined_symbol(name)) return psym;
+    if ((psym = lookup_predefined_symbol(name)) != NULL) return psym;
 
     return lookup_symbol(pdic, name);
 }
@@ -2176,10 +2204,15 @@ EIMIL_destruct_symbol(
 )
 {
 
+    if (!psym) return;
+
     if (psym->name) free(psym->name);
     switch(psym->cat) {
       case EIMIL_CAT_VARIABLE:
-       EIMIL_RMREF(*psym->obj.v.pv);
+       if (psym->obj.v.pv) {
+         if (!psym->obj.v.constp)
+             EIMIL_RMREF(*psym->obj.v.pv);
+       }
        break;
       case EIMIL_CAT_EXCEPTION:
       {
@@ -2194,7 +2227,7 @@ EIMIL_destruct_symbol(
       }
       case EIMIL_CAT_OPERATION:
       {
-	  int i, j, n;
+	  int i, n;
 	  EIMIL_dependency *pdeps;
 	  EIMIL_symbol **pps;
 
@@ -2212,6 +2245,8 @@ EIMIL_destruct_symbol(
       case EIMIL_CAT_FUNCTION:
        /* TODO */
        break;
+      default:
+       break;
     }
 
     free(psym);
@@ -2735,9 +2770,7 @@ EIMIL_dependency_element_parser(
     void **pprivate
 )
 {
-    EIMIL_parser_state* pps = &ped->pcommon->ps;
     UTF8 *name;
-    EIMIL_symbol *psym;
     EIMIL_dependency *pdep;
     EIMIL_operation *pop = (EIMIL_operation*) *pprivate;
 
@@ -2754,10 +2787,9 @@ EIMIL_dependency_element_parser(
     for (; patr->name; patr++) {
 	if (strcmp(patr->name, "depend") == 0) {
 	    Ebyte *c;
-	    int i = 0;
 
 	    c = patr->val;
-	    while (c = EIMIL_get_attr_nmtokens(c, &name)) {
+	    while ((c = EIMIL_get_attr_nmtokens(c, &name)) != NULL) {
 		if (!EIMIL_add_symbol_to_slots(ped,
 					       pdep->numdepends,
 					       &pdep->depends,
@@ -2774,10 +2806,9 @@ EIMIL_dependency_element_parser(
 	    }
 	} else if (strcmp(patr->name, "affect") == 0) {
 	    Ebyte *c;
-	    int i = 0;
 
 	    c = patr->val;
-	    while (c = EIMIL_get_attr_nmtokens(c, &name)) {
+	    while ((c = EIMIL_get_attr_nmtokens(c, &name)) != NULL) {
 		if (!EIMIL_add_symbol_to_slots(ped,
 					       pdep->numaffects,
 					       &pdep->affects,
@@ -2811,7 +2842,6 @@ EIMIL_decldata_element_parser(
 )
 {
     enum EIMIL_TYPE etype;
-    EIMIL_parser_state* pps = &ped->pcommon->ps;
     UTF8 *name;
     EIMIL_symbol *psym = NULL;
 
@@ -2857,7 +2887,6 @@ EIMIL_declprop_element_parser(
     void **pprivate
 )
 {
-    EIMIL_parser_state* pps = &ped->pcommon->ps;
     UTF8 *name;
     EIMIL_symbol *psym = NULL;
 
@@ -2903,8 +2932,6 @@ EIMIL_declop_element_parser(
     void **pprivate
 )
 {
-    EIMIL_parser_state* pps = &ped->pcommon->ps;
-    EIMIL_element_template *pet = pps->pcet;
     UTF8 *name;
     EIMIL_symbol *psym = NULL;
 
@@ -3075,8 +3102,6 @@ EIMIL_declexception_element_parser(
     void **pprivate
 )
 {
-    EIMIL_parser_state* pps = &ped->pcommon->ps;
-    EIMIL_element_template *pet = pps->pcet;
     UTF8 *name;
     EIMIL_symbol *psym = NULL;
 
@@ -3172,7 +3197,6 @@ EIMIL_inherit_element_parser(
 )
 {
     UTF8 *uaddr;
-    char *addr;
 
     if (type != EIMIL_EMPTY_TAG) return 1;
 
@@ -3212,6 +3236,25 @@ EIMIL_interface_element_parser(
 static int num_engines;
 static EIMIL_engine_table *pengines;
 
+static void
+EIMIL_free_engine()
+{
+    int i;
+    EIMIL_engine_table *p = pengines;
+    for (i = 0; ; ++i) {
+        if (i == num_engines) break;
+        free (p->classname);
+        free (p->uri);
+        ++p;
+    }
+
+    free (pengines);
+
+    /* reset */
+    pengines = NULL;
+    num_engines = 0;
+}
+
 static EIMIL_engine_table*
 EIMIL_get_engine(
     const UTF8 *classname
@@ -3238,7 +3281,6 @@ EIMIL_register_engine(
     const UTF8 *uri
 )
 {
-    int i;
     EIMIL_engine_table *p;
     p = EIMIL_get_engine(classname);
     if (!p) {
@@ -3248,7 +3290,6 @@ EIMIL_register_engine(
 	if (!p) return 0;
 	pengines = p;
 	p += num_engines;
-	num_engines++;
 	p->classname = strdup(classname);
 	if (!p->classname) return 0;
 	p->uri = strdup(uri);
@@ -3338,6 +3379,7 @@ EIMIL_engine_element_parser(
 	/* Set up the engine. */
 	pe->ptable = pt;
 	pe->private = (*pt->handler)(EIMIL_ENGINE_INSTANCIATE, ped, NULL, NULL);
+
 	if (!pe->private) {
 	    EIMIL_set_out_of_memory(ped);
 	    return 0;
@@ -3378,7 +3420,6 @@ EIMIL_make_handle_data(
 	pbase = (EIMIL_cdata*) malloc(sizeof(EIMIL_cdata));
 	if (!pbase) {
 	    free(ped);
-	    EIMIL_free_dictionary(pdic);
 	    return NULL;
 	}
 	memset(pbase, 0, sizeof(EIMIL_cdata));
@@ -3437,10 +3478,10 @@ EIMIL_duplicate_handle(
     EIMIL_handle eh
 )
 {
-    int i, j, n, size;
     EIMIL_data *peds, *pedd;
     EIMIL_symbol *psym;
     EIMIL_dictionary *pdic;
+    int i, n;
 
     peds = (EIMIL_data*) eh;
     pedd = EIMIL_make_handle_data(peds->pcommon);
@@ -3456,7 +3497,6 @@ EIMIL_duplicate_handle(
 
     /* engine context */
     {
-	int i, n;
 	void **pecs, **pecd;
 	EIMIL_engine *pe;
 	EIMIL_engine_table *pt;
@@ -3526,6 +3566,7 @@ EIMIL_free_handle(
     if (!ped->duplicated) {
 	EIMIL_cdata *pc = ped->pcommon;
 	if (pc->ps.buf) free(pc->ps.buf);
+        if (pc->ps.pxmlns) free(pc->ps.pxmlns);
 	DESTROY_SYNC_OBJECT(pc->sync_object);
 	free(pc);
     }
@@ -3598,9 +3639,9 @@ EIMIL_init_predefined_symbol()
 int
 EIMIL_initialize()
 {
-    static int inited = 0;
 
-    if (inited) return 1;
+    if (EIMIL_inited) return 1;
+
     /* initialize internal object */
     EIMIL_t_val.refcount = 1;
     EIMIL_t_val.type = EIMIL_TYPE_BOOL;
@@ -3610,7 +3651,26 @@ EIMIL_initialize()
     if (!EIMILFile_init()) return 0;
     if (!PCE_init()) return 0;
 
-    inited = 1;
+    EIMIL_inited = 1;
+    return 1;
+}
+
+int
+EIMIL_finalize()
+{
+    if (!EIMIL_inited) return 1;
+
+    EIMIL_free_dictionary_and_symbol(pdic_internal);
+
+    EIMIL_free_engine(pengines);
+
+
+    /* EIMIL_destruct_symbol(pEIMIL_t_sym); */
+    /* EIMIL_destruct_symbol(pEIMIL_feedback_sym); */
+    /* EIMIL_destruct_symbol(pEIMIL_candidates_sym); */
+    pdic_internal = NULL;
+    EIMIL_inited = 0;
+
     return 1;
 }
 
@@ -3719,6 +3779,7 @@ EIMIL_queue_event(
 {
     EIMIL_value **pqn;
 
+
     if (ped->pqueue_ev == (ped->queueslots + EIMIL_EVENT_QUEUESIZE - 1))
 	pqn = ped->queueslots;
     else
@@ -3792,6 +3853,7 @@ EIMIL_send_event(
     n = pcommon->num_engines;
     pengine_ctx = ped->pengine_context;
 
+    /* if event, store event */
     if (pv_event) {
 	ASSERT(pv_event->type == EIMIL_TYPE_EVENT);
 	EIMIL_queue_event(ped, pv_event);
@@ -3805,6 +3867,7 @@ EIMIL_send_event(
 	pengine_ctx++;
     }
 
+    if (pv_event) EIMIL_destruct_value(pv_event);
     return 1;
 }
 
