@@ -166,35 +166,35 @@ EIMIL_convert_UTF8_to_UTF32char(
     UTF32 *pch
 )
 {
-    UTF32 ch;
+    if (!p) return 0;
 
     if (*p < 0x80) {
-	*pch = *p;
+	if (pch) *pch = *p;
 	return 1;
     } else if (*p < 0xE0) {
-	*pch = (((p[0] & 0x1F) << 6)
+	if (pch) *pch = (((p[0] & 0x1F) << 6)
 		| (p[1] & 0x3F));
 	return 2;
     } else if (*p < 0xF0) {
-	*pch = (((p[0] & 0x0F) << 12)
+	if (pch) *pch = (((p[0] & 0x0F) << 12)
 		| ((p[1] & 0x3F) << 6)
 		| (p[2] & 0x3F));
 	return 3;
     } else if (*p < 0xF8) {
-	*pch = (((p[0] & 0x07) << 18)
+	if (pch) *pch = (((p[0] & 0x07) << 18)
 		| ((p[1] & 0x3F) << 12)
 		| ((p[2] & 0x3F) << 6)
 		| (p[3] & 0x3F));
 	return 4;
     } else if (*p < 0xFC) {
-	*pch = (((p[0] & 0x03) << 24)
+	if (pch) *pch = (((p[0] & 0x03) << 24)
 		| ((p[1] & 0x3F) << 18)
 		| ((p[2] & 0x3F) << 12)
 		| ((p[3] & 0x3F) << 6)
 		| (p[4] & 0x3F));
 	return 5;
     } else {
-	*pch = (((p[0] & 0x01) << 30)
+	if (pch) *pch = (((p[0] & 0x01) << 30)
 		| ((p[1] & 0x3F) << 24)
 		| ((p[2] & 0x3F) << 18)
 		| ((p[3] & 0x3F) << 12)
@@ -370,7 +370,7 @@ EIMIL_convert_UTF16_to_UTF32(
 	if (UTF16_HS_P(hs1)) {
 	    hs2 = *pu16++;
 	    i++;
-	    if (i >= u16len) {
+	    if (i > u16len) {
 		/* Invalid code.
 		   TODO:We should output error.  */
 		break;
@@ -437,7 +437,6 @@ EIMIL_construct_mtext_from_UTF8(
     const UTF8 *in
 )
 {
-    int len;
     UTF32 *pu;
     EIMIL_value *pv;
     EIMIL_mtext *pm;
@@ -929,8 +928,6 @@ EIMIL_mtext_concat(
     EIMIL_mtext_props *pmp, *pmp2;
     UTF32 *pu;
 
-    EIMIL_value **pprops;
-
     pvr = (EIMIL_value*) malloc(sizeof(EIMIL_value));
     if (!pvr) return NULL;
     memset(pvr, 0, sizeof(EIMIL_value));
@@ -1004,7 +1001,6 @@ EIMIL_mtext_substr(
     EIMIL_value **ppv, **ppv2, *pv, *pvr;
     UTF32 *pu;
 
-    EIMIL_value **pprops;
 
     ASSERT(end > st);
 
@@ -1161,7 +1157,7 @@ EIMIL_prop_convert_IMProp(
 )
 {
     int i;
-    EIMIL_value *pv, *pv2, *ppv;
+    EIMIL_value *pv, *pv2;
     EIMIL_prop *pprop;
 
     switch(pim->type) {
@@ -1217,7 +1213,7 @@ create_feedback(
 )
 {
     int i;
-    IMFeedbackList *feedback, *fbl;
+    IMFeedbackList *feedback;
     IMFeedback *fb;
     
     feedback = (IMFeedbackList *) malloc(sizeof(IMFeedbackList) * size);
