@@ -23,12 +23,20 @@
 
 #include "iiimcfint.h"
 
-#define NODE_DEFAULT	"localhost"
-#define NODE_KEY	"node="
-#define NODE_KEY_LEN	(5)
-#define SERVICE_DEFAULT	"9010"
-#define SERVICE_KEY	"service="
-#define SERVICE_KEY_LEN	(8)
+#define NODE_KEY	  "node="
+#define NODE_KEY_LEN	  (5)
+#define SERVICE_KEY	  "service="
+#define SERVICE_KEY_LEN	  (8)
+#define CERT_FILE         "tls_cert="
+#define CERT_FILE_LEN     (9)
+#define CERT_KEY_FILE     "tls_key="
+#define CERT_KEY_FILE_LEN (8)
+#define CERT_CA_FILE      "tls_cafile="
+#define CERT_CA_FILE_LEN  (11)
+#define CERT_CA_PATH      "tls_capath="
+#define CERT_CA_PATH_LEN  (11)
+#define USE_TLS           "use_tls="
+#define USE_TLS_LEN       (8)
 
 #define SERVER_FILE		".iiim/server"
 #define SERVER_FILE_LEN		(12)
@@ -179,6 +187,43 @@ get_param_compat(
     return IIIMF_STATUS_SUCCESS;
 }
 
+static void
+iiimcf_client_config_free(
+    IIIMCF_client_conf *pconf
+)
+{
+    IIIMCF_client_conf *pc;
+    IIIMCF_client_conf *pc2;
+
+    if (!pconf) return;
+    for (pc = pconf; pc; ) {
+        pc2 = pc->next;
+        if(pc->server_node) free(pc->server_node);
+        if(pc->service) free(pc->service);
+        if(pc->cert_file) free(pc->cert_file);
+        if(pc->cert_key) free(pc->cert_key);
+        if(pc->ca_file) free(pc->ca_file);
+        if(pc->ca_path) free(pc->ca_path);
+        free(pc);
+        pc = pc2;
+    }
+}
+
+static IIIMCF_client_conf *
+iiimcf_client_config_new(
+    char *node,
+    char *service
+)
+{
+    IIIMCF_client_conf *ret;
+    ret = malloc(sizeof(*ret));
+    if (!ret) return NULL;
+    memset(ret, 0, sizeof(*ret));
+
+    ret->server_node = node;
+    ret->service = service;
+    return ret;
+}
 
 static IIIMF_status
 iiimcf_client_file_server(
@@ -195,8 +240,8 @@ iiimcf_client_file_server(
     char *		pa;
     const char *	p;
     size_t		rest;
-    char *		node;
-    char *		service;
+    IIIMCF_client_conf  *pconf = NULL;
+    char *		node = NULL;
 
     if (!penv->home_dir) return IIIMF_STATUS_CONFIG;
 
@@ -228,33 +273,89 @@ iiimcf_client_file_server(
     p = pa;
     rest = st.st_size;
 
-    POSITION_HEAD(p, rest);
-    if (rest < NODE_KEY_LEN) {
-	(void)munmap(pa, st.st_size);
-	return IIIMF_STATUS_CONFIG;
+    /* create client pconfig */
+    for (; rest > 0; ) {
+        POSITION_HEAD(p, rest);
+
+        if (!strncasecmp(p, NODE_KEY, NODE_KEY_LEN)) {
+            IIIMCF_client_conf *pconf_next;
+            status = get_param(&p, &rest, NODE_KEY, NODE_KEY_LEN, &node);
+            if (IIIMF_STATUS_SUCCESS != status) {
+                goto error;
+            }
+            if (!penv->pconf) {
+                /* if penv->pconf is not yet initialized, the first one */
+                penv->pconf = iiimcf_client_config_new(node, NULL);
+                if (!penv->pconf) {
+                    free(node);
+                    goto error;
+                }
+                pconf = penv->pconf;
+            } else {
+                /* otherwise, create new pconfig and link to penv->pconf */
+                pconf_next = iiimcf_client_config_new(node, NULL);
+                if (!pconf_next) {
+                    free(node);
+                    goto error;
+                }
+                pconf->next = pconf_next;
+                pconf = pconf_next;
+            }
+        } else if (!strncasecmp(p, SERVICE_KEY, SERVICE_KEY_LEN)) {
+            if (!pconf) goto error;
+            status = get_param(&p, &rest, SERVICE_KEY, SERVICE_KEY_LEN, &pconf->service);
+            if (IIIMF_STATUS_SUCCESS != status) {
+               goto error;
+            }
+        } else if (!strncasecmp(p, CERT_FILE, CERT_FILE_LEN)) {
+            if (!pconf) goto error;
+            status = get_param(&p, &rest, CERT_FILE, CERT_FILE_LEN, &pconf->cert_file);
+            if (IIIMF_STATUS_SUCCESS != status) {
+               goto error;
+            }
+        } else if (!strncasecmp(p, CERT_KEY_FILE, CERT_KEY_FILE_LEN)) {
+            if (!pconf) goto error;
+            status = get_param(&p, &rest, CERT_FILE, CERT_FILE_LEN, &pconf->cert_key);
+            if (IIIMF_STATUS_SUCCESS != status) {
+               goto error;
+            }
+        } else if (!strncasecmp(p, CERT_CA_FILE, CERT_CA_FILE_LEN)) {
+            if (!pconf) goto error;
+            status = get_param(&p, &rest, CERT_CA_FILE, CERT_CA_FILE_LEN, &pconf->ca_file);
+            if (IIIMF_STATUS_SUCCESS != status) {
+               goto error;
+            }
+        } else if (!strncasecmp(p, CERT_CA_PATH, CERT_CA_PATH_LEN)) {
+            if (!pconf) goto error;
+            status = get_param(&p, &rest, CERT_CA_PATH, CERT_CA_PATH_LEN, &pconf->ca_path);
+            if (IIIMF_STATUS_SUCCESS != status) {
+               goto error;
+            }
+        } else if (!strncasecmp(p, USE_TLS, USE_TLS_LEN)) {
+            char *use_tls;
+            if (!pconf) goto error;
+            status = get_param(&p, &rest, USE_TLS, USE_TLS_LEN, &use_tls);
+            if (IIIMF_STATUS_SUCCESS != status) {
+               goto error;
+            }
+            if (!strcasecmp(use_tls, "yes")) {
+                pconf->use_tls = 1;
+            }
+            free(use_tls);
+        } else {
+            POSITION_TAIL(p, rest);
+        }
     }
 
-    status = get_param(&p, &rest, NODE_KEY, NODE_KEY_LEN, &node);
-    if (IIIMF_STATUS_SUCCESS != status) {
-	(void)munmap(pa, st.st_size);
-	return status;
-    }
-
-    penv->server_node = node;
-
-    POSITION_HEAD(p, rest);
-    if (rest < SERVICE_KEY_LEN) {
-	(void)munmap(pa, st.st_size);
-	return IIIMF_STATUS_SUCCESS;
-    }
-
-    status = get_param(&p, &rest, SERVICE_KEY, SERVICE_KEY_LEN, &service);
-
     (void)munmap(pa, st.st_size);
-
-    penv->service = service;
-
     return IIIMF_STATUS_SUCCESS;
+
+error:
+
+    iiimcf_client_config_free(penv->pconf);
+    penv->pconf = NULL;
+    (void)munmap(pa, st.st_size);
+    return IIIMF_STATUS_CONFIG;
 }
 
 
@@ -273,7 +374,7 @@ iiimcf_client_file_compat_server(
     char *		pa;
     const char *	p;
     size_t		rest;
-    char *		node;
+    char *		node = NULL;
     char *		service;
 
     if (!penv->home_dir) return IIIMF_STATUS_CONFIG;
@@ -319,13 +420,12 @@ iiimcf_client_file_compat_server(
 				  SERVER_COMPAT_KEY_LEN,
 				  &node);
 	if (IIIMF_STATUS_SUCCESS == status) {
-	    penv->server_node = node;
 	    p += 1;
 	    rest -= 1;
 	    break;
 	}
     }
-    if (NULL == penv->server_node) {
+    if (!node) {
 	return IIIMF_STATUS_CONFIG;
     }
 
@@ -339,7 +439,7 @@ iiimcf_client_file_compat_server(
     }
 
     if (IIIMF_STATUS_SUCCESS == status) {
-	penv->service = service;
+        penv->pconf = iiimcf_client_config_new(node, service);
     }
 
     return status;
@@ -366,13 +466,11 @@ iiimcf_client_environ_server(
 	return status;
     }
 
-    penv->server_node = node;
-
     POSITION_HEAD(p, rest);
 
     status = get_param(&p, &rest, "", 0, &service);
 
-    penv->service = service;
+    penv->pconf = iiimcf_client_config_new(node, service);
 
     return IIIMF_STATUS_SUCCESS;
 }
@@ -563,13 +661,13 @@ iiimcf_client_os_arch(
     IIIMCF_client_env *penv
 )
 {
-    char arch[16];
 
 #if defined(OS_ARCH)
     penv->os_arch = strdup(OS_ARCH);
 #else /* !OS_ARCH */
 #if defined(HAVE_SYSINFO) && defined(SI_ARCHITECTURE)
     {
+        char arch[16];
 	int ret;
 	ret = sysinfo(SI_ARCHITECTURE, arch, sizeof (arch));
 	if (-1 == ret) {
@@ -614,23 +712,21 @@ iiimcf_create_client_env(
     st = iiimcf_attr_get_string_value(attr,
 				      IIIMCF_ATTR_SERVER_ADDRESS,
 				      &server_node);
-    if (st == IIIMF_STATUS_SUCCESS) {
-	penv->server_node = strdup(server_node);
-	if (!penv->server_node) goto memory_error;
-    } else if (st != IIIMF_STATUS_NO_ATTR_VALUE) {
-	goto error;
-    }
+
+    if (st != IIIMF_STATUS_SUCCESS && st != IIIMF_STATUS_NO_ATTR_VALUE) goto error;
 
     st = iiimcf_attr_get_string_value(attr,
 				      IIIMCF_ATTR_SERVER_SERVICE,
 				      &service);
-    if (st == IIIMF_STATUS_SUCCESS) {
-	penv->service = strdup(service);
-	if (!penv->service) goto memory_error;
-    } else if (st != IIIMF_STATUS_NO_ATTR_VALUE) {
-	goto error;
+    if (st != IIIMF_STATUS_SUCCESS && st != IIIMF_STATUS_NO_ATTR_VALUE) goto error;
+
+    if (server_node || service) {
+        penv->pconf = iiimcf_client_config_new(server_node ? strdup(server_node) : NULL,
+                                              service ?  strdup(service) : NULL);
+        if (!penv->pconf) goto memory_error;
     }
 
+
     st = iiimcf_attr_get_string_value(attr,
 				      IIIMCF_ATTR_USERNAME,
 				      &user_name);
@@ -644,7 +740,7 @@ iiimcf_create_client_env(
 	struct passwd *pwd;
 	/* TODO!  Use penv->user_name to check the
 	   home directory. */
-	if (pwd = getpwuid(geteuid())) {
+	if ((pwd = getpwuid(geteuid()))) {
 	    if (!penv->user_name)
 		penv->user_name = strdup(pwd->pw_name);
 	    if (!penv->user_name) goto memory_error;
@@ -683,13 +779,39 @@ iiimcf_create_client_env(
 	    }
 	}
     }
-    if (!penv->server_node) {
-	penv->server_node = strdup(NODE_DEFAULT);
-	if (!penv->server_node) goto memory_error;
-    }
-    if (!penv->service) {
-	penv->service = strdup(SERVICE_DEFAULT);
-	if (!penv->service) goto memory_error;
+    if (!penv->pconf) {
+        /* 
+         * we don't have any pconfiguration.
+         * prepare default one.
+         */
+        int node_len;
+        char *tmp_node;
+
+        /* 
+         * first try to /tmp/.iiimp-unix-${USERNAME}/
+         */
+        node_len = strlen(penv->user_name) + 17;
+        tmp_node = malloc(node_len + 1);
+        if (!tmp_node) goto memory_error;
+        snprintf(tmp_node, node_len + 1, "/tmp/.iiimp-unix-%s", penv->user_name);
+        tmp_node[node_len] = '\0';
+        penv->pconf = iiimcf_client_config_new(tmp_node, NULL);
+        if (!penv->pconf) {
+            free(tmp_node);
+            goto memory_error;
+        }
+
+        /* 
+         * second try to /tmp/.iiimp-unix/
+         */
+        tmp_node = strdup("/tmp/.iiimp-unix");
+        if (!tmp_node) goto memory_error;
+        penv->pconf->next = iiimcf_client_config_new(tmp_node, NULL);
+        if (!penv->pconf->next) {
+            free(tmp_node);
+            goto memory_error;
+        }
+
     }
 
     st = iiimcf_attr_get_string_value(attr,
@@ -761,16 +883,16 @@ iiimcf_delete_client_env(
     if (penv->password) free(penv->password);
     if (penv->home_dir) free(penv->home_dir);
     if (penv->node) free(penv->node);
-    if (penv->server_node) free(penv->server_node);
-    if (penv->service) free(penv->service);
     if (penv->type) free(penv->type);
     if (penv->os_name) free(penv->os_name);
     if (penv->os_arch) free(penv->os_arch);
     if (penv->os_version) free(penv->os_version);
     if (penv->X_display_name) free(penv->X_display_name);
     if (penv->X_server_vendor) free(penv->X_server_vendor);
+    if (penv->pconf)  iiimcf_client_config_free(penv->pconf);
 
     free(penv);
+    return IIIMF_STATUS_SUCCESS;
 }
 
 IIIMF_status
