@@ -2,7 +2,24 @@
   iiimcf.c
 */
 
+#define NODE_DEFAULT      "localhost"
+#define SERVICE_DEFAULT   "9010"
+#define TLS_DEFAULT       "9011"
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
+#include <string.h>
+#endif
+#ifdef HAVE_MEMORY_H
+#include <memory.h>
+#endif
+
 #include "iiimcfint.h"
 #include <EIMIL.h>
 
@@ -28,7 +45,8 @@ iiimcf_make_string(
 
     pu16 = (IIIMP_card16*) malloc(sizeof(*pu16) * (len + 1));
     if (!pu16) return NULL;
-    memcpy(pu16, str, sizeof(*pu16) * len);
+    if (len > 0)
+	memcpy(pu16, str, sizeof(*pu16) * len);
     pu16[len] = 0;
 
     return pu16;
@@ -92,7 +110,7 @@ iiimcf_unlock_handle(
 			       message handlers
 ********************************************************************************/
 
-static void
+void
 iiimcf_unregister_langs(
     int n,
     IIIMCF_language_rec **ppl
@@ -221,12 +239,131 @@ iiimcf_register_trigger_keys(
     return IIIMF_STATUS_SUCCESS;
 }
 
+IIIMF_status
+iiimcf_register_hotkeys(
+    IIIMCF_handle_rec *ph,
+    IIIMP_message *pmes
+)
+{
+    int i, j, n, n_hotkeys;
+    IIIMCF_hotkey *hk;
+    HOTKEY_LIST *hklist;
+    IIIMCF_keyevent *pkev;
+    IIIMP_keyevent *pimk;
+
+    IIIMP_register_hotkeys_v *pimrk = &pmes->v.register_hotkeys;
+
+    ASSERT(pmes->opcode == IM_REGISTER_HOTKEYS);
+
+    hklist = pimrk->hotkeys;
+    n_hotkeys = pimrk->hotkeys->count;
+    ph->flag = pimrk->flag;
+
+    if (n_hotkeys > 0) {
+	hk = (IIIMCF_hotkey *) malloc(sizeof(IIIMCF_hotkey) * n_hotkeys);
+	if (!hk) return IIIMF_STATUS_MALLOC;
+
+        if (ph->phot_keys) {
+            for (i = 0; i < ph->num_hotkeys; ++i) {
+                free(ph->phot_keys[i].keys);
+            }
+            free(ph->phot_keys);
+        }
+	ph->phot_keys = hk;
+
+	for(i=0; i<n_hotkeys; i++) {
+	    hk[i].hotkey_id = hklist->hotkey[i].hotkeyctrl.hotkey_id;
+	    hk[i].state_flag = hklist->hotkey[i].hotkeyctrl.state_flag;
+	    hk[i].action_flag = hklist->hotkey[i].hotkeyctrl.action_flag;
+
+	    n = hklist->hotkey[i].hotkeylist->count;
+	    if (n > 0) {
+		pkev = (IIIMCF_keyevent*) malloc(sizeof(IIIMCF_keyevent) * n);
+		if (!pkev) return IIIMF_STATUS_MALLOC;
+
+		hk[i].keys = pkev;
+		pimk = hklist->hotkey[i].hotkeylist->keyevent;
+		for (j = 0; j < n; j++, pkev++, pimk++) {
+		    pkev->keycode = pimk->keycode;
+		    pkev->keychar = pimk->keychar;
+		    pkev->modifier = pimk->modifier;
+		    pkev->time_stamp = pimk->time_stamp;
+		}
+		hk[i].nkeys = j;
+	    }
+	}
+        ph->num_hotkeys = n_hotkeys;
+    }
+    return IIIMF_STATUS_SUCCESS;
+}
+
+static IIIMF_status
+iiimcf_register_object_descriptor_list(
+    IIIMCF_handle_rec *ph,
+    IIIMP_object_descriptor *piiimp_objdesc
+)
+{
+    int n;
+    IIIMP_object_descriptor *pimod;
+    IIIMCF_object_descriptor *pod, *podh;
+    IIIMP_card16 *pu16;
+
+    free_object_descriptor(ph->pobjdesc, ph->object_descriptor_size);
+    ph->pobjdesc = NULL;
+    ph->object_descriptor_size = 0;
+    for (n = 0, pimod = piiimp_objdesc; pimod; pimod = pimod->next) n++;
+
+    pod = podh = (IIIMCF_object_descriptor*) malloc(sizeof(*pod) * n);
+    if (!pod) return IIIMF_STATUS_MALLOC;
+    memset(podh, 0, sizeof(*podh) * n);
+
+    for (pimod = piiimp_objdesc; pimod; pimod = pimod->next, pod++) {
+	pod->category = pimod->category;
+	pod->size = pimod->size;
+	pod->predefined_id = pimod->id_pre;
+	pod->dynamic_id = pimod->id_dyn;
+	pu16 = iiimcf_make_string(pimod->rdun->ptr, pimod->rdun->len);
+	if (!pu16) {
+	    free_object_descriptor(podh, n);
+	    return IIIMF_STATUS_MALLOC;
+	}
+	pod->domain = pu16;
+
+	pu16 = iiimcf_make_string(pimod->hrn->ptr, pimod->hrn->len);
+	if (!pu16) {
+	    free_object_descriptor(podh, n);
+	    return IIIMF_STATUS_MALLOC;
+	}
+	pod->hrn = pu16;
+
+	pu16 = iiimcf_make_string(pimod->signature->ptr,
+				  pimod->signature->len);
+	if (!pu16) {
+	    free_object_descriptor(podh, n);
+	    return IIIMF_STATUS_MALLOC;
+	}
+	pod->signature = pu16;
+
+	pu16 = iiimcf_make_string(pimod->user->ptr, pimod->user->len);
+	if (!pu16) {
+	    free_object_descriptor(podh, n);
+	    return IIIMF_STATUS_MALLOC;
+	}
+	pod->user = pu16;
+    }
+    ph->pobjdesc = podh;
+    ph->object_descriptor_size = n;
+
+    return IIIMF_STATUS_SUCCESS;
+}
+
 IIIMF_status
 iiimcf_setimvalues(
     IIIMCF_handle_rec *ph,
     IIIMP_message *pmes
 )
 {
+    IIIMF_status st;
     IIIMP_imattribute *pimattrs;
     ASSERT(pmes->opcode == IM_SETIMVALUES);
 
@@ -235,63 +372,15 @@ iiimcf_setimvalues(
 	 pimattrs = pimattrs->next) {
 	switch (pimattrs->id_pre) {
 	  case IIIMP_IMATTRIBUTE_INPUT_METHOD_LIST:
-	   /* TODO */
+	   st = iiimcf_register_input_method_list(ph, pimattrs->value.inputmethod_descriptor);
+	   if (st != IIIMF_STATUS_SUCCESS) return st;
 	   break;
+
 	  case IIIMP_IMATTRIBUTE_OBJECT_DESCRIPTOR_LIST:
-	  {
-	      int n;
-	      IIIMP_object_descriptor *pimod;
-	      IIIMCF_object_descriptor *pod, *podh;
-	      IIIMP_card16 *pu16;
+	   st = iiimcf_register_object_descriptor_list(ph, pimattrs->value.object_descriptor);
+	   if (st != IIIMF_STATUS_SUCCESS) return st;
+	   break;
 
-	      free_object_descriptor(ph->pobjdesc, ph->object_descriptor_size);
-	      ph->pobjdesc = NULL;
-	      ph->object_descriptor_size = 0;
-	      for (n = 0, pimod = pimattrs->value.object_descriptor;
-		   pimod;
-		   pimod = pimod->next) n++;
-	      pod = podh = (IIIMCF_object_descriptor*) malloc(sizeof(*pod) * n);
-	      if (!pod) return IIIMF_STATUS_MALLOC;
-	      memset(podh, 0, sizeof(*podh) * n);
-	      for (pimod = pimattrs->value.object_descriptor;
-		   pimod;
-		   pimod = pimod->next, pod++) {
-		  pod->category = pimod->category;
-		  pod->size = pimod->size;
-		  pod->predefined_id = pimod->id_pre;
-		  pod->dynamic_id = pimod->id_dyn;
-		  pu16 = iiimcf_make_string(pimod->rdun->ptr, pimod->rdun->len);
-		  if (!pu16) {
-		      free_object_descriptor(podh, n);
-		      return IIIMF_STATUS_MALLOC;
-		  }
-		  pod->domain = pu16;
-
-		  pu16 = iiimcf_make_string(pimod->hrn->ptr, pimod->hrn->len);
-		  if (!pu16) {
-		      free_object_descriptor(podh, n);
-		      return IIIMF_STATUS_MALLOC;
-		  }
-		  pod->hrn = pu16;
-
-		  pu16 = iiimcf_make_string(pimod->signature->ptr,
-					    pimod->signature->len);
-		  if (!pu16) {
-		      free_object_descriptor(podh, n);
-		      return IIIMF_STATUS_MALLOC;
-		  }
-		  pod->signature = pu16;
-
-		  pu16 = iiimcf_make_string(pimod->user->ptr, pimod->user->len);
-		  if (!pu16) {
-		      free_object_descriptor(podh, n);
-		      return IIIMF_STATUS_MALLOC;
-		  }
-		  pod->user = pu16;
-	      }
-	      ph->pobjdesc = podh;
-	      ph->object_descriptor_size = n;
-	  }
 	  default:
 	   /* simply ignore */
 	   break;
@@ -329,6 +418,19 @@ free_object_descriptor(
 		            connection/send/receive management
 ********************************************************************************/
 
+static void
+iiimcf_delete_stream(
+    IIIMCF_handle_rec *ph
+)
+{
+    if (ph->pconf && ph->pconf->use_tls) {
+        iiimf_delete_tls_stream(ph->pstream);
+    } else {
+        iiimf_delete_socket_stream(ph->pstream);
+    }
+    ph->pstream = NULL;
+}
+
 IIIMF_status
 iiimcf_connect(
     IIIMCF_handle_rec *ph
@@ -341,23 +443,71 @@ iiimcf_connect(
     if (!ph->data_s) return IIIMF_STATUS_MALLOC;
 
     if (!ph->pstream) {
-	st = iiimf_connect_socket_stream(ph->penv->server_node,
-					 ph->penv->service,
-					 IIIMCF_DEFAULT_TIMEOUT,
-					 &ph->pstream);
-	if (st != IIIMF_STATUS_SUCCESS) return st;
+        IIIMCF_client_conf *pconf;
+        if (ph->pconf) {
+            pconf = ph->pconf;
+            ph->pconf = NULL;
+        } else {
+            pconf = ph->penv->pconf;
+        }
+        for ( ; pconf; pconf = pconf->next) {
+            /* use TLS */
+            if (pconf->use_tls) {
+                /* check whther libiiimp is compiled with TLS support */
+                st = iiimf_tls_supported();
+                if (st != IIIMF_STATUS_SUCCESS) continue;
+
+                st = iiimf_connect_tls_stream(pconf->server_node ? pconf->server_node : NODE_DEFAULT,
+                                              pconf->service ? pconf->service : TLS_DEFAULT,
+                                              IIIMCF_DEFAULT_TIMEOUT,
+                                              &ph->pstream);
+                if (st != IIIMF_STATUS_SUCCESS) continue;
+
+                /* TODO*/
+                iiimf_tls_set_certificate(ph->pstream,
+                                          pconf->cert_file,
+                                          pconf->cert_key,
+                                          pconf->ca_file,
+                                          pconf->ca_path);
+            } else {
+	        st = iiimf_connect_socket_stream(pconf->server_node ? pconf->server_node : NODE_DEFAULT,
+				                 pconf->service ? pconf->service : SERVICE_DEFAULT,
+					         IIIMCF_DEFAULT_TIMEOUT,
+        					 &ph->pstream);
+            }
+
+            if (st == IIIMF_STATUS_SUCCESS) {
+                /* set current configuration to IIIMCF_handle */
+                ph->pconf = pconf;
+                break;
+            }
+        }
     }
+    if (st != IIIMF_STATUS_SUCCESS) {
+        /* finally try to connect to 127.0.0.1:9010 */
+        st = iiimf_connect_socket_stream(NODE_DEFAULT, SERVICE_DEFAULT,
+                                         IIIMCF_DEFAULT_TIMEOUT,
+                                         &ph->pstream);
+    }
+    if (st != IIIMF_STATUS_SUCCESS) return st;
 
     st = iiimcf_create_im_connect_message(ph, &pmes);
     if (st != IIIMF_STATUS_SUCCESS) return st;
 
     st = iiimf_stream_send(ph->pstream, ph->data_s, pmes);
     iiimp_message_delete(ph->data_s, pmes);
-    if (st != IIIMF_STATUS_SUCCESS) return st;
+    if (st != IIIMF_STATUS_SUCCESS) goto reconnect;
 
     st = iiimcf_wait_message(ph, NULL, IM_CONNECT_REPLY, &pmes);
-    if (st != IIIMF_STATUS_SUCCESS) return st;
+    if (st != IIIMF_STATUS_SUCCESS) {
+        if (pmes) iiimp_message_delete(ph->data_s, pmes);
+        goto reconnect;
+    }
+
     ph->im_id = pmes->im_id;
+    st = iiimp_data_s_limit_protocol_version(ph->data_s,
+					     ph->server_protocol_version);
+    if (st != IIIMF_STATUS_SUCCESS) return st;
     st = iiimcf_register_langs(ph, pmes->v.connect_reply.language);
     iiimp_message_delete(ph->data_s, pmes);
 
@@ -371,6 +521,23 @@ iiimcf_connect(
     st = iiimcf_wait_message(ph, NULL, IM_SETIMVALUES_REPLY, NULL);
 
     return st;
+
+reconnect:
+    /* 
+     * in this case, iiimcf connects iiimsf successfully,
+     * but iiimsf refuses the connection.
+     *
+     * so, try to next iiimsf if any.
+     */
+    if (!ph->pconf) return st;
+    if (ph->pconf->next) {
+        /* first, destroy current stream */
+        iiimcf_delete_stream(ph);
+        /* ph->pconf store current configuration */
+        ph->pconf = ph->pconf->next;
+        return iiimcf_connect(ph);
+    }
+    return st;
 }
 
 static IIIMF_status
@@ -383,13 +550,14 @@ iiimcf_hungup(
     /* Should we delete downloaded objects?  */
     /* iiimcf_delete_all_downloaded_objects(ph); */
     if (ph->pstream) {
-	iiimf_delete_socket_stream(ph->pstream);
-	ph->pstream = NULL;
+        iiimcf_delete_stream(ph);
     }
+#if 0
     if (ph->data_s) {
 	iiimp_data_s_delete(ph->data_s);
 	ph->data_s = NULL;
     }
+#endif
 
     return iiimcf_cleanup_context(ph, 0);
 }
@@ -507,6 +675,11 @@ iiimcf_create_handle(
     memset(ppc, 0, sizeof(*ppc) * ph->context_table_size);
     ph->ppcontext_table = ppc;
     ph->im_id = -1;
+
+    /* When not received IM_PROTOCOL_VERSION,
+       we assume the server supports protocol version 2.  */
+    ph->server_protocol_version = 2;
+
     INIT_SYNC_OBJECT(ph->so);
     if (disable_automatic_connection_restoration)
 	ph->disable_automatic_connection_restoration = 1;
@@ -530,27 +703,10 @@ iiimcf_create_handle(
 	return st;
     }
     
-    st = iiimf_connect_socket_stream(ph->penv->server_node,
-				     ph->penv->service,
-				     IIIMCF_DEFAULT_TIMEOUT,
-				     &ph->pstream);
-    if (st != IIIMF_STATUS_SUCCESS) {
-	iiimcf_delete_all_components(ph);
-	iiimcf_delete_client_env(ph->penv);
-	DESTROY_SYNC_OBJECT(ph->so);
-	free(ppc);
-	free(ph);
-	return st;
-    }
-
     st = iiimcf_connect(ph);
     if (st != IIIMF_STATUS_SUCCESS) {
-	iiimcf_delete_all_components(ph);
-	iiimcf_delete_client_env(ph->penv);
-	DESTROY_SYNC_OBJECT(ph->so);
-	free(ppc);
-	free(ph);
-	return st;
+        iiimcf_destroy_handle(ph);
+        return st;
     }
 
     *phandle = ph;
@@ -564,16 +720,27 @@ iiimcf_destroy_handle(
 )
 {
     IIIMCF_handle_rec *ph = (IIIMCF_handle_rec*) handle;
+    int i;
 
     iiimcf_disconnect(ph);
     iiimcf_unregister_langs(ph->num_langs, ph->pplangs);
+    iiimcf_unregister_input_method_list(ph->num_input_methods,
+					ph->ppinput_methods);
+    /* must be before iiimcf_delete_client_env */
+    if (ph->pstream)  iiimcf_delete_stream(ph);
     iiimcf_delete_client_env(ph->penv);
     iiimcf_delete_all_components(ph);
     iiimcf_delete_all_downloaded_objects(ph);
     free_object_descriptor(ph->pobjdesc, ph->object_descriptor_size);
     if (ph->pon_keys) free(ph->pon_keys);
     if (ph->poff_keys) free(ph->poff_keys);
-    if (ph->pstream) iiimf_delete_socket_stream(ph->pstream);
+
+    if (ph->phot_keys) {
+        for (i=0; i < ph->num_hotkeys; ++i) {
+	    if (ph->phot_keys[i].keys) free(ph->phot_keys[i].keys);
+        }
+        free(ph->phot_keys);
+    }
     if (ph->data_s) iiimp_data_s_delete(ph->data_s);
     free(ph->ppcontext_table);
     DESTROY_SYNC_OBJECT(ph->so);
@@ -592,7 +759,7 @@ iiimcf_initialize(
 {
     static int inited;
 
-    if (inited) return 1;
+    if (inited) return IIIMF_STATUS_SUCCESS;
     if (!EIMIL_initialize())
 	return IIIMF_STATUS_FAIL;
     inited = 1;
@@ -603,6 +770,7 @@ iiimcf_initialize(
 IIIMF_status
 iiimcf_finalize()
 {
+    EIMIL_finalize();
     return IIIMF_STATUS_SUCCESS;
 }
 
@@ -652,11 +820,7 @@ iiimcf_get_version_number(
 		!= IIIMCF_MAJOR_VERSION) {
 		return IIIMF_STATUS_ARGUMENT;
 	    }
-	    /* As for IIIMP spec version 2, we don't have any
-	       specification to be informed client of the protocol
-	       version so that currently return the fixed version 2
-	       as a version number! */
-	    *pversion_number = 2;
+	    *pversion_number = iiimp_data_s_get_protocol_version(ph->data_s);
 	} else if (flag & IIIMCF_MINOR_VERSION) {
 	    if ((flag & ~IIIMCF_VERSION_FLAGS_CATEGORY_MASK)
 		!= IIIMCF_MINOR_VERSION) {
@@ -739,6 +903,22 @@ iiimcf_get_object_descriptor_list(
     return IIIMF_STATUS_SUCCESS;
 }
 
+IIIMF_status
+iiimcf_get_im_id(
+    IIIMCF_handle handle,
+    int *pim_id
+)
+{
+    IIIMCF_handle_rec *ph = (IIIMCF_handle_rec*) handle;
+
+    *pim_id = ph->im_id;
+
+    if (ph->im_id < 0)
+	return IIIMF_STATUS_IM_INVALID;
+
+    return IIIMF_STATUS_SUCCESS;
+}
+
 /* Local Variables: */
 /* c-file-style: "iiim-project" */
 /* End: */
