@@ -27,6 +27,8 @@ extern void test_assert(
 
 extern void test_init();
 extern void test_uninit();
+extern void test_show_objects();
+extern void test_show_input_methods();
 
 extern void test_forwardevent(
     IIIMCF_context c,
