@@ -590,12 +590,13 @@ iiimp_inputmethod_descriptor_create(
     IIIMP_data_s *			data_s,
     IIIMP_inputmethod_descriptor **	ptr_ret,
     IIIMP_card16			id,
-    IIIMP_string *			rdun,
+    IIIMP_string *			idname,
     IIIMP_string *			hrn,
-    IIIMP_language *			language)
+    IIIMP_language *			language,
+    IIIMP_string *			rdun)
 {
-    *ptr_ret = iiimp_inputmethod_descriptor_new(data_s, id, rdun, hrn,
-						language);
+    *ptr_ret = iiimp_inputmethod_descriptor_new(data_s, id, idname, hrn,
+						language, rdun);
     return ((NULL == *ptr_ret) ? data_s->status : IIIMP_DATA_NO_ERROR);
 }
 
