@@ -271,6 +271,7 @@ struct iiimp_data_s {
 IIIMP_card16_list *		iiimp_card16_list_unpack(IIIMP_dal);
 IIIMP_card32_list *		iiimp_card32_list_unpack(IIIMP_dal);
 IIIMP_keyevent_list *		iiimp_keyevent_list_unpack(IIIMP_dal);
+HOTKEY_LIST *		iiimp_hotkey_list_unpack(IIIMP_dal);
 IIIMP_string *			iiimp_string_unpack(IIIMP_dal);
 IIIMP_string *			iiimp_string_list_unpack(IIIMP_dal);
 IIIMP_feedback_attr_list *	iiimp_feedback_attr_list_unpack(IIIMP_dal);
@@ -305,190 +306,196 @@ IIIMP_contents *		iiimp_contents_unpack(IIIMP_dal);
 /* prototype of composite data pack function */
 
 extern void	iiimp_card16_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_card16_list *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_card16_list *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_card32_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_card32_list *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_card32_list *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_string_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_string *		m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_string *		m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_string_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_string *		m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_string *		m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_feedback_attr_list_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_feedback_attr_list *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_feedback_attr_list *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_char_with_feedback_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_char_with_feedback *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_char_with_feedback *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_char_with_feedback_list_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_char_with_feedback *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_char_with_feedback *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_operation_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_operation *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_operation *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_operation_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_operation *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_operation *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_annotation_text_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_annotation_value *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_annotation_value *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_annotation_string_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_annotation_value *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_annotation_value *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_annotation_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_annotation *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_annotation *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_annotation_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_annotation *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_annotation *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_text_pack(
-			IIIMP_data_s *	data_s,
-			IIIMP_text *	m,
-			size_t *	nbyte,
-			uchar_t **	ptr);
+    IIIMP_data_s *	data_s,
+    IIIMP_text *	m,
+    size_t *	nbyte,
+    uchar_t **	ptr);
 
 extern void	iiimp_text_list_pack(
-			IIIMP_data_s *	data_s,
-			IIIMP_text *	text,
-			size_t *	nbyte,
-			uchar_t **	ptr);
+    IIIMP_data_s *	data_s,
+    IIIMP_text *	text,
+    size_t *	nbyte,
+    uchar_t **	ptr);
 
 extern void	iiimp_client_descriptor_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_client_descriptor *	cd,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_client_descriptor *	cd,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_object_descriptor_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_object_descriptor *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_object_descriptor *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_object_descriptor_list_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_object_descriptor *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_object_descriptor *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_ccdef_pack(
-			IIIMP_data_s *	data_s,
-			IIIMP_ccdef *	m,
-			size_t *	nbyte,
-			uchar_t **	ptr);
+    IIIMP_data_s *	data_s,
+    IIIMP_ccdef *	m,
+    size_t *	nbyte,
+    uchar_t **	ptr);
 
 extern void	iiimp_language_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_language *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_language *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_language_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_language *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_language *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_jarfile_object_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_jarfile_object *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_jarfile_object *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_inputmethod_descriptor_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_inputmethod_descriptor *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_inputmethod_descriptor *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_inputmethod_descriptor_list_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_inputmethod_descriptor *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_inputmethod_descriptor *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_binaryfile_object_pack(
-			IIIMP_data_s *			data_s,
-			IIIMP_binaryfile_object *	m,
-			size_t *			nbyte,
-			uchar_t **			ptr);
+    IIIMP_data_s *			data_s,
+    IIIMP_binaryfile_object *	m,
+    size_t *			nbyte,
+    uchar_t **			ptr);
 
 extern void	iiimp_imattribute_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_imattribute *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_imattribute *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_imattribute_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_imattribute *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_imattribute *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_icattribute_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_icattribute *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_icattribute *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_icattribute_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_icattribute *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_icattribute *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_keyevent_list_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_keyevent_list *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_keyevent_list *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
+
+extern void	iiimp_hotkey_list_pack(
+    IIIMP_data_s *		data_s,
+    HOTKEY_LIST *	        m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 extern void	iiimp_contents_pack(
-			IIIMP_data_s *		data_s,
-			IIIMP_contents *	m,
-			size_t *		nbyte,
-			uchar_t **		ptr);
+    IIIMP_data_s *		data_s,
+    IIIMP_contents *	m,
+    size_t *		nbyte,
+    uchar_t **		ptr);
 
 
 extern void	iiimp_message_header_print(IIIMP_data_s *	data_s,
@@ -515,16 +522,16 @@ extern void	iiimp_feedback_attr_print(IIIMP_data_s *	data_s,
 					  IIIMP_feedback_attr *	m);
 
 extern void	iiimp_feedback_attr_list_print(
-			IIIMP_data_s *			data_s,
-			IIIMP_feedback_attr_list *	m);
+    IIIMP_data_s *			data_s,
+    IIIMP_feedback_attr_list *	m);
 
 extern void	iiimp_char_with_feedback_print(
-			IIIMP_data_s *			data_s,
-			IIIMP_char_with_feedback *	m);
+    IIIMP_data_s *			data_s,
+    IIIMP_char_with_feedback *	m);
 
 extern void	iiimp_char_with_feedback_list_print(
-			IIIMP_data_s *			data_s,
-			IIIMP_char_with_feedback *	m);
+    IIIMP_data_s *			data_s,
+    IIIMP_char_with_feedback *	m);
 
 extern void	iiimp_operation_print(IIIMP_data_s *	data_s,
 				      IIIMP_operation *	m);
@@ -546,8 +553,8 @@ extern void	iiimp_object_descriptor_print(IIIMP_data_s *		data_s,
 					      IIIMP_object_descriptor *	m);
 
 extern void	iiimp_object_descriptor_list_print(
-			IIIMP_data_s *			data_s,
-			IIIMP_object_descriptor *	m);
+    IIIMP_data_s *			data_s,
+    IIIMP_object_descriptor *	m);
 
 extern void	iiimp_ccdef_print(IIIMP_data_s * data_s, IIIMP_ccdef * m);
 
@@ -560,12 +567,12 @@ extern void	iiimp_binaryfile_object_print(IIIMP_data_s *		data_s,
 					      IIIMP_binaryfile_object *	m);
 
 extern void	iiimp_inputmethod_descriptor_print(
-			IIIMP_data_s *			data_s,
-			IIIMP_inputmethod_descriptor *	m);
+    IIIMP_data_s *			data_s,
+    IIIMP_inputmethod_descriptor *	m);
 
 extern void	iiimp_inputmethod_descriptor_list_print(
-			IIIMP_data_s *			data_s,
-			IIIMP_inputmethod_descriptor *	m);
+    IIIMP_data_s *			data_s,
+    IIIMP_inputmethod_descriptor *	m);
 
 extern void	iiimp_imattribute_print(IIIMP_data_s *		data_s,
 					IIIMP_imattribute *	m);
@@ -616,6 +623,15 @@ extern void	iiimp_register_trigger_keys_print(IIIMP_data_s *	data_s,
 extern void	iiimp_trigger_notify_print(IIIMP_data_s *	data_s,
 					   IIIMP_message *	m);
 
+extern void	iiimp_register_hotkeys_print(IIIMP_data_s *	data_s,
+					     IIIMP_message *	m);
+
+extern void	iiimp_hotkey_notify_print(IIIMP_data_s *	data_s,
+					  IIIMP_message *	m);
+
+extern void	iiimp_hotkey_state_notify_print(IIIMP_data_s *	data_s,
+					  IIIMP_message *	m);
+
 extern void	iiimp_setimvalues_print(IIIMP_data_s *	data_s,
 					IIIMP_message *	m);
 
@@ -649,12 +665,12 @@ extern void	iiimp_forward_event_print(IIIMP_data_s *	data_s,
 					  IIIMP_message *	m);
 
 extern void	iiimp_forward_event_with_operations_print(
-			IIIMP_data_s *	data_s,
-			IIIMP_message *	m);
+    IIIMP_data_s *	data_s,
+    IIIMP_message *	m);
 
 extern void	iiimp_forward_event_with_operations_reply_print(
-			IIIMP_data_s *	data_s,
-			IIIMP_message *	m);
+    IIIMP_data_s *	data_s,
+    IIIMP_message *	m);
 
 extern void	iiimp_commit_string_print(IIIMP_data_s *	data_s,
 					  IIIMP_message *	m);
@@ -672,8 +688,8 @@ extern void	iiimp_protocol_version_print(IIIMP_data_s *	data_s,
 					     IIIMP_message *	m);
 
 extern IIIMP_card16	iiimp_attribute_id_search(
-				IIIMP_data_s *	data_s,
-				IIIMP_card16	id_dynamic);
+    IIIMP_data_s *	data_s,
+    IIIMP_card16	id_dynamic);
 
 extern void	iiimp_attribute_id_update(IIIMP_data_s *	data_s,
 					  IIIMP_card16		id_predefined,
