@@ -21,7 +21,7 @@ static IIIMP_message_delete_func message_delete_func[] = {
 	iiimp_simple_delete,		/* IM_DISCONNECT */
 	iiimp_simple_delete,		/* IM_DISCONNECT_REPLY */
 	iiimp_register_trigger_keys_delete,
-					/* IM_REGISTER_TRIGGER_KEYS */
+	/* IM_REGISTER_TRIGGER_KEYS */
 	iiimp_trigger_notify_delete,	/* IM_TRIGGER_NOTIFY */
 	iiimp_simple_delete,		/* IM_TRIGGER_NOTIFY_REPLY */
 	iiimp_setimvalues_delete,	/* IM_SETIMVALUES */
@@ -33,13 +33,13 @@ static IIIMP_message_delete_func message_delete_func[] = {
 	iiimp_simple_delete,		/* IM_FORWARD_EVENT_REPLY */
 	iiimp_commit_string_delete,	/* IM_COMMIT_STRING */
 	iiimp_forward_event_with_operations_delete,
-					/* IM_FORWARD_EVENT_WITH_OPERATIONS */
+	/* IM_FORWARD_EVENT_WITH_OPERATIONS */
 	iiimp_forward_event_with_operations_reply_delete,
-				  /* IM_FORWARD_EVENT_WITH_OPERATIONS_REPLY */
-
-	iiimp_invalid_message_delete,	/* 17 */
-	iiimp_invalid_message_delete,	/* 18 */
-	iiimp_invalid_message_delete,	/* 19 */
+	/* IM_FORWARD_EVENT_WITH_OPERATIONS_REPLY */
+	iiimp_register_hotkeys_delete,
+	/* IM_REGISTER_HOTKEYS */
+	iiimp_hotkey_notify_delete,	/* IM_HOTKEY_NOTIFY */
+	iiimp_simple_delete,		/* IM_HOTKEY_NOTIFY_REPLY */
 
 	iiimp_createic_delete,		/* IM_CREATEIC */
 	iiimp_simple_delete,		/* IM_CREATEIC_REPLY */
@@ -56,8 +56,8 @@ static IIIMP_message_delete_func message_delete_func[] = {
 	iiimp_simple_delete,		/* IM_RESETIC */
 	iiimp_simple_delete,		/* IM_RESETIC_REPLY */
 
-	iiimp_invalid_message_delete,	/* 34 */
-	iiimp_invalid_message_delete,	/* 35 */
+	iiimp_hotkey_state_notify_delete,	/* IM_HOTKEY_STATE_NOTIFY */
+	iiimp_simple_delete,	/* IM_HOTKEY_STATE_NOTIFY_REPLY */
 	iiimp_invalid_message_delete,	/* 36 */
 	iiimp_invalid_message_delete,	/* 37 */
 	iiimp_invalid_message_delete,	/* 38 */
@@ -65,7 +65,7 @@ static IIIMP_message_delete_func message_delete_func[] = {
 
 	iiimp_simple_delete,		/* IM_PREEDIT_START */
 	iiimp_preedit_start_reply_delete,
-					/* IM_PREEDIT_START_REPLY */
+	/* IM_PREEDIT_START_REPLY */
 	iiimp_preedit_draw_delete,	/* IM_PREEDIT_DRAW */
 	iiimp_simple_delete,		/* IM_PREEDIT_DRAW_REPLY */
 	iiimp_invalid_message_delete,	/* 44 */
@@ -99,14 +99,14 @@ static IIIMP_message_delete_func message_delete_func[] = {
 	iiimp_invalid_message_delete,	/* 69 */
 
 	iiimp_lookup_choice_start_delete,
-					/* IM_LOOKUP_CHOICE_START */
+	/* IM_LOOKUP_CHOICE_START */
 	iiimp_simple_delete,		/* IM_LOOKUP_CHOICE_START_REPLY */
 	iiimp_lookup_choice_draw_delete,/* IM_LOOKUP_CHOICE_DRAW */
 	iiimp_simple_delete,		/* IM_LOOKUP_CHOICE_DRAW_REPLY */
 	iiimp_simple_delete,		/* IM_LOOKUP_CHOICE_DONE */
 	iiimp_simple_delete,		/* IM_LOOKUP_CHOICE_DONE_REPLY */
 	iiimp_lookup_choice_process_delete,
-					/* IM_LOOKUP_CHOICE_PROCESS */
+	/* IM_LOOKUP_CHOICE_PROCESS */
 	iiimp_simple_delete,		/* IM_LOOKUP_CHOICE_PROCESS_REPLY */
 
 	iiimp_invalid_message_delete,	/* 78 */
@@ -130,9 +130,8 @@ static IIIMP_message_delete_func message_delete_func[] = {
 	iiimp_aux_simple_delete,	/* IM_AUX_DONE_REPLY */
 	iiimp_aux_value_delete,		/* IM_AUX_SETVALUES */
 	iiimp_aux_simple_delete,	/* IM_AUX_SETVALUES_REPLY */
-
-	iiimp_invalid_message_delete,	/* 98 */
-	iiimp_invalid_message_delete,	/* 99 */
+	iiimp_aux_value_delete,		/* IM_AUX_GETVALUES */
+	iiimp_aux_value_delete,		/* IM_AUX_GETVALUES_REPLY */
 
 	iiimp_protocol_version_delete,	/* IM_PROTOCOL_VERSION */
 
@@ -183,203 +182,224 @@ iiimp_message_pack(
     uchar_t *	p;
 
     switch (m->opcode) {
-    case IM_DISCONNECT:
-    case IM_DISCONNECT_REPLY:
-    case IM_TRIGGER_NOTIFY_REPLY:
-    case IM_SETIMVALUES_REPLY:
-    case IM_FORWARD_EVENT_REPLY:
-    case IM_CREATEIC_REPLY:
-    case IM_DESTROYIC:
-    case IM_DESTROYIC_REPLY:
-    case IM_SETICVALUES_REPLY:
-    case IM_SETICFOCUS:
-    case IM_SETICFOCUS_REPLY:
-    case IM_UNSETICFOCUS:
-    case IM_UNSETICFOCUS_REPLY:
-    case IM_RESETIC:
-    case IM_RESETIC_REPLY:
-    case IM_PREEDIT_START:
-    case IM_PREEDIT_DRAW_REPLY:
-    case IM_PREEDIT_DONE:
-    case IM_PREEDIT_DONE_REPLY:
-    case IM_STATUS_START:
-    case IM_STATUS_START_REPLY:
-    case IM_STATUS_DRAW_REPLY:
-    case IM_STATUS_DONE:
-    case IM_STATUS_DONE_REPLY:
-    case IM_LOOKUP_CHOICE_START_REPLY:
-    case IM_LOOKUP_CHOICE_DRAW_REPLY:
-    case IM_LOOKUP_CHOICE_DONE:
-    case IM_LOOKUP_CHOICE_DONE_REPLY:
-    case IM_LOOKUP_CHOICE_PROCESS_REPLY:
-	p = iiimp_simple_pack(data_s, m->opcode, m->im_id, m->ic_id, nbyte);
-	break;
+      case IM_DISCONNECT:
+      case IM_DISCONNECT_REPLY:
+      case IM_TRIGGER_NOTIFY_REPLY:
+      case IM_HOTKEY_NOTIFY_REPLY:
+      case IM_SETIMVALUES_REPLY:
+      case IM_FORWARD_EVENT_REPLY:
+      case IM_CREATEIC_REPLY:
+      case IM_DESTROYIC:
+      case IM_DESTROYIC_REPLY:
+      case IM_SETICVALUES_REPLY:
+      case IM_SETICFOCUS:
+      case IM_SETICFOCUS_REPLY:
+      case IM_UNSETICFOCUS:
+      case IM_UNSETICFOCUS_REPLY:
+      case IM_RESETIC:
+      case IM_RESETIC_REPLY:
+      case IM_PREEDIT_START:
+      case IM_PREEDIT_DRAW_REPLY:
+      case IM_PREEDIT_DONE:
+      case IM_PREEDIT_DONE_REPLY:
+      case IM_STATUS_START:
+      case IM_STATUS_START_REPLY:
+      case IM_STATUS_DRAW_REPLY:
+      case IM_STATUS_DONE:
+      case IM_STATUS_DONE_REPLY:
+      case IM_LOOKUP_CHOICE_START_REPLY:
+      case IM_LOOKUP_CHOICE_DRAW_REPLY:
+      case IM_LOOKUP_CHOICE_DONE:
+      case IM_LOOKUP_CHOICE_DONE_REPLY:
+      case IM_LOOKUP_CHOICE_PROCESS_REPLY:
+       p = iiimp_simple_pack(data_s, m->opcode, m->im_id, m->ic_id, nbyte);
+       break;
 
-    case IM_CONNECT:
-	p = iiimp_connect_pack(data_s, m->v.connect.user_name,
-			       m->v.connect.auth, nbyte);
-	break;
+      case IM_CONNECT:
+       p = iiimp_connect_pack(data_s, m->v.connect.user_name,
+			      m->v.connect.auth, nbyte);
+       break;
 
-    case IM_CONNECT_REPLY:
-	p = iiimp_connect_reply_pack(data_s, m->im_id,
-				     m->v.connect_reply.language, nbyte);
-	break;
+      case IM_CONNECT_REPLY:
+       p = iiimp_connect_reply_pack(data_s, m->im_id,
+				    m->v.connect_reply.language, nbyte);
+       break;
 
-    case IM_REGISTER_TRIGGER_KEYS:
-	p = iiimp_register_trigger_keys_pack(
-	    data_s, m->im_id,
-	    m->v.register_trigger_keys.trigger_on,
-	    m->v.register_trigger_keys.trigger_off, nbyte);
-	break;
+      case IM_REGISTER_TRIGGER_KEYS:
+       p = iiimp_register_trigger_keys_pack(
+	   data_s, m->im_id,
+	   m->v.register_trigger_keys.trigger_on,
+	   m->v.register_trigger_keys.trigger_off, nbyte);
+       break;
 
-    case IM_TRIGGER_NOTIFY:
-	p = iiimp_trigger_notify_pack(data_s, m->im_id, m->ic_id,
-				      m->v.trigger_notify.flag, nbyte);
-	break;
+      case IM_TRIGGER_NOTIFY:
+       p = iiimp_trigger_notify_pack(data_s, m->im_id, m->ic_id,
+				     m->v.trigger_notify.flag, nbyte);
+       break;
 
-    case IM_SETIMVALUES:
-	p = iiimp_setimvalues_pack(data_s, m->im_id,
-				   m->v.setimvalues.attr_list, nbyte);
-	break;
+      case IM_REGISTER_HOTKEYS:
+       p = iiimp_register_hotkeys_pack(
+	   data_s, m->im_id, m->v.register_hotkeys.flag,
+	   m->v.register_hotkeys.hotkeys, nbyte);
+       break;
 
-    case IM_GETIMVALUES:
-	p = iiimp_getimvalues_pack(data_s, m->im_id,
-				   m->v.getimvalues.attr_list, nbyte);
-	break;
+      case IM_HOTKEY_NOTIFY:
+       p = iiimp_hotkey_notify_pack(data_s, m->im_id, m->ic_id,
+				    m->v.hotkey_notify.hotkey_id, 
+				    m->v.hotkey_notify.index,nbyte);
+       break;
 
-    case IM_GETIMVALUES_REPLY:
-	p = iiimp_getimvalues_reply_pack(data_s, m->im_id,
-					 m->v.getimvalues_reply.attr_list,
-					 nbyte);
-	break;
+      case IM_HOTKEY_STATE_NOTIFY:
+       p = iiimp_hotkey_state_notify_pack(data_s, m->im_id, m->ic_id,
+				    m->v.hotkey_state_notify.hotkey_id,
+				    m->v.hotkey_state_notify.current_state_flag,nbyte);
+       break;
 
-    case IM_FORWARD_EVENT:
-	p = iiimp_forward_event_pack(data_s, m->im_id, m->ic_id,
-				     m->v.forward_event.contents,
-				     nbyte);
-	break;
+      case IM_SETIMVALUES:
+       p = iiimp_setimvalues_pack(data_s, m->im_id,
+				  m->v.setimvalues.attr_list, nbyte);
+       break;
 
-    case IM_COMMIT_STRING:
-	p = iiimp_commit_string_pack(data_s, m->im_id, m->ic_id,
-				     m->v.commit_string.contents,
-				     nbyte);
-	break;
+      case IM_GETIMVALUES:
+       p = iiimp_getimvalues_pack(data_s, m->im_id,
+				  m->v.getimvalues.attr_list, nbyte);
+       break;
 
-    case IM_FORWARD_EVENT_WITH_OPERATIONS:
-	p = iiimp_forward_event_with_operations_pack(
-	    data_s, m->im_id, m->ic_id,
-	    m->v.forward_event_with_operations.contents,
-	    m->v.forward_event_with_operations.operation, nbyte);
-	break;
-
-    case IM_FORWARD_EVENT_WITH_OPERATIONS_REPLY:
-	p = iiimp_forward_event_with_operations_reply_pack(
-	    data_s, m->im_id, m->ic_id,
-	    m->v.forward_event_with_operations_reply.operation,
-	    nbyte);
-	break;
-
-    case IM_CREATEIC:
-	p = iiimp_createic_pack(data_s, m->im_id,
-				m->v.createic.attr_list, nbyte);
-	break;
-
-    case IM_SETICVALUES:
-	p = iiimp_seticvalues_pack(data_s, m->im_id, m->ic_id,
-				   m->v.seticvalues.attr_list, nbyte);
-	break;
-
-    case IM_GETICVALUES:
-	p = iiimp_geticvalues_pack(data_s, m->im_id, m->ic_id,
-				   m->v.geticvalues.attr_list, nbyte);
-	break;
-
-    case IM_GETICVALUES_REPLY:
-	p = iiimp_geticvalues_reply_pack(data_s, m->im_id, m->ic_id,
-					 m->v.geticvalues_reply.attr_list,
-					 nbyte);
-	break;
-
-    case IM_PREEDIT_START_REPLY:
-	p = iiimp_preedit_start_reply_pack(
-	    data_s, m->im_id, m->ic_id,
-	    m->v.preedit_start_reply.maximum_length, nbyte);
-	break;
-
-    case IM_PREEDIT_DRAW:
-	p = iiimp_preedit_draw_pack(data_s, m->im_id, m->ic_id,
-				    m->v.preedit_draw.caret,
-				    m->v.preedit_draw.change_first,
-				    m->v.preedit_draw.change_length,
-				    m->v.preedit_draw.preedit,
-				    nbyte);
-	break;
-
-    case IM_STATUS_DRAW:
-	p = iiimp_status_draw_pack(data_s, m->im_id, m->ic_id,
-				   m->v.status_draw.status, nbyte);
-	break;
-
-    case IM_LOOKUP_CHOICE_START:
-	p = iiimp_lookup_choice_start_pack(
-	    data_s, m->im_id, m->ic_id,
-	    m->v.lookup_choice_start.master,
-	    m->v.lookup_choice_start.choice_per_window,
-	    m->v.lookup_choice_start.rows,
-	    m->v.lookup_choice_start.columns,
-	    m->v.lookup_choice_start.direction,
-	    m->v.lookup_choice_start.label_owner,
-	    nbyte);
-	break;
-
-    case IM_LOOKUP_CHOICE_DRAW:
-	p = iiimp_lookup_choice_draw_pack(data_s, m->im_id, m->ic_id,
-					  m->v.lookup_choice_draw.first,
-					  m->v.lookup_choice_draw.last,
-					  m->v.lookup_choice_draw.current,
-					  m->v.lookup_choice_draw.choice,
-					  m->v.lookup_choice_draw.index_label,
-					  m->v.lookup_choice_draw.title, nbyte);
-	break;
-
-    case IM_LOOKUP_CHOICE_PROCESS:
-	p = iiimp_lookup_choice_process_pack(data_s, m->im_id, m->ic_id,
-					     m->v.lookup_choice_process.type,
-					     m->v.lookup_choice_process.value,
-					     nbyte);
-	break;
-
-    case IM_AUX_START:
-    case IM_AUX_START_REPLY:
-    case IM_AUX_DRAW_REPLY:
-    case IM_AUX_DONE_REPLY:
-    case IM_AUX_DONE:
-    case IM_AUX_SETVALUES_REPLY:
-	p = iiimp_aux_simple_pack(data_s, m->opcode, m->im_id, m->ic_id,
-				  m->v.aux_simple.class_index,
-				  m->v.aux_simple.input_method_name,
-				  nbyte);
-	break;
-
-    case IM_AUX_DRAW:
-    case IM_AUX_SETVALUES:
-	p = iiimp_aux_value_pack(data_s, m->opcode, m->im_id, m->ic_id,
-				 m->v.aux_value.class_index,
-				 m->v.aux_value.input_method_name,
-				 m->v.aux_value.integer_value,
-				 m->v.aux_value.string_value,
-				 nbyte);
-	break;
-
-    case IM_PROTOCOL_VERSION:
-        p = iiimp_protocol_version_pack(data_s, m->im_id,
-					m->v.protocol_version.number,
+      case IM_GETIMVALUES_REPLY:
+       p = iiimp_getimvalues_reply_pack(data_s, m->im_id,
+					m->v.getimvalues_reply.attr_list,
 					nbyte);
-	break;
+       break;
 
-    default:
-	p = NULL;
-	data_s->status = IIIMP_DATA_INVALID;
+      case IM_FORWARD_EVENT:
+       p = iiimp_forward_event_pack(data_s, m->im_id, m->ic_id,
+				    m->v.forward_event.contents,
+				    nbyte);
+       break;
+
+      case IM_COMMIT_STRING:
+       p = iiimp_commit_string_pack(data_s, m->im_id, m->ic_id,
+				    m->v.commit_string.contents,
+				    nbyte);
+       break;
+
+      case IM_FORWARD_EVENT_WITH_OPERATIONS:
+       p = iiimp_forward_event_with_operations_pack(
+	   data_s, m->im_id, m->ic_id,
+	   m->v.forward_event_with_operations.contents,
+	   m->v.forward_event_with_operations.operation, nbyte);
+       break;
+
+      case IM_FORWARD_EVENT_WITH_OPERATIONS_REPLY:
+       p = iiimp_forward_event_with_operations_reply_pack(
+	   data_s, m->im_id, m->ic_id,
+	   m->v.forward_event_with_operations_reply.operation,
+	   nbyte);
+       break;
+
+      case IM_CREATEIC:
+       p = iiimp_createic_pack(data_s, m->im_id,
+			       m->v.createic.attr_list, nbyte);
+       break;
+
+      case IM_SETICVALUES:
+       p = iiimp_seticvalues_pack(data_s, m->im_id, m->ic_id,
+				  m->v.seticvalues.attr_list, nbyte);
+       break;
+
+      case IM_GETICVALUES:
+       p = iiimp_geticvalues_pack(data_s, m->im_id, m->ic_id,
+				  m->v.geticvalues.attr_list, nbyte);
+       break;
+
+      case IM_GETICVALUES_REPLY:
+       p = iiimp_geticvalues_reply_pack(data_s, m->im_id, m->ic_id,
+					m->v.geticvalues_reply.attr_list,
+					nbyte);
+       break;
+
+      case IM_PREEDIT_START_REPLY:
+       p = iiimp_preedit_start_reply_pack(
+	   data_s, m->im_id, m->ic_id,
+	   m->v.preedit_start_reply.maximum_length, nbyte);
+       break;
+
+      case IM_PREEDIT_DRAW:
+       p = iiimp_preedit_draw_pack(data_s, m->im_id, m->ic_id,
+				   m->v.preedit_draw.caret,
+				   m->v.preedit_draw.change_first,
+				   m->v.preedit_draw.change_length,
+				   m->v.preedit_draw.preedit,
+				   nbyte);
+       break;
+
+      case IM_STATUS_DRAW:
+       p = iiimp_status_draw_pack(data_s, m->im_id, m->ic_id,
+				  m->v.status_draw.status, nbyte);
+       break;
+
+      case IM_LOOKUP_CHOICE_START:
+       p = iiimp_lookup_choice_start_pack(
+	   data_s, m->im_id, m->ic_id,
+	   m->v.lookup_choice_start.master,
+	   m->v.lookup_choice_start.choice_per_window,
+	   m->v.lookup_choice_start.rows,
+	   m->v.lookup_choice_start.columns,
+	   m->v.lookup_choice_start.direction,
+	   m->v.lookup_choice_start.label_owner,
+	   nbyte);
+       break;
+
+      case IM_LOOKUP_CHOICE_DRAW:
+       p = iiimp_lookup_choice_draw_pack(data_s, m->im_id, m->ic_id,
+					 m->v.lookup_choice_draw.first,
+					 m->v.lookup_choice_draw.last,
+					 m->v.lookup_choice_draw.current,
+					 m->v.lookup_choice_draw.choice,
+					 m->v.lookup_choice_draw.index_label,
+					 m->v.lookup_choice_draw.title, nbyte);
+       break;
+
+      case IM_LOOKUP_CHOICE_PROCESS:
+       p = iiimp_lookup_choice_process_pack(data_s, m->im_id, m->ic_id,
+					    m->v.lookup_choice_process.type,
+					    m->v.lookup_choice_process.value,
+					    nbyte);
+       break;
+
+      case IM_AUX_START:
+      case IM_AUX_START_REPLY:
+      case IM_AUX_DRAW_REPLY:
+      case IM_AUX_DONE_REPLY:
+      case IM_AUX_DONE:
+      case IM_AUX_SETVALUES_REPLY:
+       p = iiimp_aux_simple_pack(data_s, m->opcode, m->im_id, m->ic_id,
+				 m->v.aux_simple.class_index,
+				 m->v.aux_simple.input_method_name,
+				 nbyte);
+       break;
+
+      case IM_AUX_DRAW:
+      case IM_AUX_SETVALUES:
+      case IM_AUX_GETVALUES:
+      case IM_AUX_GETVALUES_REPLY:
+       p = iiimp_aux_value_pack(data_s, m->opcode, m->im_id, m->ic_id,
+				m->v.aux_value.class_index,
+				m->v.aux_value.input_method_name,
+				m->v.aux_value.integer_value,
+				m->v.aux_value.string_value,
+				nbyte);
+       break;
+
+      case IM_PROTOCOL_VERSION:
+       p = iiimp_protocol_version_pack(data_s, m->im_id,
+				       m->v.protocol_version.number,
+				       nbyte);
+       break;
+
+      default:
+       p = NULL;
+       data_s->status = IIIMP_DATA_INVALID;
     }
 
     return p;
@@ -394,7 +414,7 @@ static IIIMP_message_unpack_func message_unpack_func[] = {
 	iiimp_simple_im_unpack,		/* IM_DISCONNECT */
 	iiimp_simple_im_unpack,		/* IM_DISCONNECT_REPLY */
 	iiimp_register_trigger_keys_unpack,
-					/* IM_REGISTER_TRIGGER_KEYS */
+	/* IM_REGISTER_TRIGGER_KEYS */
 	iiimp_trigger_notify_unpack,	/* IM_TRIGGER_NOTIFY */
 	iiimp_simple_unpack,		/* IM_TRIGGER_NOTIFY_REPLY */
 	iiimp_setimvalues_unpack,	/* IM_SETIMVALUES */
@@ -406,13 +426,14 @@ static IIIMP_message_unpack_func message_unpack_func[] = {
 	iiimp_simple_unpack,		/* IM_FORWARD_EVENT_REPLY */
 	iiimp_commit_string_unpack,	/* IM_COMMIT_STRING */
 	iiimp_forward_event_with_operations_unpack,
-					/* IM_FORWARD_EVENT_WITH_OPERATIONS */
+	/* IM_FORWARD_EVENT_WITH_OPERATIONS */
 	iiimp_forward_event_with_operations_reply_unpack,
-				  /* IM_FORWARD_EVENT_WITH_OPERATIONS_REPLY */
+	/* IM_FORWARD_EVENT_WITH_OPERATIONS_REPLY */
 
-	iiimp_invalid_message_unpack,	/* 17 */
-	iiimp_invalid_message_unpack,	/* 18 */
-	iiimp_invalid_message_unpack,	/* 19 */
+	iiimp_register_hotkeys_unpack,
+	/* IM_REGISTER_HOTKEYS */
+	iiimp_hotkey_notify_unpack,	/* IM_HOTKEY_NOTIFY */
+	iiimp_simple_unpack,		/* IM_HOTKEY_NOTIFY_REPLY */
 
 	iiimp_createic_unpack,		/* IM_CREATEIC */
 	iiimp_simple_unpack,		/* IM_CREATEIC_REPLY */
@@ -429,8 +450,8 @@ static IIIMP_message_unpack_func message_unpack_func[] = {
 	iiimp_simple_unpack,		/* IM_RESETIC */
 	iiimp_simple_unpack,		/* IM_RESETIC_REPLY */
 
-	iiimp_invalid_message_unpack,	/* 34 */
-	iiimp_invalid_message_unpack,	/* 35 */
+	iiimp_hotkey_state_notify_unpack,	/* IM_HOTKEY_STATE_NOTIFY */
+	iiimp_simple_unpack,	/* IM_HOTKEY_STATE_NOTIFY_REPLY */
 	iiimp_invalid_message_unpack,	/* 36 */
 	iiimp_invalid_message_unpack,	/* 37 */
 	iiimp_invalid_message_unpack,	/* 38 */
@@ -438,7 +459,7 @@ static IIIMP_message_unpack_func message_unpack_func[] = {
 
 	iiimp_simple_unpack,		/* IM_PREEDIT_START */
 	iiimp_preedit_start_reply_unpack,
-					/* IM_PREEDIT_START_REPLY */
+	/* IM_PREEDIT_START_REPLY */
 	iiimp_preedit_draw_unpack,	/* IM_PREEDIT_DRAW */
 	iiimp_simple_unpack,		/* IM_PREEDIT_DRAW_REPLY */
 	iiimp_invalid_message_unpack,	/* 44 */
@@ -472,14 +493,14 @@ static IIIMP_message_unpack_func message_unpack_func[] = {
 	iiimp_invalid_message_unpack,	/* 69 */
 
 	iiimp_lookup_choice_start_unpack,
-					/* IM_LOOKUP_CHOICE_START */
+	/* IM_LOOKUP_CHOICE_START */
 	iiimp_simple_unpack,		/* IM_LOOKUP_CHOICE_START_REPLY */
 	iiimp_lookup_choice_draw_unpack,/* IM_LOOKUP_CHOICE_DRAW */
 	iiimp_simple_unpack,		/* IM_LOOKUP_CHOICE_DRAW_REPLY */
 	iiimp_simple_unpack,		/* IM_LOOKUP_CHOICE_DONE */
 	iiimp_simple_unpack,		/* IM_LOOKUP_CHOICE_DONE_REPLY */
 	iiimp_lookup_choice_process_unpack,
-					/* IM_LOOKUP_CHOICE_PROCESS */
+	/* IM_LOOKUP_CHOICE_PROCESS */
 	iiimp_simple_unpack,		/* IM_LOOKUP_CHOICE_PROCESS_REPLY */
 
 	iiimp_invalid_message_unpack,	/* 78 */
@@ -503,9 +524,8 @@ static IIIMP_message_unpack_func message_unpack_func[] = {
 	iiimp_aux_simple_unpack,	/* IM_AUX_DONE_REPLY */
 	iiimp_aux_value_unpack,		/* IM_AUX_SETVALUES */
 	iiimp_aux_simple_unpack,	/* IM_AUX_SETVALUES_REPLY */
-
-	iiimp_invalid_message_unpack,	/* 98 */
-	iiimp_invalid_message_unpack,	/* 99 */
+	iiimp_aux_value_unpack,		/* IM_AUX_GETVALUES */
+	iiimp_aux_value_unpack,		/* IM_AUX_GETVALUES_REPLY */
 
 	iiimp_protocol_version_unpack,	/* IM_PROTOCOL_VERSION */
 
@@ -581,7 +601,7 @@ static IIIMP_message_print_func message_print_func[] = {
 	iiimp_simple_im_print,		/* IM_DISCONNECT */
 	iiimp_simple_im_print,		/* IM_DISCONNECT_REPLY */
 	iiimp_register_trigger_keys_print,
-					/* IM_REGISTER_TRIGGER_KEYS */
+	/* IM_REGISTER_TRIGGER_KEYS */
 	iiimp_trigger_notify_print,	/* IM_TRIGGER_NOTIFY */
 	iiimp_simple_print,		/* IM_TRIGGER_NOTIFY_REPLY */
 	iiimp_setimvalues_print,	/* IM_SETIMVALUES */
@@ -593,13 +613,14 @@ static IIIMP_message_print_func message_print_func[] = {
 	iiimp_simple_print,		/* IM_FORWARD_EVENT_REPLY */
 	iiimp_commit_string_print,	/* IM_COMMIT_STRING */
 	iiimp_forward_event_with_operations_print,
-					/* IM_FORWARD_EVENT_WITH_OPERATIONS */
+	/* IM_FORWARD_EVENT_WITH_OPERATIONS */
 	iiimp_forward_event_with_operations_reply_print,
-					/* IM_FORWARD_EVENT_WITH_OPERATIONS_REPLY */
+	/* IM_FORWARD_EVENT_WITH_OPERATIONS_REPLY */
 
-	iiimp_invalid_message_print,	/* 17 */
-	iiimp_invalid_message_print,	/* 18 */
-	iiimp_invalid_message_print,	/* 19 */
+	iiimp_register_hotkeys_print,
+	/* IM_REGISTER_HOTKEYS */
+	iiimp_hotkey_notify_print,	/* IM_HOTKEY_NOTIFY */
+	iiimp_simple_print,		/* IM_HOTKEY_NOTIFY_REPLY */
 
 	iiimp_createic_print,		/* IM_CREATEIC */
 	iiimp_simple_print,		/* IM_CREATEIC_REPLY */
@@ -616,8 +637,8 @@ static IIIMP_message_print_func message_print_func[] = {
 	iiimp_simple_print,		/* IM_RESETIC */
 	iiimp_simple_print,		/* IM_RESETIC_REPLY */
 
-	iiimp_invalid_message_print,	/* 34 */
-	iiimp_invalid_message_print,	/* 35 */
+	iiimp_hotkey_state_notify_print,	/* IM_HOTKEY_STATE_NOTIFY */
+	iiimp_simple_print,	/* IM_HOTKEY_STATE_NOTIFY_REPLY */
 	iiimp_invalid_message_print,	/* 36 */
 	iiimp_invalid_message_print,	/* 37 */
 	iiimp_invalid_message_print,	/* 38 */
@@ -664,7 +685,7 @@ static IIIMP_message_print_func message_print_func[] = {
 	iiimp_simple_print,		/* IM_LOOKUP_CHOICE_DONE */
 	iiimp_simple_print,		/* IM_LOOKUP_CHOICE_DONE_REPLY */
 	iiimp_lookup_choice_process_print,
-					/* IM_LOOKUP_CHOICE_PROCESS */
+	/* IM_LOOKUP_CHOICE_PROCESS */
 	iiimp_simple_print,		/* IM_LOOKUP_CHOICE_PROCESS_REPLY */
 
 	iiimp_invalid_message_print,	/* 78 */
@@ -688,9 +709,8 @@ static IIIMP_message_print_func message_print_func[] = {
 	iiimp_aux_simple_print,		/* IM_AUX_DONE_REPLY */
 	iiimp_aux_value_print,		/* IM_AUX_SETVALUES */
 	iiimp_aux_simple_print,		/* IM_AUX_SETVALUES_REPLY */
-
-	iiimp_invalid_message_print,	/* 98 */
-	iiimp_invalid_message_print,	/* 99 */
+	iiimp_aux_value_print,		/* IM_AUX_GETVALUES */
+	iiimp_aux_value_print,		/* IM_AUX_GETVALUES_REPLY */
 
 	iiimp_protocol_version_print,	/* IM_PROTOCOL_VERSION */
 
